"""Autogenerated API"""
from argus_api.session import _legacy_session as session
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def clear_persistent_component(
    host: str,
    component: str,
    container: str,
    waitFor: int = 0,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Clear persistent component (INTERNAL)

    :param str host: Host ID or name
    :param str component: Component name
    :param str container: Container name
    :param int waitFor: Wait for completion of this task \(milliseconds\)\, default is 0 \(no wait\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/host/{host}/persistence".format(host=host)
    headers = {}

    query_parameters = {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        query_parameters.update({"waitFor": waitFor})
    # Only send component if the argument was provided, don't send null values
    if component is not None:
        query_parameters.update({"component": component})
    # Only send container if the argument was provided, don't send null values
    if container is not None:
        query_parameters.update({"container": container})

    log.debug("DELETE %s (headers: %s)" % (route, str(headers)))

    response = session.delete(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def compact_persistence(
    host: str,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Initiate persistence compaction on specified host (INTERNAL)

    :param str host: Host ID or name
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/host/{host}/persistence/compact".format(
        host=host
    )
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def fetch_instance_performance_data(
    instance: int,
    waitFor: int = 0,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Fetch performance data for specified instance (INTERNAL)

    :param int instance: ID of the instance to fetch performance data from
    :param int waitFor: Wait for completion of this task \(milliseconds\)\, default is 0 \(no wait\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/instance/{instance}/performance".format(
        instance=instance
    )
    headers = {}

    query_parameters = {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        query_parameters.update({"waitFor": waitFor})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def get_task_response(
    taskID: str,
    waitFor: int = 0,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Fetch status for specific task (INTERNAL)

    :param str taskID: ID of the task to fetch
    :param int waitFor: Wait for completion of this task \(milliseconds\)\, default is 0 \(no wait\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/task/{taskID}".format(taskID=taskID)
    headers = {}

    query_parameters = {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        query_parameters.update({"waitFor": waitFor})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def host_install_all_components(
    host: str,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Install all instances on specified host (INTERNAL)

    :param str host: Host ID or name
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/host/{host}/installAll".format(host=host)
    headers = {}

    body = body or {}

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def host_restart_all_components(
    host: str,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Restart all instances on specified host (INTERNAL)

    :param str host: Host ID or name
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/host/{host}/restart".format(host=host)
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def host_uninstall_all(
    host: str,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Uninstall all instances on specified host (INTERNAL)

    :param str host: Host ID or name
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/host/{host}/uninstallAll".format(host=host)
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def install_instance(
    instance: int,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Install specified instance (INTERNAL)

    :param int instance: ID of the instance
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/instance/{instance}/install".format(
        instance=instance
    )
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def instance_expire_cache(
    instance: int,
    resources: str = None,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Expire cache on resources on specified instance (INTERNAL)

    :param int instance: ID of the instance
    :param list resources: Set of resources to expire
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/instance/{instance}/expirecache".format(
        instance=instance
    )
    headers = {}

    body = body or {}
    # Only send resources if the argument was provided, don't send null values
    if resources is not None:
        body.update({"resources": resources})
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def list_host_persistent_components(
    host: str,
    waitFor: int = 0,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""List persistent components (INTERNAL)

    :param str host: Host ID or name
    :param int waitFor: Wait for completion of this task \(milliseconds\)\, default is 0 \(no wait\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/host/{host}/persistence".format(host=host)
    headers = {}

    query_parameters = {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        query_parameters.update({"waitFor": waitFor})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def list_task_response(
    state: str = None,
    limit: int = 25,
    offset: int = 0,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""List task status (INTERNAL)

    :param list state: Filter response by status
    :param int limit: Limit response to this number of results \(0 means unlimited\)
    :param int offset: Skip the first results in the response \(default is 0\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/task".format()
    headers = {}

    query_parameters = {}
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    # Only send state if the argument was provided, don't send null values
    if state is not None:
        query_parameters.update({"state": state})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def request_updated_host_status(
    host: str,
    includeComponents: bool = None,
    instanceID: int = None,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Request updated status of a specified host or instance (INTERNAL)

    :param str host: ID or name of the host
    :param bool includeComponents: If true\, the components will be included
    :param int instanceID: The ID of the instance from which to request updated status
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/host/{host}/refreshstatus".format(host=host)
    headers = {}

    body = body or {}
    # Only send includeComponents if the argument was provided, don't send null values
    if includeComponents is not None:
        body.update({"includeComponents": includeComponents})
    # Only send instanceID if the argument was provided, don't send null values
    if instanceID is not None:
        body.update({"instanceID": instanceID})
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def restart_instance(
    instance: int,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Restart specified instance (INTERNAL)

    :param int instance: ID of the instance
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/instance/{instance}/restart".format(
        instance=instance
    )
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def start_instance(
    instance: int,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Start specified instance (INTERNAL)

    :param int instance: ID of the instance
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/instance/{instance}/start".format(
        instance=instance
    )
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def stop_instance(
    instance: int,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Stop specified instance (INTERNAL)

    :param int instance: ID of the instance
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/instance/{instance}/stop".format(
        instance=instance
    )
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("componentadmin", "v2", "invoke", "host", "installAll"),
    module=argus_cli_module,
)
def uninstall_instance(
    instance: int,
    waitFor: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Uninstall specified instance (INTERNAL)

    :param int instance: ID of the instance
    :param int waitFor: The max amount of millis to wait for a success or exception response
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/componentadmin/v2/invoke/instance/{instance}/uninstall".format(
        instance=instance
    )
    headers = {}

    body = body or {}
    # Only send waitFor if the argument was provided, don't send null values
    if waitFor is not None:
        body.update({"waitFor": waitFor})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response
