"""Autogenerated API"""
from argus_api.session import _legacy_session as session
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def add_instance_comment(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Add a comment to configuration instance (INTERNAL)

    :param int id: ID of instance to comment
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/comment".format(id=id)
    headers = {}

    body = body or {}

    log.debug(
        "POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or "")
    )

    response = session.post(
        route,
        data=body or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def create_instance(
    codeProfile: str = None,
    hostID: int = None,
    identifiers: dict = None,
    information: str = None,
    parentID: int = None,
    pinnedTemplateRevision: int = None,
    properties: dict = None,
    templateID: int = None,
    daytimeMonitored: bool = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Create new configuration instance (INTERNAL)

    :param str codeProfile: The code profile to use for this instance.
    :param int hostID: The ID of the host to run this instance on
    :param dict identifiers: Identifiers to set for this instance. Use this to differentiate between multiple instances of the same template on the same host.
    :param str information: Informational text. Will be default sanitized.
    :param int parentID: If set\, define a parent instance ID which this instance should run below.
    :param int pinnedTemplateRevision: If set\, pin this instance to the specified template revision. Revision 0 means use latest revision.
    :param dict properties: Configuration properties to set for this instance. Will override any conflicting properties set on host or template.
    :param int templateID: The ID of the template to use for this instance
    :param bool daytimeMonitored: If true\, mark this instance as a DAYTIME\_MONITORED host.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance".format()
    headers = {}

    body = body or {}
    # Only send codeProfile if the argument was provided, don't send null values
    if codeProfile is not None:
        body.update({"codeProfile": codeProfile})
    # Only send daytimeMonitored if the argument was provided, don't send null values
    if daytimeMonitored is not None:
        body.update({"daytimeMonitored": daytimeMonitored})
    # Only send hostID if the argument was provided, don't send null values
    if hostID is not None:
        body.update({"hostID": hostID})
    # Only send identifiers if the argument was provided, don't send null values
    if identifiers is not None:
        body.update({"identifiers": identifiers})
    # Only send information if the argument was provided, don't send null values
    if information is not None:
        body.update({"information": information})
    # Only send parentID if the argument was provided, don't send null values
    if parentID is not None:
        body.update({"parentID": parentID})
    # Only send pinnedTemplateRevision if the argument was provided, don't send null values
    if pinnedTemplateRevision is not None:
        body.update({"pinnedTemplateRevision": pinnedTemplateRevision})
    # Only send properties if the argument was provided, don't send null values
    if properties is not None:
        body.update({"properties": properties})
    # Only send templateID if the argument was provided, don't send null values
    if templateID is not None:
        body.update({"templateID": templateID})

    log.debug(
        "POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or "")
    )

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def delete_instance(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Delete configuration instance (INTERNAL)

    :param int id: ID of instance to delete
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}".format(id=id)
    headers = {}

    log.debug("DELETE %s (headers: %s)" % (route, str(headers)))

    response = session.delete(
        route,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def disable_instance_monitoring(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Disable monitoring of configuration instance (INTERNAL)

    :param int id: ID of instance to disable monitoring on
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/monitoring/disable".format(id=id)
    headers = {}

    body = body or {}

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        data=body or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def disable_instance_monitoring_request(
    instanceID: int,
    comment: str,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Disable monitoring of configuration instance (INTERNAL)

    :param int instanceID: ID of instance to disable monitoring on
    :param str comment: Reason for disabling monitoring
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{instanceID}/monitoring/disable".format(
        instanceID=instanceID
    )
    headers = {}

    body = body or {}
    # Only send comment if the argument was provided, don't send null values
    if comment is not None:
        body.update({"comment": comment})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def duplicate_instance(
    instanceID: int,
    hostID: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Duplicate a configuration instance (INTERNAL)

    :param int instanceID: ID of instance to duplicate
    :param int hostID: If set\, duplicate the instance onto the specified host
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{instanceID}/duplicate".format(
        instanceID=instanceID
    )
    headers = {}

    body = body or {}
    # Only send hostID if the argument was provided, don't send null values
    if hostID is not None:
        body.update({"hostID": hostID})

    log.debug(
        "POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or "")
    )

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def enable_instance_monitoring(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Enable monitoring of configuration instance (INTERNAL)

    :param int id: ID of instance to enable monitoring on
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/monitoring/enable".format(id=id)
    headers = {}

    body = body or {}

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def get_instance_configuration_data(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Fetch configuration data which is active for this configuration instance (INTERNAL)

    :param int id: ID or shortname of template
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/configuration".format(id=id)
    headers = {}

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def get_instance_properties(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Fetch configuration instance properties (INTERNAL)

    :param int id: ID of instance
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/properties".format(id=id)
    headers = {}

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def instance_revisions(
    id: int,
    offset: int = None,
    limit: int = 25,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""List configuration instance revisions (INTERNAL)

    :param int id: ID of instance to fetch revisions for
    :param int offset: Offset results
    :param int limit: Limit results
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/revisions".format(id=id)
    headers = {}

    query_parameters = {}
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def list_instances(
    offset: int = None,
    search: str = None,
    includeComments: bool = None,
    includeDeleted: bool = None,
    templateID: int = None,
    hostID: int = None,
    limit: int = 25,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""List configuration instances (INTERNAL)

    :param int offset: Offset results
    :param str search: Limit results to instances matching this searchstring
    :param bool includeComments: Include comments in output
    :param bool includeDeleted: Include deleted instances
    :param list templateID: Limit results to instances with these templates
    :param list hostID: Limit results to instances on these hosts
    :param int limit: Limit results
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance".format()
    headers = {}

    query_parameters = {}
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    # Only send search if the argument was provided, don't send null values
    if search is not None:
        query_parameters.update({"search": search})
    # Only send includeComments if the argument was provided, don't send null values
    if includeComments is not None:
        query_parameters.update({"includeComments": includeComments})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        query_parameters.update({"includeDeleted": includeDeleted})
    # Only send templateID if the argument was provided, don't send null values
    if templateID is not None:
        query_parameters.update({"templateID": templateID})
    # Only send hostID if the argument was provided, don't send null values
    if hostID is not None:
        query_parameters.update({"hostID": hostID})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def move_instance(
    instanceID: int,
    hostID: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Move a configuration instance to another host (INTERNAL)

    :param int instanceID: ID of instance to move
    :param int hostID: Host to move the instance to
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{instanceID}/move".format(
        instanceID=instanceID
    )
    headers = {}

    body = body or {}
    # Only send hostID if the argument was provided, don't send null values
    if hostID is not None:
        body.update({"hostID": hostID})

    log.debug(
        "POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or "")
    )

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def schedule_instance_monitoring(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Schedule downtime for configuration instance (INTERNAL)

    :param int id: ID of instance to schedule downtime for
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/monitoring/schedule".format(id=id)
    headers = {}

    body = body or {}

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        data=body or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def schedule_instance_monitoring_request(
    instanceID: int,
    comment: str,
    _from: int = None,
    to: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Schedule downtime for configuration instance (INTERNAL)

    :param int instanceID: ID of instance to schedule downtime for
    :param str comment: Reason for scheduling downtime
    :param int _from: Timestamp to schedule downtime from
    :param int to: Timestamp to schedule downtime to
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{instanceID}/monitoring/schedule".format(
        instanceID=instanceID
    )
    headers = {}

    body = body or {}
    # Only send comment if the argument was provided, don't send null values
    if comment is not None:
        body.update({"comment": comment})
    # Only send from if the argument was provided, don't send null values
    if _from is not None:
        body.update({"from": _from})
    # Only send to if the argument was provided, don't send null values
    if to is not None:
        body.update({"to": to})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def search_instances(
    limit: int = None,
    offset: int = None,
    includeDeleted: bool = None,
    subCriteria: dict = None,
    exclude: bool = None,
    required: bool = None,
    template: str = None,
    customer: str = None,
    instance: int = None,
    host: str = None,
    parentID: int = None,
    codeProfile: str = None,
    keywords: str = None,
    masterID: int = None,
    codeProfileOverride: bool = None,
    monitoringDisabled: bool = None,
    inDowntime: bool = None,
    daytimeMonitored: bool = None,
    hostID: int = None,
    templateID: int = None,
    instanceID: int = None,
    searchString: dict = None,
    sortBy: str = None,
    includeFlags: str = None,
    excludeFlags: str = None,
    includeComments: bool = None,
    includeProperties: bool = True,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Search configuration instances (INTERNAL)

    :param int limit: Max number of results.
    :param int offset: Skip the first \(offset\) objects. By default\, return result from first object.
    :param bool includeDeleted: Set to true to include deleted objects. By default\, exclude deleted objects.
    :param list subCriteria: Set additional criterias which are applied using a logical OR.
    :param bool exclude: Only relevant for subcriteria. If set to true\, objects matching this subcriteria object will be excluded.
    :param bool required: Only relevant for subcriteria. If set to true\, objects matching this subcriteria are required \(AND\-ed together with parent criteria\).
    :param list template: If set\, filter instances by template ID or shortname.
    :param list customer: If set\, filter instances by host customer ID or shortname.
    :param list instance: If set\, filter instances by ID.
    :param list host: If set\, filter instances by host ID or hostname.
    :param list parentID: If set\, filter instances by parent instance ID.
    :param list codeProfile: If set\, filter instances by active code profile.
    :param list keywords: If set\, filter instances matching these keywords.
    :param int masterID: If set\, only include instance revisions of given master. If not set\, revisions are excluded.
    :param bool codeProfileOverride: If true\, only return instances with overridden code profile. If false\, exclude. If not set\, do not filter.
    :param bool monitoringDisabled: If true\, only return instances with disabled monitoring\, or whose host has disabled monitoring. If false\, exclude. If not set\, do not filter.
    :param bool inDowntime: If true\, only return instances which are in scheduled downtime\, or whose host is in downtime. If false\, exclude. If not set\, do not filter.
    :param bool daytimeMonitored: If true\, only return instances which are daytime monitored \(either set on the instance\, or parent host\/instance\). If false\, exclude. If not set\, do not filter.
    :param list hostID:
    :param list templateID:
    :param list instanceID:
    :param dict searchString:
    :param list sortBy: List of properties to sort by \(prefix with \"\-\" to sort descending\).
    :param list includeFlags: Only include objects which have includeFlags set.
    :param list excludeFlags: Exclude objects which have excludeFlags set.
    :param bool includeComments: If true\, include comments in output.
    :param bool includeProperties: If true\, include properties in output.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/search".format()
    headers = {}

    body = body or {}
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send subCriteria if the argument was provided, don't send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send exclude if the argument was provided, don't send null values
    if exclude is not None:
        body.update({"exclude": exclude})
    # Only send required if the argument was provided, don't send null values
    if required is not None:
        body.update({"required": required})
    # Only send includeComments if the argument was provided, don't send null values
    if includeComments is not None:
        body.update({"includeComments": includeComments})
    # Only send includeProperties if the argument was provided, don't send null values
    if includeProperties is not None:
        body.update({"includeProperties": includeProperties})
    # Only send template if the argument was provided, don't send null values
    if template is not None:
        body.update({"template": template})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send instance if the argument was provided, don't send null values
    if instance is not None:
        body.update({"instance": instance})
    # Only send host if the argument was provided, don't send null values
    if host is not None:
        body.update({"host": host})
    # Only send parentID if the argument was provided, don't send null values
    if parentID is not None:
        body.update({"parentID": parentID})
    # Only send codeProfile if the argument was provided, don't send null values
    if codeProfile is not None:
        body.update({"codeProfile": codeProfile})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send masterID if the argument was provided, don't send null values
    if masterID is not None:
        body.update({"masterID": masterID})
    # Only send codeProfileOverride if the argument was provided, don't send null values
    if codeProfileOverride is not None:
        body.update({"codeProfileOverride": codeProfileOverride})
    # Only send monitoringDisabled if the argument was provided, don't send null values
    if monitoringDisabled is not None:
        body.update({"monitoringDisabled": monitoringDisabled})
    # Only send inDowntime if the argument was provided, don't send null values
    if inDowntime is not None:
        body.update({"inDowntime": inDowntime})
    # Only send daytimeMonitored if the argument was provided, don't send null values
    if daytimeMonitored is not None:
        body.update({"daytimeMonitored": daytimeMonitored})
    # Only send hostID if the argument was provided, don't send null values
    if hostID is not None:
        body.update({"hostID": hostID})
    # Only send templateID if the argument was provided, don't send null values
    if templateID is not None:
        body.update({"templateID": templateID})
    # Only send instanceID if the argument was provided, don't send null values
    if instanceID is not None:
        body.update({"instanceID": instanceID})
    # Only send searchString if the argument was provided, don't send null values
    if searchString is not None:
        body.update({"searchString": searchString})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send includeFlags if the argument was provided, don't send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send excludeFlags if the argument was provided, don't send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})

    log.debug(
        "POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or "")
    )

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def update_instance(
    id: int,
    codeProfile: str = None,
    configurationID: int = None,
    daytimeMonitored: bool = None,
    hostID: int = None,
    identifiers: dict = None,
    information: str = None,
    parentID: int = None,
    pinnedTemplateRevision: int = None,
    properties: dict = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Update configuration instance (INTERNAL)

    :param int id: ID of instance to update
    :param str codeProfile: Name of code profile to use. If null\, do not change. If blank\, reset to template profile.
    :param int configurationID: Deprecated option\, it is not allowed to change the template of an existing instance.
    :param bool daytimeMonitored: If set\, change the DAYTIME\_MONITORED flag on this instance
    :param int hostID: Deprecated option. Use the \/move endpoint to move instance to another host.
    :param dict identifiers: If set\, set these identifiers for instance. Identifier with null value will delete existing identifier.
    :param str information: If set\, change information field for instance.
    :param int parentID: Deprecated option\, it is not allowed to change the parent of an existing instance.
    :param int pinnedTemplateRevision: If set\, pin this instance to the specified template revision. Revision 0 means use latest revision.
    :param dict properties: If set\, set these properties for instance. Property with null value will delete existing property.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}".format(id=id)
    headers = {}

    body = body or {}
    # Only send codeProfile if the argument was provided, don't send null values
    if codeProfile is not None:
        body.update({"codeProfile": codeProfile})
    # Only send configurationID if the argument was provided, don't send null values
    if configurationID is not None:
        body.update({"configurationID": configurationID})
    # Only send daytimeMonitored if the argument was provided, don't send null values
    if daytimeMonitored is not None:
        body.update({"daytimeMonitored": daytimeMonitored})
    # Only send hostID if the argument was provided, don't send null values
    if hostID is not None:
        body.update({"hostID": hostID})
    # Only send identifiers if the argument was provided, don't send null values
    if identifiers is not None:
        body.update({"identifiers": identifiers})
    # Only send information if the argument was provided, don't send null values
    if information is not None:
        body.update({"information": information})
    # Only send parentID if the argument was provided, don't send null values
    if parentID is not None:
        body.update({"parentID": parentID})
    # Only send pinnedTemplateRevision if the argument was provided, don't send null values
    if pinnedTemplateRevision is not None:
        body.update({"pinnedTemplateRevision": pinnedTemplateRevision})
    # Only send properties if the argument was provided, don't send null values
    if properties is not None:
        body.update({"properties": properties})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def update_instance_properties(
    id: int,
    properties: dict,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Update configuration instance properties (INTERNAL)

    :param int id: ID of instance to update properties on
    :param dict properties: The properties to update to\, null value will clear the property
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}/properties".format(id=id)
    headers = {}

    body = body or {}
    # Only send properties if the argument was provided, don't send null values
    if properties is not None:
        body.update({"properties": properties})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(
    extending=("configurationadmin", "v1", "instance"), module=argus_cli_module
)
def view_instance(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Fetch configuration instance (INTERNAL)

    :param int id: ID of instance to fetch
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/configurationadmin/v1/instance/{id}".format(id=id)
    headers = {}

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response
