"""Autogenerated API"""
from argus_api.session import _legacy_session as session
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(extending=("currentuser", "v1", "functions"), module=argus_cli_module)
def get_functions(
    onlyRoles: bool = None,
    keywords: str = None,
    offset: int = 0,
    limit: int = 25,
    requirePermission: bool = True,
    maxSecurityLevel: str = "current",
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""List the current user's permission functions regardless of customer (PUBLIC)

    :param bool onlyRoles: Only return functions marked as roles
    :param list keywords: Filter functions by keywords on name and description
    :param int offset: Skip a number of functions
    :param int limit: Maximum number of returned functions
    :param bool requirePermission: Only return functions which current user has access to
    :param str maxSecurityLevel: Only return functions which are available with this security level
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises ForbiddenPermissionException: on 403
    :raises InvalidRequestException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/currentuser/v1/functions".format()
    headers = {}

    query_parameters = {}
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send requirePermission if the argument was provided, don't send null values
    if requirePermission is not None:
        query_parameters.update({"requirePermission": requirePermission})
    # Only send maxSecurityLevel if the argument was provided, don't send null values
    if maxSecurityLevel is not None:
        query_parameters.update({"maxSecurityLevel": maxSecurityLevel})
    # Only send onlyRoles if the argument was provided, don't send null values
    if onlyRoles is not None:
        query_parameters.update({"onlyRoles": onlyRoles})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        query_parameters.update({"keywords": keywords})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response
