"""Autogenerated API"""
from argus_api.session import _legacy_session as session
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(extending=("metrics", "v1", "descriptor"), module=argus_cli_module)
def add_metric_descriptor(
    adminFunction: str,
    description: str,
    keys: dict,
    readFunction: str,
    shortName: str,
    values: dict,
    writeFunction: str,
    deduplicationFunction: str = "none",
    customer: str = None,
    domain: str = None,
    localizedDescriptions: dict = None,
    localizedNames: dict = None,
    name: str = None,
    timeFrameSize: int = None,
    category: str = "other",
    globalDescriptor: bool = None,
    keepAuditData: bool = True,
    keepBackup: bool = True,
    timeFrameSizeUnit: str = "milliseconds",
    useSeparateIndex: bool = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Add new metric descriptor. (INTERNAL)

    :param str adminFunction: Function required to update this metric descriptor.
    :param str description: Description of the metric descriptor.
    :param list keys: Descriptors for metric keys.
    :param str readFunction: Function required to fetch metric descriptor and associated records.
    :param str shortName: Short name of the metric descriptor. Must be unique per domain.
    :param list values: Descriptors for metric values.
    :param str writeFunction: Function required to submit metric record.
    :param str deduplicationFunction: Function used while submitting a metric record\, to calculate value if duplicates within same time frame appear.
    :param str customer: ID or shortname of customer bound to this metric descriptor. If not specified\, descriptor is domain global
    :param str domain: ID or shortname of domain bound to this metric descriptor.
    :param dict localizedDescriptions: Metric descriptor descriptions mapped by language. Supported languages are\: english\, norwegian.
    :param dict localizedNames: Metric descriptor names mapped by language. Supported languages are\: english\, norwegian.
    :param str name: Name of the metric descriptor. Does not need to be unique.
    :param int timeFrameSize: Time period used for setting up time frames. For each time frame\, deduplicationFunction property is used to determine value\(s\) of metric record.
    :param str category: Category of metric descriptor
    :param bool globalDescriptor: Whether this descriptor is intended for records that are not customer specific.
    :param bool keepAuditData: Whether to maintain a record of data useful for auditing.
    :param bool keepBackup: Whether to keep a separate record of the metric as a backup outside ElasticSearch.
    :param str timeFrameSizeUnit: Unit of the timeFrameSize value\, this is used as the default resolution unit during aggregation.
    :param bool useSeparateIndex: Whether the separate index for metric records is used or not.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/metrics/v1/descriptor".format()
    headers = {}

    body = body or {}
    # Only send adminFunction if the argument was provided, don't send null values
    if adminFunction is not None:
        body.update({"adminFunction": adminFunction})
    # Only send category if the argument was provided, don't send null values
    if category is not None:
        body.update({"category": category})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send deduplicationFunction if the argument was provided, don't send null values
    if deduplicationFunction is not None:
        body.update({"deduplicationFunction": deduplicationFunction})
    # Only send description if the argument was provided, don't send null values
    if description is not None:
        body.update({"description": description})
    # Only send domain if the argument was provided, don't send null values
    if domain is not None:
        body.update({"domain": domain})
    # Only send globalDescriptor if the argument was provided, don't send null values
    if globalDescriptor is not None:
        body.update({"globalDescriptor": globalDescriptor})
    # Only send keepAuditData if the argument was provided, don't send null values
    if keepAuditData is not None:
        body.update({"keepAuditData": keepAuditData})
    # Only send keepBackup if the argument was provided, don't send null values
    if keepBackup is not None:
        body.update({"keepBackup": keepBackup})
    # Only send keys if the argument was provided, don't send null values
    if keys is not None:
        body.update({"keys": keys})
    # Only send localizedDescriptions if the argument was provided, don't send null values
    if localizedDescriptions is not None:
        body.update({"localizedDescriptions": localizedDescriptions})
    # Only send localizedNames if the argument was provided, don't send null values
    if localizedNames is not None:
        body.update({"localizedNames": localizedNames})
    # Only send name if the argument was provided, don't send null values
    if name is not None:
        body.update({"name": name})
    # Only send readFunction if the argument was provided, don't send null values
    if readFunction is not None:
        body.update({"readFunction": readFunction})
    # Only send shortName if the argument was provided, don't send null values
    if shortName is not None:
        body.update({"shortName": shortName})
    # Only send timeFrameSize if the argument was provided, don't send null values
    if timeFrameSize is not None:
        body.update({"timeFrameSize": timeFrameSize})
    # Only send timeFrameSizeUnit if the argument was provided, don't send null values
    if timeFrameSizeUnit is not None:
        body.update({"timeFrameSizeUnit": timeFrameSizeUnit})
    # Only send useSeparateIndex if the argument was provided, don't send null values
    if useSeparateIndex is not None:
        body.update({"useSeparateIndex": useSeparateIndex})
    # Only send values if the argument was provided, don't send null values
    if values is not None:
        body.update({"values": values})
    # Only send writeFunction if the argument was provided, don't send null values
    if writeFunction is not None:
        body.update({"writeFunction": writeFunction})

    log.debug(
        "POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or "")
    )

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(extending=("metrics", "v1", "descriptor"), module=argus_cli_module)
def delete_metric_descriptor(
    descriptor: str,
    descriptorDomain: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Delete metric descriptor (INTERNAL)

    :param str descriptor: Shortname or ID of descriptor
    :param str descriptorDomain: Domain which is used if short name is specified for \'descriptor\' parameter. Defaults to current user domain.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/metrics/v1/descriptor/{descriptor}".format(descriptor=descriptor)
    headers = {}

    query_parameters = {}
    # Only send descriptorDomain if the argument was provided, don't send null values
    if descriptorDomain is not None:
        query_parameters.update({"descriptorDomain": descriptorDomain})

    log.debug("DELETE %s (headers: %s)" % (route, str(headers)))

    response = session.delete(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(extending=("metrics", "v1", "descriptor"), module=argus_cli_module)
def get_metric_descriptor(
    descriptor: str,
    descriptorDomain: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Fetch metric descriptor. (PUBLIC)

    :param str descriptor: ID or short name of metric descriptor
    :param str descriptorDomain: Domain which is used if short name is specified for \'descriptor\' parameter. Defaults to current user domain.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/metrics/v1/descriptor/{descriptor}".format(descriptor=descriptor)
    headers = {}

    query_parameters = {}
    # Only send descriptorDomain if the argument was provided, don't send null values
    if descriptorDomain is not None:
        query_parameters.update({"descriptorDomain": descriptorDomain})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(extending=("metrics", "v1", "descriptor"), module=argus_cli_module)
def get_metric_descriptors(
    descriptorDomain: str = None,
    customer: str = None,
    keywords: str = None,
    offset: int = 0,
    limit: int = 25,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
) -> dict:
    r"""Fetch metric descriptor based on criteria (PUBLIC)

    :param list descriptorDomain: Restrict search to entries bound to specified domains \(domain ID or shortname\)
    :param list customer: Restrict search to entries bound to specified customers \(customer ID or shortname\)
    :param list keywords: Search by keywords
    :param int offset:
    :param int limit: Maximum number of returned results
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/metrics/v1/descriptor".format()
    headers = {}

    query_parameters = {}
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send descriptorDomain if the argument was provided, don't send null values
    if descriptorDomain is not None:
        query_parameters.update({"descriptorDomain": descriptorDomain})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        query_parameters.update({"customer": customer})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        query_parameters.update({"keywords": keywords})

    log.debug("GET %s (headers: %s)" % (route, str(headers)))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(extending=("metrics", "v1", "descriptor"), module=argus_cli_module)
def search_metric_descriptors(
    customer: str = None,
    domain: str = None,
    excludeFlags: str = None,
    includeFlags: str = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = None,
    keywords: str = None,
    offset: int = None,
    includeDeleted: bool = None,
    limit: int = 25,
    subCriteria: dict = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Fetch metric descriptor based on criteria (PUBLIC)

    :param list customer: Restrict search to entries bound to specified customers \(customer ID or shortname\).
    :param list domain: Restrict search to entries bound to specified domains \(domain ID or shortname\).
    :param list excludeFlags: Exclude descriptors which have excludeFlags set.
    :param list includeFlags: Include descriptors which have includeFlags set.
    :param list keywordFieldStrategy: Defines which fields will be searched by keywords
    :param str keywordMatchStrategy: Defines the MatchStrategy for keywords.
    :param list keywords: Restrict search to entries containing keywords in description.
    :param int offset: Set this value to skip the first \(offset\) objects. By default\, return result from first object.
    :param bool includeDeleted: Include descriptors which have been deleted.
    :param int limit: Set this value to set max number of results.
    :param list subCriteria: Set additional criterias to be applied as specified in the subcriteria object\(s\) \(AND\, AND NOT\, OR\).
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/metrics/v1/descriptor/search".format()
    headers = {}

    body = body or {}
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send domain if the argument was provided, don't send null values
    if domain is not None:
        body.update({"domain": domain})
    # Only send excludeFlags if the argument was provided, don't send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send includeFlags if the argument was provided, don't send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send keywordFieldStrategy if the argument was provided, don't send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send keywordMatchStrategy if the argument was provided, don't send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send subCriteria if the argument was provided, don't send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})

    log.debug(
        "POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or "")
    )

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


@register_command(extending=("metrics", "v1", "descriptor"), module=argus_cli_module)
def update_metric_descriptor(
    descriptor: str,
    addKeys: dict = None,
    addValues: dict = None,
    adminFunction: str = None,
    category: str = None,
    deduplicationFunction: str = None,
    description: str = None,
    keepAuditData: bool = None,
    keepBackup: bool = None,
    localizedDescriptions: dict = None,
    localizedNames: dict = None,
    name: str = None,
    readFunction: str = None,
    removeKeys: str = None,
    removeValues: str = None,
    shortName: str = None,
    timeFrameSize: int = None,
    updateKeys: dict = None,
    updateValues: dict = None,
    useSeparateIndex: bool = None,
    writeFunction: str = None,
    descriptorDomain: str = None,
    timeFrameSizeUnit: str = "milliseconds",
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    r"""Update existing metric descriptor. (INTERNAL)

    :param str descriptor: Shortname or ID of descriptor
    :param list addKeys: Descriptors for metric keys to add. Previously accepted metrics will still exist and considered valid\, but new submissions must match the changed descriptor.
    :param list addValues: Descriptors for metric values to add. Previously accepted metrics will still exist and considered valid\, but new submissions must match the changed descriptor.
    :param str adminFunction: Function required to update this metric descriptor.
    :param str category: Category of metric descriptor.
    :param str deduplicationFunction: Function used while submitting a metric record\, to calculate value if duplicates within same time frame appear.
    :param str description: Description of the metric descriptor.
    :param bool keepAuditData: Whether to maintain a record of data useful for auditing.
    :param bool keepBackup: Whether to keep a separate record of the metric as a backup outside ElasticSearch.
    :param dict localizedDescriptions: Metric descriptor descriptions mapped by language. Supported languages are\: english\, norwegian.
    :param dict localizedNames: Metric descriptor names mapped by language. Supported languages are\: english\, norwegian.
    :param str name: Name of the metric descriptor. Does not need to be unique.
    :param str readFunction: Function required to fetch metric descriptor and associated records.
    :param list removeKeys: Descriptors for metric keys to remove. Previously accepted metrics will still exist and considered valid\, but new submissions must match the changed descriptor.
    :param list removeValues: Descriptors for metric values to remove. Previously accepted metrics will still exist and considered valid\, but new submissions must match the changed descriptor.
    :param str shortName: Short name of the metric descriptor. Must be unique per domain.
    :param int timeFrameSize: Time period used for setting up time frames. For each time frame\, deduplicationFunction property is used to determine value\(s\) of metric record.
    :param list updateKeys: Descriptors for metric keys to update. Only the translator and language can be updated while name is used to identify the key to modify.
    :param list updateValues: Descriptors for metric values to update. Only the default aggregation and value by weight can be updated while the name is used to identify the value to modify.
    :param bool useSeparateIndex: Whether the separate index for metric records is used or not. If value of this property is \'true\'\, changing is not permitted.
    :param str writeFunction: Function required to submit metric record.
    :param str descriptorDomain: Domain which is used if short name is specified for \'descriptor\' parameter. Defaults to current user domain.
    :param str timeFrameSizeUnit: Unit of the timeFrameSize value\, this is used as the default resolution unit during aggregation.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/metrics/v1/descriptor/{descriptor}".format(descriptor=descriptor)
    headers = {}

    body = body or {}
    # Only send addKeys if the argument was provided, don't send null values
    if addKeys is not None:
        body.update({"addKeys": addKeys})
    # Only send addValues if the argument was provided, don't send null values
    if addValues is not None:
        body.update({"addValues": addValues})
    # Only send adminFunction if the argument was provided, don't send null values
    if adminFunction is not None:
        body.update({"adminFunction": adminFunction})
    # Only send category if the argument was provided, don't send null values
    if category is not None:
        body.update({"category": category})
    # Only send deduplicationFunction if the argument was provided, don't send null values
    if deduplicationFunction is not None:
        body.update({"deduplicationFunction": deduplicationFunction})
    # Only send description if the argument was provided, don't send null values
    if description is not None:
        body.update({"description": description})
    # Only send keepAuditData if the argument was provided, don't send null values
    if keepAuditData is not None:
        body.update({"keepAuditData": keepAuditData})
    # Only send keepBackup if the argument was provided, don't send null values
    if keepBackup is not None:
        body.update({"keepBackup": keepBackup})
    # Only send localizedDescriptions if the argument was provided, don't send null values
    if localizedDescriptions is not None:
        body.update({"localizedDescriptions": localizedDescriptions})
    # Only send localizedNames if the argument was provided, don't send null values
    if localizedNames is not None:
        body.update({"localizedNames": localizedNames})
    # Only send name if the argument was provided, don't send null values
    if name is not None:
        body.update({"name": name})
    # Only send readFunction if the argument was provided, don't send null values
    if readFunction is not None:
        body.update({"readFunction": readFunction})
    # Only send removeKeys if the argument was provided, don't send null values
    if removeKeys is not None:
        body.update({"removeKeys": removeKeys})
    # Only send removeValues if the argument was provided, don't send null values
    if removeValues is not None:
        body.update({"removeValues": removeValues})
    # Only send shortName if the argument was provided, don't send null values
    if shortName is not None:
        body.update({"shortName": shortName})
    # Only send timeFrameSize if the argument was provided, don't send null values
    if timeFrameSize is not None:
        body.update({"timeFrameSize": timeFrameSize})
    # Only send timeFrameSizeUnit if the argument was provided, don't send null values
    if timeFrameSizeUnit is not None:
        body.update({"timeFrameSizeUnit": timeFrameSizeUnit})
    # Only send updateKeys if the argument was provided, don't send null values
    if updateKeys is not None:
        body.update({"updateKeys": updateKeys})
    # Only send updateValues if the argument was provided, don't send null values
    if updateValues is not None:
        body.update({"updateValues": updateValues})
    # Only send useSeparateIndex if the argument was provided, don't send null values
    if useSeparateIndex is not None:
        body.update({"useSeparateIndex": useSeparateIndex})
    # Only send writeFunction if the argument was provided, don't send null values
    if writeFunction is not None:
        body.update({"writeFunction": writeFunction})

    query_parameters = {}
    # Only send descriptorDomain if the argument was provided, don't send null values
    if descriptorDomain is not None:
        query_parameters.update({"descriptorDomain": descriptorDomain})

    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response
