"""Autogenerated API"""
from argus_api.session import get_session as _get_session
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from argus_api.session import ArgusAPISession
from requests import Response


def add_vulnerability(
    asset: str,
    vulnerability: str,
    components: dict = None,
    customer: str = None,
    cvss: float = None,
    dataSource: str = None,
    observationDescription: str = None,
    observationReferences: str = None,
    rawOutput: str = None,
    severity: str = None,
    ttl: int = None,
    forceIndex: bool = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Creates a new vulnerability instance (PUBLIC)

    :param str asset: ID or short name of asset. If short name is specified\, then the asset is looked up from the customer.
    :param str vulnerability: Either vulnerability definition ID \(UUID\) or vulnerability identifier \(e.g. CVE\-2015\-3429\). First the value will be used to lookup vulnerability definition \(UUID will only be supported for vulnerability definition lookup\). If vulnerability definition exists\, then associate this vulnerability instance to it\, and require read permission on vulnerability source of the definition. Otherwise create vulnerability instance with vulnerability identifier.
    :param list components: Specify components to be added with the vulnerability.
    :param str customer: ID or short name of customer\, if not specified then current user\'s customer will be assigned.
    :param float cvss: CVSS score of vulnerability \(range from 0 to 10\). Setting this field will overwrite vulnerability definition settings and the value will not be updated when CVSS or Severity in the definition changes.
    :param str dataSource: ID or short name of the Data Source
    :param str observationDescription: Description specific to this vulnerability. May contain customer specific data. Will be sanitized as HTML.
    :param list observationReferences: References specific to this vulnerability e.g. internal URLs. May contain customer specific data.
    :param str rawOutput: Raw output from vulnerability scan. Will be sanitized as html.
    :param str severity: Severity level of vulnerability. If not provided\, then severity will be calculated based on CVSS value. Setting this field will overwrite vulnerability definition settings and the value will not be updated when CVSS or Severity in the definition changes.
    :param int ttl: Time to live duration of this vulnerability since last seen\, 0 means won\'t expire.
    :param bool forceIndex: Whether to force an Elasticsearch index refresh making the new vulnerability immediately searchable.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send asset if the argument was provided, don't send null values
    if asset is not None:
        body.update({"asset": asset})
    # Only send components if the argument was provided, don't send null values
    if components is not None:
        body.update({"components": components})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send cvss if the argument was provided, don't send null values
    if cvss is not None:
        body.update({"cvss": cvss})
    # Only send dataSource if the argument was provided, don't send null values
    if dataSource is not None:
        body.update({"dataSource": dataSource})
    # Only send observationDescription if the argument was provided, don't send null values
    if observationDescription is not None:
        body.update({"observationDescription": observationDescription})
    # Only send observationReferences if the argument was provided, don't send null values
    if observationReferences is not None:
        body.update({"observationReferences": observationReferences})
    # Only send rawOutput if the argument was provided, don't send null values
    if rawOutput is not None:
        body.update({"rawOutput": rawOutput})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send ttl if the argument was provided, don't send null values
    if ttl is not None:
        body.update({"ttl": ttl})
    # Only send vulnerability if the argument was provided, don't send null values
    if vulnerability is not None:
        body.update({"vulnerability": vulnerability})

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def add_vulnerability_definition(
    vulnerabilityID: str,
    vulnerabilitySource: str,
    conclusion: str = None,
    description: str = None,
    exploitAvailable: bool = None,
    name: str = None,
    references: str = None,
    severity: str = None,
    solution: str = None,
    forceIndex: bool = None,
    cvss: float = None,
    exploitAvailableTimestamp: int = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Creates a new vulnerability definition also require to have read & write permission of vulnerability source that the definition is belonging to. (INTERNAL)

    :param str vulnerabilityID: Identifier of vulnerability \(e.g. plug\-in ID from vulnerability scanner\). Needs to be unique.
    :param str vulnerabilitySource: Define vulnerability source by source ID or short name.
    :param str conclusion: Short summary of vulnerability. Will be sanitized as html.
    :param str description: Description of vulnerability. Will be sanitized as html.
    :param bool exploitAvailable: Set if an exploit is available for the vulnerability.
    :param str name: Name of vulnerability. Will be sanitized as text.
    :param list references: References to vulnerability \(e.g. CVE number\). Will be sanitized as html.
    :param str severity: Severity level of vulnerability definition. If not provided\, then severity will be calculated based on CVSS value.
    :param str solution: How to fix vulnerability. Will be sanitized as html.
    :param bool forceIndex: Whether to force an Elasticsearch index refresh making the new vulnerability definition immediately searchable.
    :param float cvss: CVSS score of vulnerability definition \(range from 0 to 10\).
    :param int exploitAvailableTimestamp: Set the timestamp when the exploit became available. Setting this value also expects setting exploitAvailable to \'true\' with this request. Setting\/leaving it to 0 will use current timestamp.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/definition".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send conclusion if the argument was provided, don't send null values
    if conclusion is not None:
        body.update({"conclusion": conclusion})
    # Only send cvss if the argument was provided, don't send null values
    if cvss is not None:
        body.update({"cvss": cvss})
    # Only send description if the argument was provided, don't send null values
    if description is not None:
        body.update({"description": description})
    # Only send exploitAvailable if the argument was provided, don't send null values
    if exploitAvailable is not None:
        body.update({"exploitAvailable": exploitAvailable})
    # Only send exploitAvailableTimestamp if the argument was provided, don't send null values
    if exploitAvailableTimestamp is not None:
        body.update({"exploitAvailableTimestamp": exploitAvailableTimestamp})
    # Only send name if the argument was provided, don't send null values
    if name is not None:
        body.update({"name": name})
    # Only send references if the argument was provided, don't send null values
    if references is not None:
        body.update({"references": references})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send solution if the argument was provided, don't send null values
    if solution is not None:
        body.update({"solution": solution})
    # Only send vulnerabilityID if the argument was provided, don't send null values
    if vulnerabilityID is not None:
        body.update({"vulnerabilityID": vulnerabilityID})
    # Only send vulnerabilitySource if the argument was provided, don't send null values
    if vulnerabilitySource is not None:
        body.update({"vulnerabilitySource": vulnerabilitySource})

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def add_vulnerability_source(
    name: str,
    readPermission: str,
    shortName: str,
    writePermission: str,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Creates a new VulnerabilitySource (INTERNAL)

    :param str name: Name of source
    :param str readPermission: Permission required to allow read access of source \& vulnerability definition belong to this source
    :param str shortName: Vulnerability source short name\, required and unique
    :param str writePermission: Permission required to allow write access \(create\/update\/delete\) source \& of vulnerability definition belong to this source
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/source".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send name if the argument was provided, don't send null values
    if name is not None:
        body.update({"name": name})
    # Only send readPermission if the argument was provided, don't send null values
    if readPermission is not None:
        body.update({"readPermission": readPermission})
    # Only send shortName if the argument was provided, don't send null values
    if shortName is not None:
        body.update({"shortName": shortName})
    # Only send writePermission if the argument was provided, don't send null values
    if writePermission is not None:
        body.update({"writePermission": writePermission})

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def bulk_recast_async_observations(
    severity: str,
    cvss: float = None,
    observations: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Bulk recasts a set of observations. (PUBLIC)

    :param str severity: New Severity.
    :param float cvss: New CVSS score \(range from 0 to 10\).
    :param list observations: The observations\' unique ids
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/bulk/recast/async".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send cvss if the argument was provided, don't send null values
    if cvss is not None:
        body.update({"cvss": cvss})
    # Only send observations if the argument was provided, don't send null values
    if observations is not None:
        body.update({"observations": observations})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def bulk_recast_observations(
    severity: str,
    cvss: float = None,
    observations: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Bulk recasts a set of observations. (PUBLIC)

    :param str severity: New Severity.
    :param float cvss: New CVSS score \(range from 0 to 10\).
    :param list observations: The observations\' unique ids
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/bulk/recast".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send cvss if the argument was provided, don't send null values
    if cvss is not None:
        body.update({"cvss": cvss})
    # Only send observations if the argument was provided, don't send null values
    if observations is not None:
        body.update({"observations": observations})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def bulk_resolve_vulnerabilities(
    resolution: str,
    comment: str = None,
    observations: str = None,
    ignoreResolved: bool = None,
    resolutionExpiryTimestamp: str = "0",
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Bulk resolves a set of vulnerabilities. (PUBLIC)

    :param str resolution: Specify resolution reason.
    :param str comment: Comment on why the vulnerability is resolved. If present will apply to all vulnerabilities in the request.
    :param list observations: The observations\' unique ids
    :param bool ignoreResolved: Whether to reject the entire request if any of the observations are already resolved.
    :param str resolutionExpiryTimestamp: When will the current resolution expire \(0 means no expiry\). Allows unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time values. See https\:\/\/docs.mnemonic.no\/x\/AQDXAQ  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/bulk/resolve".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send comment if the argument was provided, don't send null values
    if comment is not None:
        body.update({"comment": comment})
    # Only send ignoreResolved if the argument was provided, don't send null values
    if ignoreResolved is not None:
        body.update({"ignoreResolved": ignoreResolved})
    # Only send observations if the argument was provided, don't send null values
    if observations is not None:
        body.update({"observations": observations})
    # Only send resolution if the argument was provided, don't send null values
    if resolution is not None:
        body.update({"resolution": resolution})
    # Only send resolutionExpiryTimestamp if the argument was provided, don't send null values
    if resolutionExpiryTimestamp is not None:
        body.update({"resolutionExpiryTimestamp": resolutionExpiryTimestamp})

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def bulk_task_resolve_observations(
    resolution: str,
    comment: str = None,
    criteria: dict = None,
    observations: str = None,
    resolutionExpiryTimestamp: str = "0",
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Bulk resolves a set of observations or observations matching the requested criteria. (PUBLIC)

    :param str resolution: Specify resolution reason.
    :param str comment: Comment on why the vulnerability is resolved. If present will apply to all vulnerabilities in the request.
    :param dict criteria:
    :param list observations: The observations\' unique ids
    :param str resolutionExpiryTimestamp: When will the current resolution expire \(0 means no expiry\). Allows unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time values. See https\:\/\/docs.mnemonic.no\/x\/AQDXAQ  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/bulk/resolve/async".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send comment if the argument was provided, don't send null values
    if comment is not None:
        body.update({"comment": comment})
    # Only send criteria if the argument was provided, don't send null values
    if criteria is not None:
        body.update({"criteria": criteria})
    # Only send observations if the argument was provided, don't send null values
    if observations is not None:
        body.update({"observations": observations})
    # Only send resolution if the argument was provided, don't send null values
    if resolution is not None:
        body.update({"resolution": resolution})
    # Only send resolutionExpiryTimestamp if the argument was provided, don't send null values
    if resolutionExpiryTimestamp is not None:
        body.update({"resolutionExpiryTimestamp": resolutionExpiryTimestamp})

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def delete_vulnerability(
    id: str,
    forceIndex: bool = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Delete vulnerability (PUBLIC)

    :param str id: vulnerability ID
    :param bool forceIndex: Whether to force an Elasticsearch index refresh such that updates to the Vulnerability are immediately searchable
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/{id}".format(id=id)

    session = api_session or _get_session()
    headers = {}

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})

    response = session.delete(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def delete_vulnerability_definition(
    idOrVulnerabilityID: str,
    forceIndex: bool = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Delete Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to. (INTERNAL)

    :param str idOrVulnerabilityID: Vulnerability definition ID or identifier of vulnerability
    :param bool forceIndex: Whether to force an Elasticsearch index refresh such that updates to the VulnerabilityDefinition are immediately searchable.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/definition/{idOrVulnerabilityID}".format(
        idOrVulnerabilityID=idOrVulnerabilityID
    )

    session = api_session or _get_session()
    headers = {}

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})

    response = session.delete(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def delete_vulnerability_source(
    source: str,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Marks a VulnerabilitySource as deleted (INTERNAL)

    :param str source: VulnerabilitySource ID or short name
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/source/{source}".format(source=source)

    session = api_session or _get_session()
    headers = {}

    response = session.delete(
        route,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def get_observations_by_vulnerability_compact_view(
    asset: str = None,
    assetComponent: dict = None,
    assetGroup: str = None,
    component: dict = None,
    customer: str = None,
    dataSource: str = None,
    excludeFlags: str = None,
    includeFlags: str = None,
    keywords: str = None,
    maximumCvss: float = None,
    minimumCvss: float = None,
    observationReferences: str = None,
    references: str = None,
    resolution: str = None,
    severity: str = None,
    subCriteria: dict = None,
    user: str = None,
    vulnerabilityDefinition: str = None,
    vulnerabilityID: str = None,
    endTimestamp: str = "0",
    includeDeleted: bool = None,
    includeRawOutput: bool = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = "any",
    limit: int = 25,
    offset: int = None,
    sortBy: str = None,
    startTimestamp: str = "0",
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = "any",
    userFieldStrategy: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Search for observations and group them by their vulnerability along with each vulnerability's observation count. This differs from the stats endpoint in that it supports sorting and pagination of the resulting collapsed vulnerability results. (TEST)

    :param list asset: Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria\, if any. If none are provided\, we attempt to use the customers the current user has access to.
    :param list assetComponent: Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.\<br \/\>If you want different behaviour try using subcriteria
    :param list assetGroup: Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria\, if any. If none are provided\, we attempt to use the customers the current user has access to.
    :param list component: Search for Vulnerabilities with these components
    :param list customer: Retrieve vulnerabilities by Customer id or name
    :param list dataSource: Search for vulnerabilities detected and reported by these data sources\, identified by id or shortName
    :param list excludeFlags: Explicitly filter out Vulnerabilities that contain these flags
    :param list includeFlags: Explicitly retrieve Vulnerabilities that contain these flags
    :param list keywords: Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy
    :param float maximumCvss: Retrieve Vulnerabilities with at most this CVSS score
    :param float minimumCvss: Retrieve Vulnerabilities with at least this CVSS score
    :param list observationReferences: Search for vulnerabilities by exact matches in their references
    :param list references: Search for vulnerabilities by exact matches in their definition references
    :param list resolution: Retrieve vulnerabilities with these resolutions
    :param list severity: Retrieve vulnerabilities with these severities
    :param list subCriteria: Set additional criteria that are applied with logical OR by default
    :param list user: Retrieve vulnerabilities by these users\, identified by their id or username
    :param list vulnerabilityDefinition: Retrieve vulnerabilities by their vulnerability definition id or name
    :param list vulnerabilityID: Retrieve vulnerabilities by their external ID e.g. CVE\-XXXX
    :param str endTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(end timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param bool includeDeleted: Whether to include deleted Vulnerabilities
    :param bool includeRawOutput: Whether to include raw output in the result objects
    :param list keywordFieldStrategy: Defines which fields will be queried for keywords \(defaults to all observation fields\)
    :param str keywordMatchStrategy: Defines how strict different keywords should be matched \(default match all keywords\)
    :param int limit: Limit the results to the specified amount.
    :param int offset: Skip specified amount of results.
    :param list sortBy: Field\(s\) by which to sort the results
    :param str startTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(start timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param list timeFieldStrategy: Defines which timestamps fields will be filtered with start\/endTimestamp
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\)
    :param list userFieldStrategy: Defines which user fields will be filtered with user ids or shortnames
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/compact/definition".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send asset if the argument was provided, don't send null values
    if asset is not None:
        body.update({"asset": asset})
    # Only send assetComponent if the argument was provided, don't send null values
    if assetComponent is not None:
        body.update({"assetComponent": assetComponent})
    # Only send assetGroup if the argument was provided, don't send null values
    if assetGroup is not None:
        body.update({"assetGroup": assetGroup})
    # Only send component if the argument was provided, don't send null values
    if component is not None:
        body.update({"component": component})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send dataSource if the argument was provided, don't send null values
    if dataSource is not None:
        body.update({"dataSource": dataSource})
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send excludeFlags if the argument was provided, don't send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send includeFlags if the argument was provided, don't send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send includeRawOutput if the argument was provided, don't send null values
    if includeRawOutput is not None:
        body.update({"includeRawOutput": includeRawOutput})
    # Only send keywordFieldStrategy if the argument was provided, don't send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send keywordMatchStrategy if the argument was provided, don't send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send maximumCvss if the argument was provided, don't send null values
    if maximumCvss is not None:
        body.update({"maximumCvss": maximumCvss})
    # Only send minimumCvss if the argument was provided, don't send null values
    if minimumCvss is not None:
        body.update({"minimumCvss": minimumCvss})
    # Only send observationReferences if the argument was provided, don't send null values
    if observationReferences is not None:
        body.update({"observationReferences": observationReferences})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send references if the argument was provided, don't send null values
    if references is not None:
        body.update({"references": references})
    # Only send resolution if the argument was provided, don't send null values
    if resolution is not None:
        body.update({"resolution": resolution})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send subCriteria if the argument was provided, don't send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send timeFieldStrategy if the argument was provided, don't send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send timeMatchStrategy if the argument was provided, don't send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})
    # Only send user if the argument was provided, don't send null values
    if user is not None:
        body.update({"user": user})
    # Only send userFieldStrategy if the argument was provided, don't send null values
    if userFieldStrategy is not None:
        body.update({"userFieldStrategy": userFieldStrategy})
    # Only send vulnerabilityDefinition if the argument was provided, don't send null values
    if vulnerabilityDefinition is not None:
        body.update({"vulnerabilityDefinition": vulnerabilityDefinition})
    # Only send vulnerabilityID if the argument was provided, don't send null values
    if vulnerabilityID is not None:
        body.update({"vulnerabilityID": vulnerabilityID})

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def get_vulnerability_by_id(
    id: str,
    includeRawOutput: bool = False,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Get Vulnerability by ID. (PUBLIC)

    :param str id: Vulnerability ID \(UUID\)
    :param bool includeRawOutput: Whether include raw output\, default not include
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/{id}".format(id=id)

    session = api_session or _get_session()
    headers = {}

    query_parameters = {}
    # Only send includeRawOutput if the argument was provided, don't send null values
    if includeRawOutput is not None:
        query_parameters.update({"includeRawOutput": includeRawOutput})

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def get_vulnerability_definition(
    idOrVulnerabilityID: str,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Get Vulnerability definition, also require to have read permission of vulnerability source that the definition is belonging to. (PUBLIC)

    :param str idOrVulnerabilityID: Vulnerability definition ID or identifier of vulnerability
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/definition/{idOrVulnerabilityID}".format(
        idOrVulnerabilityID=idOrVulnerabilityID
    )

    session = api_session or _get_session()
    headers = {}

    response = session.get(
        route,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def get_vulnerability_source(
    source: str,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Get VulnerabilitySource (INTERNAL)

    :param str source: VulnerabilitySource ID or short name
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/source/{source}".format(source=source)

    session = api_session or _get_session()
    headers = {}

    response = session.get(
        route,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def get_vulnerability_statistics(
    asset: str = None,
    assetComponent: dict = None,
    assetGroup: str = None,
    component: dict = None,
    customer: str = None,
    dataSource: str = None,
    excludeFlags: str = None,
    fieldAggregation: dict = None,
    includeFlags: str = None,
    keywords: str = None,
    maximumCvss: float = None,
    minimumCvss: float = None,
    observationReferences: str = None,
    rangeMetric: dict = None,
    references: str = None,
    resolution: str = None,
    severity: str = None,
    statisticsMetric: dict = None,
    subCriteria: dict = None,
    user: str = None,
    vulnerabilityDefinition: str = None,
    vulnerabilityID: str = None,
    endTimestamp: str = "0",
    includeDeleted: bool = None,
    includeRawOutput: bool = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = "any",
    startTimestamp: str = "0",
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = "any",
    userFieldStrategy: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Generate the requested statistics for Vulnerabilities matching the provided search criteria. (PUBLIC)

    :param list asset: Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria\, if any. If none are provided\, we attempt to use the customers the current user has access to.
    :param list assetComponent: Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.\<br \/\>If you want different behaviour try using subcriteria
    :param list assetGroup: Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria\, if any. If none are provided\, we attempt to use the customers the current user has access to.
    :param list component: Search for Vulnerabilities with these components
    :param list customer: Retrieve vulnerabilities by Customer id or name
    :param list dataSource: Search for vulnerabilities detected and reported by these data sources\, identified by id or shortName
    :param list excludeFlags: Explicitly filter out Vulnerabilities that contain these flags
    :param list fieldAggregation:  The fields by which to group statistics. The order of fieldAggregations in the request will determine the hierarchy of the aggregations and their buckets.
    :param list includeFlags: Explicitly retrieve Vulnerabilities that contain these flags
    :param list keywords: Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy
    :param float maximumCvss: Retrieve Vulnerabilities with at most this CVSS score
    :param float minimumCvss: Retrieve Vulnerabilities with at least this CVSS score
    :param list observationReferences: Search for vulnerabilities by exact matches in their references
    :param list rangeMetric: The timestamp fields for which to generate statistics.
    :param list references: Search for vulnerabilities by exact matches in their definition references
    :param list resolution: Retrieve vulnerabilities with these resolutions
    :param list severity: Retrieve vulnerabilities with these severities
    :param list statisticsMetric: The numerical fields for which to generate metrics.
    :param list subCriteria: Set additional criteria that are applied with logical OR by default
    :param list user: Retrieve vulnerabilities by these users\, identified by their id or username
    :param list vulnerabilityDefinition: Retrieve vulnerabilities by their vulnerability definition id or name
    :param list vulnerabilityID: Retrieve vulnerabilities by their external ID e.g. CVE\-XXXX
    :param str endTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(end timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param bool includeDeleted: Whether to include deleted Vulnerabilities
    :param bool includeRawOutput: Whether to include raw output in the result objects
    :param list keywordFieldStrategy: Defines which fields will be queried for keywords \(defaults to all observation fields\)
    :param str keywordMatchStrategy: Defines how strict different keywords should be matched \(default match all keywords\)
    :param str startTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(start timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param list timeFieldStrategy: Defines which timestamps fields will be filtered with start\/endTimestamp
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\)
    :param list userFieldStrategy: Defines which user fields will be filtered with user ids or shortnames
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/statistics".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send asset if the argument was provided, don't send null values
    if asset is not None:
        body.update({"asset": asset})
    # Only send assetComponent if the argument was provided, don't send null values
    if assetComponent is not None:
        body.update({"assetComponent": assetComponent})
    # Only send assetGroup if the argument was provided, don't send null values
    if assetGroup is not None:
        body.update({"assetGroup": assetGroup})
    # Only send component if the argument was provided, don't send null values
    if component is not None:
        body.update({"component": component})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send dataSource if the argument was provided, don't send null values
    if dataSource is not None:
        body.update({"dataSource": dataSource})
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send excludeFlags if the argument was provided, don't send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})
    # Only send fieldAggregation if the argument was provided, don't send null values
    if fieldAggregation is not None:
        body.update({"fieldAggregation": fieldAggregation})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send includeFlags if the argument was provided, don't send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send includeRawOutput if the argument was provided, don't send null values
    if includeRawOutput is not None:
        body.update({"includeRawOutput": includeRawOutput})
    # Only send keywordFieldStrategy if the argument was provided, don't send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send keywordMatchStrategy if the argument was provided, don't send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send maximumCvss if the argument was provided, don't send null values
    if maximumCvss is not None:
        body.update({"maximumCvss": maximumCvss})
    # Only send minimumCvss if the argument was provided, don't send null values
    if minimumCvss is not None:
        body.update({"minimumCvss": minimumCvss})
    # Only send observationReferences if the argument was provided, don't send null values
    if observationReferences is not None:
        body.update({"observationReferences": observationReferences})
    # Only send rangeMetric if the argument was provided, don't send null values
    if rangeMetric is not None:
        body.update({"rangeMetric": rangeMetric})
    # Only send references if the argument was provided, don't send null values
    if references is not None:
        body.update({"references": references})
    # Only send resolution if the argument was provided, don't send null values
    if resolution is not None:
        body.update({"resolution": resolution})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send statisticsMetric if the argument was provided, don't send null values
    if statisticsMetric is not None:
        body.update({"statisticsMetric": statisticsMetric})
    # Only send subCriteria if the argument was provided, don't send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send timeFieldStrategy if the argument was provided, don't send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send timeMatchStrategy if the argument was provided, don't send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})
    # Only send user if the argument was provided, don't send null values
    if user is not None:
        body.update({"user": user})
    # Only send userFieldStrategy if the argument was provided, don't send null values
    if userFieldStrategy is not None:
        body.update({"userFieldStrategy": userFieldStrategy})
    # Only send vulnerabilityDefinition if the argument was provided, don't send null values
    if vulnerabilityDefinition is not None:
        body.update({"vulnerabilityDefinition": vulnerabilityDefinition})
    # Only send vulnerabilityID if the argument was provided, don't send null values
    if vulnerabilityID is not None:
        body.update({"vulnerabilityID": vulnerabilityID})

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def list_vulnerabilities(
    customer: str = None,
    asset: str = None,
    dataSource: str = None,
    vulnerabilityID: str = None,
    severity: str = None,
    resolution: str = None,
    sortBy: str = None,
    limit: int = 25,
    offset: int = 0,
    includeDeleted: bool = False,
    includeRawOutput: bool = False,
    startTimestamp: str = "0",
    endTimestamp: str = "0",
    timeFieldStrategy: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Basic Vulnerability search. Will not return vulnerabilities seen more than 31 days ago. Please use advanced search to return older vulnerabilities. If start/end timestamps are present, the range must be valid for at least one of the time stamp fields. Check the documentation for the TimestampFieldStrategy field for more details. (PUBLIC)

    :param list customer: Customer ids or short names
    :param list asset: Asset ids or short names. Will resolve with the provided customers\, or those the current user has access to if none are provided
    :param list dataSource: Data source\, identified by id or shortname\, that detected the Vulnerability
    :param list vulnerabilityID: Vulnerability external ID
    :param list severity: Vulnerability Severity
    :param list resolution: Vulnerability Resolution
    :param list sortBy: Field\(s\) to sort by \- refer to the search POST endpoint to see which fields are permitted
    :param int limit: Limit results. The sum of limit and offset must be \<\= 10\'000 when not streaming
    :param int offset: Offset results. The sum of limit and offset must be \<\= 10\'000 when not streaming
    :param bool includeDeleted: Include deleted
    :param bool includeRawOutput: Include raw output from scanner
    :param str startTimestamp: Lower bound timestamp filter value
    :param str endTimestamp: Upper bound timestamp filter value
    :param list timeFieldStrategy: Which time field\(s\) to filter by timestamp start\/end
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability".format()

    session = api_session or _get_session()
    headers = {}

    query_parameters = {}
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        query_parameters.update({"includeDeleted": includeDeleted})
    # Only send includeRawOutput if the argument was provided, don't send null values
    if includeRawOutput is not None:
        query_parameters.update({"includeRawOutput": includeRawOutput})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        query_parameters.update({"startTimestamp": startTimestamp})
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        query_parameters.update({"endTimestamp": endTimestamp})
    # Only send timeFieldStrategy if the argument was provided, don't send null values
    if timeFieldStrategy is not None:
        query_parameters.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        query_parameters.update({"customer": customer})
    # Only send asset if the argument was provided, don't send null values
    if asset is not None:
        query_parameters.update({"asset": asset})
    # Only send dataSource if the argument was provided, don't send null values
    if dataSource is not None:
        query_parameters.update({"dataSource": dataSource})
    # Only send vulnerabilityID if the argument was provided, don't send null values
    if vulnerabilityID is not None:
        query_parameters.update({"vulnerabilityID": vulnerabilityID})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        query_parameters.update({"severity": severity})
    # Only send resolution if the argument was provided, don't send null values
    if resolution is not None:
        query_parameters.update({"resolution": resolution})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        query_parameters.update({"sortBy": sortBy})

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def list_vulnerability_definitions(
    vulnerabilityID: str = None,
    keywords: str = None,
    sortBy: str = None,
    limit: int = 25,
    offset: int = 0,
    includeDeleted: bool = False,
    startTimestamp: str = "0",
    endTimestamp: str = "0",
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Basic Vulnerability Definition search. (PUBLIC)

    :param list vulnerabilityID: External Vulnerability ID
    :param list keywords: Keywords to search by. Will return matches in any field defined by the \'all\' KeywordFieldStrategy
    :param list sortBy: Field\(s\) to sort by \- refer to the search POST endpoint to see which fields are permitted
    :param int limit: Limit results. The sum of limit and offset must be \<\= 10\'000 when not streaming
    :param int offset: Offset results. The sum of limit and offset must be \<\= 10\'000 when not streaming
    :param bool includeDeleted: Include deleted
    :param str startTimestamp: Lower bound timestamp filter value
    :param str endTimestamp: Upper bound timestamp filter value
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/definition".format()

    session = api_session or _get_session()
    headers = {}

    query_parameters = {}
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        query_parameters.update({"includeDeleted": includeDeleted})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        query_parameters.update({"startTimestamp": startTimestamp})
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        query_parameters.update({"endTimestamp": endTimestamp})
    # Only send vulnerabilityID if the argument was provided, don't send null values
    if vulnerabilityID is not None:
        query_parameters.update({"vulnerabilityID": vulnerabilityID})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        query_parameters.update({"keywords": keywords})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        query_parameters.update({"sortBy": sortBy})

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def reopen_vulnerability(
    id: str,
    comment: str,
    forceIndex: bool = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Reopens a vulnerability, marking it as unresolved. (PUBLIC)

    :param str id: Vulnerability ID
    :param str comment: Comment describing why the vulnerability should be reopened
    :param bool forceIndex: Whether to force an Elasticsearch index refresh such that the reopened vulnerability is immediately searchable.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/{id}/reopen".format(id=id)

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send comment if the argument was provided, don't send null values
    if comment is not None:
        body.update({"comment": comment})

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def resolve_vulnerability(
    id: str,
    comment: str,
    resolution: str,
    forceIndex: bool = None,
    resolutionExpiryTimestamp: str = "0",
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Resolves a vulnerability. (PUBLIC)

    :param str id: Vulnerability ID
    :param str comment: Comment on why the vulnerability is resolved.
    :param str resolution: Specify resolution reason.
    :param bool forceIndex: Whether to force an Elasticsearch index refresh such that the resolved vulnerability is immediately searchable.
    :param str resolutionExpiryTimestamp: When will the current resolution expire \(0 means no expiry\). Allows unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time values. See https\:\/\/docs.mnemonic.no\/x\/AQDXAQ  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/{id}/resolve".format(id=id)

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send comment if the argument was provided, don't send null values
    if comment is not None:
        body.update({"comment": comment})
    # Only send resolution if the argument was provided, don't send null values
    if resolution is not None:
        body.update({"resolution": resolution})
    # Only send resolutionExpiryTimestamp if the argument was provided, don't send null values
    if resolutionExpiryTimestamp is not None:
        body.update({"resolutionExpiryTimestamp": resolutionExpiryTimestamp})

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def search_vulnerabilities(
    asset: str = None,
    assetComponent: dict = None,
    assetGroup: str = None,
    component: dict = None,
    customer: str = None,
    dataSource: str = None,
    excludeFlags: str = None,
    includeFlags: str = None,
    indexEndTimestamp: str = None,
    indexStartTimestamp: str = None,
    keywords: str = None,
    maximumCvss: float = None,
    minimumCvss: float = None,
    observationReferences: str = None,
    order: str = None,
    references: str = None,
    resolution: str = None,
    severity: str = None,
    sortBy: str = None,
    subCriteria: dict = None,
    user: str = None,
    vulnerabilityDefinition: str = None,
    vulnerabilityID: str = None,
    exportFields: str = None,
    exportComponents: str = None,
    endTimestamp: str = "0",
    includeDeleted: bool = None,
    includeRawOutput: bool = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = "any",
    limit: int = 25,
    offset: int = None,
    startTimestamp: str = "0",
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = "any",
    userFieldStrategy: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Search Vulnerabilities (PUBLIC)

    :param list asset: Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria\, if any. If none are provided\, we attempt to use the customers the current user has access to.
    :param list assetComponent: Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.\<br \/\>If you want different behaviour try using subcriteria
    :param list assetGroup: Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria\, if any. If none are provided\, we attempt to use the customers the current user has access to.
    :param list component: Search for Vulnerabilities with these components
    :param list customer: Retrieve vulnerabilities by Customer id or name
    :param list dataSource: Search for vulnerabilities detected and reported by these data sources\, identified by id or shortName
    :param list excludeFlags: Explicitly filter out Vulnerabilities that contain these flags
    :param list includeFlags: Explicitly retrieve Vulnerabilities that contain these flags
    :param str indexEndTimestamp: Limit search to indices created on or before this timestamp. Must be in the range \[12 months ago \- today\].  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param str indexStartTimestamp: Limit search to indices created on or after this timestamp. Must be in the range \[12 months ago \- today\].  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param list keywords: Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy
    :param float maximumCvss: Retrieve Vulnerabilities with at most this CVSS score
    :param float minimumCvss: Retrieve Vulnerabilities with at least this CVSS score
    :param list observationReferences: Search for vulnerabilities by exact matches in their references
    :param list order:
    :param list references: Search for vulnerabilities by exact matches in their definition references
    :param list resolution: Retrieve vulnerabilities with these resolutions
    :param list severity: Retrieve vulnerabilities with these severities
    :param list sortBy: Field\(s\) by which to sort the results
    :param list subCriteria: Set additional criteria that are applied with logical OR by default
    :param list user: Retrieve vulnerabilities by these users\, identified by their id or username
    :param list vulnerabilityDefinition: Retrieve vulnerabilities by their vulnerability definition id or name
    :param list vulnerabilityID: Retrieve vulnerabilities by their external ID e.g. CVE\-XXXX
    :param list exportFields: Select order of and fields to export.
    :param list exportComponents: Select which component values to include in the export.
    :param str endTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(end timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param bool includeDeleted: Whether to include deleted Vulnerabilities
    :param bool includeRawOutput: Whether to include raw output in the result objects
    :param list keywordFieldStrategy: Defines which fields will be queried for keywords \(defaults to all observation fields\)
    :param str keywordMatchStrategy: Defines how strict different keywords should be matched \(default match all keywords\)
    :param int limit: Limit the results to the specified amount. If the sum of offset and limit is greater than 10\'000 the results will likely be incomplete as 10\'000 exceeds the search engine\'s maximum result window.
    :param int offset: Skip specified amount of results. If the sum of offset and limit is greater than 10\'000 the results will likely be incomplete as 10\'000 exceeds the search engine\'s maximum result window.
    :param str startTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(start timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param list timeFieldStrategy: Defines which timestamps fields will be filtered with start\/endTimestamp
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\)
    :param list userFieldStrategy: Defines which user fields will be filtered with user ids or shortnames
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: ``requests.Response`` object or dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/search".format()

    session = api_session or _get_session()
    headers = {}
    headers["content"] = None
    if json:
        headers["content"] = "application/json"

    body = body or {}
    # Only send asset if the argument was provided, don't send null values
    if asset is not None:
        body.update({"asset": asset})
    # Only send assetComponent if the argument was provided, don't send null values
    if assetComponent is not None:
        body.update({"assetComponent": assetComponent})
    # Only send assetGroup if the argument was provided, don't send null values
    if assetGroup is not None:
        body.update({"assetGroup": assetGroup})
    # Only send component if the argument was provided, don't send null values
    if component is not None:
        body.update({"component": component})
    # Only send customer if the argument was provided, don't send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send dataSource if the argument was provided, don't send null values
    if dataSource is not None:
        body.update({"dataSource": dataSource})
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send excludeFlags if the argument was provided, don't send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send includeFlags if the argument was provided, don't send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send includeRawOutput if the argument was provided, don't send null values
    if includeRawOutput is not None:
        body.update({"includeRawOutput": includeRawOutput})
    # Only send indexEndTimestamp if the argument was provided, don't send null values
    if indexEndTimestamp is not None:
        body.update({"indexEndTimestamp": indexEndTimestamp})
    # Only send indexStartTimestamp if the argument was provided, don't send null values
    if indexStartTimestamp is not None:
        body.update({"indexStartTimestamp": indexStartTimestamp})
    # Only send keywordFieldStrategy if the argument was provided, don't send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send keywordMatchStrategy if the argument was provided, don't send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send maximumCvss if the argument was provided, don't send null values
    if maximumCvss is not None:
        body.update({"maximumCvss": maximumCvss})
    # Only send minimumCvss if the argument was provided, don't send null values
    if minimumCvss is not None:
        body.update({"minimumCvss": minimumCvss})
    # Only send observationReferences if the argument was provided, don't send null values
    if observationReferences is not None:
        body.update({"observationReferences": observationReferences})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send order if the argument was provided, don't send null values
    if order is not None:
        body.update({"order": order})
    # Only send references if the argument was provided, don't send null values
    if references is not None:
        body.update({"references": references})
    # Only send resolution if the argument was provided, don't send null values
    if resolution is not None:
        body.update({"resolution": resolution})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send subCriteria if the argument was provided, don't send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send timeFieldStrategy if the argument was provided, don't send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send timeMatchStrategy if the argument was provided, don't send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})
    # Only send user if the argument was provided, don't send null values
    if user is not None:
        body.update({"user": user})
    # Only send userFieldStrategy if the argument was provided, don't send null values
    if userFieldStrategy is not None:
        body.update({"userFieldStrategy": userFieldStrategy})
    # Only send vulnerabilityDefinition if the argument was provided, don't send null values
    if vulnerabilityDefinition is not None:
        body.update({"vulnerabilityDefinition": vulnerabilityDefinition})
    # Only send vulnerabilityID if the argument was provided, don't send null values
    if vulnerabilityID is not None:
        body.update({"vulnerabilityID": vulnerabilityID})

    query_parameters = {}
    # Only send exportFields if the argument was provided, don't send null values
    if exportFields is not None:
        query_parameters.update({"exportFields": exportFields})
    # Only send exportComponents if the argument was provided, don't send null values
    if exportComponents is not None:
        query_parameters.update({"exportComponents": exportComponents})

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def search_vulnerability_definitions(
    excludeFlags: str = None,
    includeFlags: str = None,
    keywords: str = None,
    maximumCvss: float = None,
    minimumCvss: float = None,
    order: str = None,
    severity: str = None,
    sortBy: str = None,
    subCriteria: dict = None,
    user: str = None,
    vulnerabilityDefinition: str = None,
    vulnerabilityID: str = None,
    vulnerabilitySource: str = None,
    exportFields: str = None,
    endTimestamp: str = "0",
    includeDeleted: bool = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = "any",
    limit: int = 25,
    offset: int = None,
    startTimestamp: str = "0",
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = "any",
    userFieldStrategy: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Search Vulnerability Definitions (PUBLIC)

    :param list excludeFlags: Explicitly filter out VulnerabilityDefinitions that contain these flags
    :param list includeFlags: Explicitly search for VulnerabilityDefinitions that contain these flags
    :param list keywords: Search for keywords against fields defined by a KeywordFieldStrategy
    :param float maximumCvss: Filter VulnerabilityDefinitions by CVSS lower or equal than.
    :param float minimumCvss: Filter VulnerabilityDefinitions by CVSS higher or equal than.
    :param list order:
    :param list severity: Filter VulnerabilityDefinitions by Severity.
    :param list sortBy: Field\(s\) by which to sort the results
    :param list subCriteria: Set additional criteria that are applied with logical OR by default
    :param list user: Search for VulnerabilityDefinitions by associated user IDs or short names defined by the UserFieldStrategy
    :param list vulnerabilityDefinition: Retrieve definitions by their internal db ID or vulnerabilityID
    :param list vulnerabilityID: Retrieve definitions by their external ID e.g. CVE\-XXXX
    :param list vulnerabilitySource: Retrieve definitions by their source ID or shortname
    :param list exportFields: Select order of and fields to export.
    :param str endTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(end timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param bool includeDeleted: Whether to include deleted Vulnerability Definitions
    :param list keywordFieldStrategy: Defines which fields will be searched by keywords \(default all supported fields\)
    :param str keywordMatchStrategy: Defines how strict different keywords should be matched \(default match all keywords\)
    :param int limit: Limit the results to the specified amount
    :param int offset: Skip specified amount of results
    :param str startTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(start timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param list timeFieldStrategy: Defines which timestamps fields will be filtered with start\/endTimestamp
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\)
    :param list userFieldStrategy: Defines which user fields to filter with the submitted user value\(s\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: ``requests.Response`` object or dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/definition/search".format()

    session = api_session or _get_session()
    headers = {}
    headers["content"] = None
    if json:
        headers["content"] = "application/json"

    body = body or {}
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send excludeFlags if the argument was provided, don't send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send includeFlags if the argument was provided, don't send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send keywordFieldStrategy if the argument was provided, don't send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send keywordMatchStrategy if the argument was provided, don't send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send maximumCvss if the argument was provided, don't send null values
    if maximumCvss is not None:
        body.update({"maximumCvss": maximumCvss})
    # Only send minimumCvss if the argument was provided, don't send null values
    if minimumCvss is not None:
        body.update({"minimumCvss": minimumCvss})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send order if the argument was provided, don't send null values
    if order is not None:
        body.update({"order": order})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send subCriteria if the argument was provided, don't send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send timeFieldStrategy if the argument was provided, don't send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send timeMatchStrategy if the argument was provided, don't send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})
    # Only send user if the argument was provided, don't send null values
    if user is not None:
        body.update({"user": user})
    # Only send userFieldStrategy if the argument was provided, don't send null values
    if userFieldStrategy is not None:
        body.update({"userFieldStrategy": userFieldStrategy})
    # Only send vulnerabilityDefinition if the argument was provided, don't send null values
    if vulnerabilityDefinition is not None:
        body.update({"vulnerabilityDefinition": vulnerabilityDefinition})
    # Only send vulnerabilityID if the argument was provided, don't send null values
    if vulnerabilityID is not None:
        body.update({"vulnerabilityID": vulnerabilityID})
    # Only send vulnerabilitySource if the argument was provided, don't send null values
    if vulnerabilitySource is not None:
        body.update({"vulnerabilitySource": vulnerabilitySource})

    query_parameters = {}
    # Only send exportFields if the argument was provided, don't send null values
    if exportFields is not None:
        query_parameters.update({"exportFields": exportFields})

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def search_vulnerability_sources(
    keywords: str = None,
    sortBy: str = None,
    source: str = None,
    subCriteria: dict = None,
    endTimestamp: str = "0",
    includeDeleted: bool = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = "all",
    limit: int = 25,
    offset: int = None,
    startTimestamp: str = "0",
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = "any",
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Search VulnerabilitySource with specified criteria (INTERNAL)

    :param list keywords: Search for keywords against fields defined by keywordFieldStrategy
    :param list sortBy: Field\(s\) by which to sort the results
    :param list source: Restrict to specified Vulnerability source \(ID or short name\)
    :param list subCriteria: Set additional criteria which are applied with logical OR by default
    :param str endTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(end timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param bool includeDeleted: Whether include deleted results
    :param list keywordFieldStrategy: Defines which fields will be searched by keywords \(default all supported fields\)
    :param str keywordMatchStrategy: Defines how strict different keywords should be matched \(default match all keywords\)
    :param int limit: Limit maximum amount of results
    :param int offset: Skip specified amount of results
    :param str startTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(start timestamp\)  Allows to specify a unix timestamp \(milliseconds\)\, ISO timestamp\, or relative time.
    :param list timeFieldStrategy: Defines which timestamps will be included in the search \(default lastUpdatedTimestamp\)
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/source/search".format()

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send includeDeleted if the argument was provided, don't send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send keywordFieldStrategy if the argument was provided, don't send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send keywordMatchStrategy if the argument was provided, don't send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send source if the argument was provided, don't send null values
    if source is not None:
        body.update({"source": source})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send subCriteria if the argument was provided, don't send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send timeFieldStrategy if the argument was provided, don't send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send timeMatchStrategy if the argument was provided, don't send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})

    response = session.post(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def search_vulnerability_sources_simplified(
    keywords: str = None,
    source: str = None,
    sortBy: str = None,
    offset: int = 0,
    limit: int = 25,
    keywordField: str = None,
    keywordMatch: str = "all",
    startTimestamp: str = "0",
    endTimestamp: str = "0",
    timeField: str = None,
    timeMatch: str = "any",
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Search VulnerabilitySources with specified query parameters (INTERNAL)

    :param list keywords: Search by keywords
    :param list source: Search by sources \(ID or short name\)
    :param list sortBy: Sort search result
    :param int offset: Skip a number of results
    :param int limit: Maximum number of returned results
    :param list keywordField: Set field strategy for keyword search
    :param str keywordMatch: Set match strategy for keyword search
    :param str startTimestamp: Start timestamp for time range search
    :param str endTimestamp: End timestamp for time range search
    :param list timeField: Set field strategy for time range search
    :param str timeMatch: Set match strategy for time range search
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/source".format()

    session = api_session or _get_session()
    headers = {}

    query_parameters = {}
    # Only send offset if the argument was provided, don't send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    # Only send limit if the argument was provided, don't send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    # Only send keywordField if the argument was provided, don't send null values
    if keywordField is not None:
        query_parameters.update({"keywordField": keywordField})
    # Only send keywordMatch if the argument was provided, don't send null values
    if keywordMatch is not None:
        query_parameters.update({"keywordMatch": keywordMatch})
    # Only send startTimestamp if the argument was provided, don't send null values
    if startTimestamp is not None:
        query_parameters.update({"startTimestamp": startTimestamp})
    # Only send endTimestamp if the argument was provided, don't send null values
    if endTimestamp is not None:
        query_parameters.update({"endTimestamp": endTimestamp})
    # Only send timeField if the argument was provided, don't send null values
    if timeField is not None:
        query_parameters.update({"timeField": timeField})
    # Only send timeMatch if the argument was provided, don't send null values
    if timeMatch is not None:
        query_parameters.update({"timeMatch": timeMatch})
    # Only send keywords if the argument was provided, don't send null values
    if keywords is not None:
        query_parameters.update({"keywords": keywords})
    # Only send source if the argument was provided, don't send null values
    if source is not None:
        query_parameters.update({"source": source})
    # Only send sortBy if the argument was provided, don't send null values
    if sortBy is not None:
        query_parameters.update({"sortBy": sortBy})

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def update_vulnerability(
    id: str,
    addComponents: dict = None,
    addObservationReferences: str = None,
    cvss: float = None,
    deleteComponents: str = None,
    deleteObservationReferences: str = None,
    observationDescription: str = None,
    rawOutput: str = None,
    severity: str = None,
    ttl: int = None,
    vulnerabilityDefinition: str = None,
    forceIndex: bool = None,
    updateLastSeen: bool = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Updates an existing Vulnerability (PUBLIC)

    :param str id: vulnerability ID
    :param list addComponents: Specify components to be added with the vulnerability.
    :param list addObservationReferences: Add references specific to this vulnerability e.g. internal URLs. May contain customer specific data. Will be sanitized as HTML.
    :param float cvss: Update CVSS score of vulnerability \(range from 0 to 10\).
    :param list deleteComponents: Specify IDs of the components to be deleted from the vulnerability.
    :param list deleteObservationReferences: Delete references specific to this vulnerability. Will be sanitized as HTML first before checking against existing references.
    :param str observationDescription: Description specific to this vulnerability. May contain customer specific data. Will be sanitized as HTML.
    :param str rawOutput: Update raw output from vulnerability scan. Will be sanitized as html.
    :param str severity: Update severity level of vulnerability.
    :param int ttl: Time to live duration of this vulnerability since last seen\, 0 means won\'t expire.
    :param str vulnerabilityDefinition: Assign vulnerability definition. Only allowed when there was no vulnerability definition assigned.
    :param bool forceIndex: Whether to force an Elasticsearch index refresh such that changes to the vulnerability are immediately searchable.
    :param bool updateLastSeen: Temporary field added to make updating lastSeenTimestamp\/UserId easier.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/{id}".format(id=id)

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send addComponents if the argument was provided, don't send null values
    if addComponents is not None:
        body.update({"addComponents": addComponents})
    # Only send addObservationReferences if the argument was provided, don't send null values
    if addObservationReferences is not None:
        body.update({"addObservationReferences": addObservationReferences})
    # Only send cvss if the argument was provided, don't send null values
    if cvss is not None:
        body.update({"cvss": cvss})
    # Only send deleteComponents if the argument was provided, don't send null values
    if deleteComponents is not None:
        body.update({"deleteComponents": deleteComponents})
    # Only send deleteObservationReferences if the argument was provided, don't send null values
    if deleteObservationReferences is not None:
        body.update({"deleteObservationReferences": deleteObservationReferences})
    # Only send observationDescription if the argument was provided, don't send null values
    if observationDescription is not None:
        body.update({"observationDescription": observationDescription})
    # Only send rawOutput if the argument was provided, don't send null values
    if rawOutput is not None:
        body.update({"rawOutput": rawOutput})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send ttl if the argument was provided, don't send null values
    if ttl is not None:
        body.update({"ttl": ttl})
    # Only send vulnerabilityDefinition if the argument was provided, don't send null values
    if vulnerabilityDefinition is not None:
        body.update({"vulnerabilityDefinition": vulnerabilityDefinition})

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})
    # Only send updateLastSeen if the argument was provided, don't send null values
    if updateLastSeen is not None:
        query_parameters.update({"updateLastSeen": updateLastSeen})

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def update_vulnerability_definition(
    idOrVulnerabilityID: str,
    addReferences: str = None,
    conclusion: str = None,
    cvss: float = None,
    deleteReferences: str = None,
    description: str = None,
    exploitAvailable: bool = None,
    name: str = None,
    overwriteReferences: str = None,
    severity: str = None,
    solution: str = None,
    forceIndex: bool = None,
    exploitAvailableTimestamp: int = None,
    updateObservations: bool = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Updates an existing Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to. (INTERNAL)

    :param str idOrVulnerabilityID: Vulnerability definition ID or identifier of vulnerability
    :param list addReferences: Add references to the vulnerability definition \(e.g. CVE number\). Will be sanitized as html.
    :param str conclusion: Update short summary of the vulnerability. Will be sanitized as html.
    :param float cvss: CVSS score of vulnerability \(range from 0 to 10\).
    :param list deleteReferences: Delete references from the vulnerability definition. Will be sanitized as html first before check against existing references.
    :param str description: Update description of the vulnerability definition. Will be sanitized as html.
    :param bool exploitAvailable: Update whether an exploit is available for the vulnerability.
    :param str name: Update name of the vulnerability definition. Will be sanitized as text.
    :param list overwriteReferences: Overwrite any existing references with these new values. Requests with this and add\/deleteReferences populated are considered invalid and will be rejected. Contents will be sanitized as HTML before saving.
    :param str severity: Severity level of vulnerability. If not provided\, then severity will be calculated based on CVSS value.
    :param str solution: Update how to fix the vulnerability. Will be sanitized as html.
    :param bool forceIndex: Whether to force an Elasticsearch index refresh such that changes to the VulnerabilityDefinition are immediately searchable.
    :param int exploitAvailableTimestamp: Set or update the timestamp when the exploit became available. Setting this value also expects having exploitAvailable as \'true\' or setting it with this request. If it hasn\'t been set before\, then setting\/leaving it to 0 will use current timestamp.
    :param bool updateObservations: If true this will trigger updating all active \(non\-deleted with resolution status\: unresolved\, accepted or temp\_accepted\) Observations thus overwriting their current Severity\/CVSS values. Unless the values were previously overwritten manually.
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/definition/{idOrVulnerabilityID}".format(
        idOrVulnerabilityID=idOrVulnerabilityID
    )

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send addReferences if the argument was provided, don't send null values
    if addReferences is not None:
        body.update({"addReferences": addReferences})
    # Only send conclusion if the argument was provided, don't send null values
    if conclusion is not None:
        body.update({"conclusion": conclusion})
    # Only send cvss if the argument was provided, don't send null values
    if cvss is not None:
        body.update({"cvss": cvss})
    # Only send deleteReferences if the argument was provided, don't send null values
    if deleteReferences is not None:
        body.update({"deleteReferences": deleteReferences})
    # Only send description if the argument was provided, don't send null values
    if description is not None:
        body.update({"description": description})
    # Only send exploitAvailable if the argument was provided, don't send null values
    if exploitAvailable is not None:
        body.update({"exploitAvailable": exploitAvailable})
    # Only send exploitAvailableTimestamp if the argument was provided, don't send null values
    if exploitAvailableTimestamp is not None:
        body.update({"exploitAvailableTimestamp": exploitAvailableTimestamp})
    # Only send name if the argument was provided, don't send null values
    if name is not None:
        body.update({"name": name})
    # Only send overwriteReferences if the argument was provided, don't send null values
    if overwriteReferences is not None:
        body.update({"overwriteReferences": overwriteReferences})
    # Only send severity if the argument was provided, don't send null values
    if severity is not None:
        body.update({"severity": severity})
    # Only send solution if the argument was provided, don't send null values
    if solution is not None:
        body.update({"solution": solution})
    # Only send updateObservations if the argument was provided, don't send null values
    if updateObservations is not None:
        body.update({"updateObservations": updateObservations})

    query_parameters = {}
    # Only send forceIndex if the argument was provided, don't send null values
    if forceIndex is not None:
        query_parameters.update({"forceIndex": forceIndex})

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def update_vulnerability_source(
    source: str,
    name: str = None,
    readPermission: str = None,
    shortName: str = None,
    writePermission: str = None,
    json: bool = True,
    verify: Optional[bool] = None,
    proxies: Optional[dict] = None,
    apiKey: Optional[str] = None,
    authentication: Optional[dict] = None,
    server_url: Optional[str] = None,
    body: Optional[dict] = None,
    api_session: Optional["ArgusAPISession"] = None,
) -> dict:
    r"""Updates an existing VulnerabilitySource (INTERNAL)

    :param str source: VulnerabilitySource ID or short name
    :param str name: Update name of vulnerability source
    :param str readPermission: Update the permission required to allow read access of source \& vulnerability definition belong to the source
    :param str shortName: Update short name of vulnerability source
    :param str writePermission: Update the permission required to allow write access \(create\/update\/delete\) of source \& vulnerability definition belong to the source
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :param api_session: session to use for this request. If not set, the global session will be used.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes

    :returns: dictionary translated from JSON
    """

    route = "/assets/v2/vulnerability/source/{source}".format(source=source)

    session = api_session or _get_session()
    headers = {}

    body = body or {}
    # Only send name if the argument was provided, don't send null values
    if name is not None:
        body.update({"name": name})
    # Only send readPermission if the argument was provided, don't send null values
    if readPermission is not None:
        body.update({"readPermission": readPermission})
    # Only send shortName if the argument was provided, don't send null values
    if shortName is not None:
        body.update({"shortName": shortName})
    # Only send writePermission if the argument was provided, don't send null values
    if writePermission is not None:
        body.update({"writePermission": writePermission})

    response = session.put(
        route,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response
