"""Autogenerated API schema"""
from argus_api import session


add_asset = {'tags': ['assets/v2/asset'], 'summary': 'Creates a new Asset (PUBLIC)', 'description': '', 'operationId': 'addAsset', 'parameters': [{'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh making the new Asset immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to create an Asset', 'properties': {'assetDefinition': {'type': 'str', 'description': "ID or short name of asset definition, it should exist within current user's domain"}, 'components': {'type': 'list', 'description': 'Specify components to be added with the asset', 'items': {'type': 'dict', 'description': 'Request to add Components to an Asset', 'properties': {'componentDefinition': {'type': 'str', 'description': 'ID or short name of component definition.\nIt should exist within asset definition for the owning asset\n'}, 'ttl': {'type': 'int', 'description': "Time to live duration of this component since last seen, 0 means won't expire.", 'minimum': 0}, 'unique': {'type': 'bool', 'description': 'Whether the Asset can only have a single active component of this type\n<b>NB: Adding unique components to an existing asset will expire existing components with the same component definition</b>\n'}, 'value': {'type': 'str', 'description': "Value for the asset component.\nType of value should match component definition's requirement.\n"}}, 'required': ['componentDefinition', 'value']}, 'uniqueItems': True}, 'criticality': {'type': 'dict', 'description': "Request to set a Group or Asset's criticality triad", 'properties': {'availability': {'type': 'str', 'default': 'unknown', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'str', 'default': 'unknown', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'str', 'default': 'unknown', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}, 'required': ['availability', 'confidentiality', 'integrity']}, 'customer': {'type': 'str', 'description': "ID or short name of customer, if not specified then current user's customer will be assigned"}, 'dataSource': {'type': 'str', 'description': 'ID or short name of the Data Source'}, 'description': {'type': 'str', 'description': 'Description of asset. May use HTML, which will be sanitized'}, 'name': {'type': 'str', 'description': 'Name of asset', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'owner': {'type': 'str', 'description': "ID or username of user who owns the asset. When username is specified, it should exist within current user's domain"}, 'shortName': {'type': 'str', 'description': "Customer unique short name of asset. If not specified will be set to the new asset's generated/assigned UUID.", 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'ttl': {'type': 'int', 'description': "Time to live duration of this asset since last seen, 0 means won't expire.", 'minimum': 0}}, 'required': ['assetDefinition', 'criticality', 'name']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new Asset', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['addAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
add_asset_definition = {'tags': ['assets/v2/asset/definition'], 'summary': 'Creates a new AssetDefinition (INTERNAL)', 'description': '', 'operationId': 'addAssetDefinition', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to create an Asset Definition', 'properties': {'component': {'type': 'list', 'description': 'Component definitions (ID or short name) to be included for the asset definition', 'items': {'type': 'str', 'description': 'Component definitions (ID or short name) to be included for the asset definition'}, 'uniqueItems': True}, 'description': {'type': 'str', 'description': 'Optional Asset definition description. Will be HTML sanitized'}, 'domain': {'type': 'str', 'description': "Asset/Component definitions will be available globally (in all domains), so 'domain' no longer needs to be specified."}, 'name': {'type': 'str', 'description': 'Asset definition name', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'shortName': {'type': 'str', 'description': 'Asset definition short name, required and unique per domain', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}}, 'required': ['name', 'shortName']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new AssetDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Definition determines what kind of components may be attached to Assets of this type', 'properties': {'componentDefinitions': {'type': 'array', 'description': 'Component definitions belong to asset definition', 'items': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset definition'}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': 'Flags of asset definition', 'items': {'type': 'string', 'description': 'Flags of asset definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was last updated'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['addAssetDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
add_asset_link = {'tags': ['assets/v2/asset/link'], 'summary': 'Creates a new AssetLink (DEV)', 'description': '', 'operationId': 'addAssetLink', 'parameters': [{'name': 'origin', 'in': 'path', 'description': 'Asset Link Origin Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when asset short name was provided", 'schema': {'type': 'string'}, 'type': 'str'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to add an Asset Link between two active Assets.', 'properties': {'definition': {'type': 'str', 'description': 'The id or shortname of the link definition.'}, 'target': {'type': 'str', 'description': 'The id or the shortname of the target asset.'}, 'ttl': {'type': 'int', 'description': "Optional TTL in millis that will set the link's expiry timestamp on creation and when refreshing the link if a new TTL is not provided.", 'minimum': 0}}, 'required': ['definition', 'target']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new AssetLink', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A link between two active Assets', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was created'}, 'definition': {'type': 'object', 'description': "Compact representation of an Asset Link Definition's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was deleted'}, 'expiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link is scheduled to expire. 0 means never.'}, 'flags': {'type': 'array', 'items': {'type': 'string', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was lastUpdated'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'How long the link is expected to live. Updates not specifying a new ttl will reuse this value. 0 means forever.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'DEV'}
add_asset_link_definition = {'tags': ['assets/v2/asset/link/definition'], 'summary': 'Creates a new AssetLinkDefinition (TEST)', 'description': '', 'operationId': 'addAssetLinkDefinition', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to create an Asset Link Definition', 'properties': {'bindings': {'type': 'list', 'description': 'The Asset Link Bindings permitted by the Asset Link Definition.', 'items': {'type': 'dict', 'description': 'Request to create an Asset Link Binding when creating an Asset Link Definition', 'properties': {'origin': {'type': 'str', 'description': 'The Asset Definition id or shortname for origin Assets of the relationship represented by this binding.'}, 'target': {'type': 'str', 'description': 'The Asset Definition id or shortname for target Assets of the relationship represented by this binding.'}}, 'required': ['origin', 'target']}, 'uniqueItems': True}, 'inverse': {'type': 'str', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'localizedDescriptions': {'type': 'dict', 'additionalProperties': {'type': 'str', 'description': 'The localized descriptions of the link definition. There must be at least one entry and English is required.'}, 'description': 'The localized descriptions of the link definition. There must be at least one entry and English is required.'}, 'property': {'type': 'str', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'str', 'description': 'The globally unique shortname of the link definition'}}, 'required': ['localizedDescriptions', 'property', 'shortName']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new AssetLinkDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Link Definition describes and restricts relationships between specific types of Assets.', 'properties': {'bindings': {'type': 'array', 'description': 'Asset Link Bindings belonging to this Asset Link Definition.', 'items': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Definition', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was last updated.'}, 'localizedDescription': {'type': 'string', 'description': "Description of the Asset Link Definition based on the current user's language settings. Defaults to English if a localised version is unavailable."}, 'localizedDescriptions': {'type': 'object', 'additionalProperties': {'type': 'string', 'description': 'All available descriptions of the Asset Link Definition'}, 'description': 'All available descriptions of the Asset Link Definition'}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}
add_asset_link_definition_binding = {'tags': ['assets/v2/asset/link/binding'], 'summary': 'Creates a new AssetLinkBinding on an existing active AssetLinkDefinition (TEST)', 'description': '', 'operationId': 'addAssetLinkBinding', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'Asset Link Definition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to add an Asset Link Binding to an existing active Asset Link Definition', 'properties': {'origin': {'type': 'str', 'description': 'The Asset Definition id or shortname for origin Assets of the relationship represented by this binding.'}, 'target': {'type': 'str', 'description': 'The Asset Definition id or shortname for target Assets of the relationship represented by this binding.'}}, 'required': ['origin', 'target']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new AssetLinkBinding on an existing active AssetLinkDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'assetLinkDefinition': {'type': 'object', 'description': "Compact representation of an Asset Link Definition's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Binding was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Binding was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}
calculate_asset_statistics = {'tags': ['assets/v2/asset'], 'summary': 'Generate the requested statistics for Assets matching the provided search criteria. (PUBLIC)', 'description': '', 'operationId': 'calculateAssetStatistics', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Parent request for statistics for a set of Assets', 'properties': {'ancestor': {'type': 'list', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups", 'items': {'type': 'str', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups"}, 'uniqueItems': True}, 'asset': {'type': 'list', 'description': 'Search for Assets with these Asset IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Asset IDs or short names'}, 'uniqueItems': True}, 'assetByVulnerability': {'type': 'dict', 'description': 'Search criteria for searching for Assets by Vulnerability properties', 'properties': {'references': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'items': {'type': 'str', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs'}, 'uniqueItems': True}}}, 'assetDefinition': {'type': 'list', 'description': 'Search for Assets with these AssetDefinition IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these AssetDefinition IDs or short names'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Assets by their components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'criticality': {'type': 'list', 'description': 'Search for Assets by Confidentiality, Availability and Integrity.<br />All criteria values must match for an Asset to match. Use sub-criteria for OR query behaviour.', 'items': {'type': 'dict', 'description': 'Search criteria for searching for Assets by their criticality triad properties', 'properties': {'dimension': {'type': 'str', 'description': 'Which criticality dimension to search for', 'enum': ['confidentiality', 'availability', 'integrity']}, 'level': {'type': 'list', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'items': {'type': 'str', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'maxItems': 2147483647, 'minItems': 1, 'uniqueItems': True}}, 'required': ['dimension', 'level']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Search for Assets with these Customer IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Customer IDs or short names'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'fieldAggregation': {'type': 'list', 'description': '  The fields by which to group statistics.\n  The order of fieldAggregations in the request will determine the hierarchy of the aggregations and their buckets.\n', 'items': {'type': 'dict', 'description': 'Request to instruct the service by which field(s) to group Asset statistics', 'properties': {'field': {'type': 'str', 'description': 'The field by which to group Assets when generating statistics.', 'enum': ['customer', 'assetGroup', 'assetDefinition']}, 'limit': {'type': 'int', 'default': 25, 'description': 'The maximum number of distinct categories for the given group field for which to generate statistics', 'minimum': 1}}, 'required': ['field']}}, 'hasGroups': {'type': 'bool', 'description': 'Set to filter on/out Assets belonging to a group'}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether to include deleted assets'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly search for Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly search for Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'includeGroupInfo': {'type': 'bool', 'default': True, 'description': "Whether to include Assets' parent group info in the search response"}, 'keywordFieldStrategy': {'type': 'list', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['all', 'assetAll', 'shortName', 'name', 'description', 'vulnerabilityDefinitionAll', 'references', 'vulnerabilityID', 'vulnerabilityName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match any keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at most this CVSS score', 'minimum': 0}, 'minimumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at least this CVSS score', 'minimum': 0}, 'parent': {'type': 'list', 'description': 'Search for Assets with these parent AssetGroup IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these parent AssetGroup IDs or short names'}, 'uniqueItems': True}, 'rangeMetric': {'type': 'list', 'description': 'The timestamp fields for which to generate statistics.', 'items': {'type': 'dict', 'description': "Request for statistics for Assets filtered by a time range where 'from' is inclusive and 'to' is exclusive", 'properties': {'endTimestamp': {'type': 'str', 'description': 'Filter by this max value. Defaults to NOW  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'field': {'type': 'str', 'description': 'The timestamp field against which to filter', 'enum': ['created', 'deleted', 'lastSeen', 'lastUpdated']}, 'includeEmpty': {'type': 'bool', 'default': False, 'description': 'Whether to return temporal buckets for the entire range regardless of whether they contain valuesMust be used with resolution.'}, 'name': {'type': 'str', 'description': 'The name for this aggregation. Must be unique for the parent statistics request.', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'resolution': {'type': 'str', 'description': 'If set, the temporal resolution by which to sub-aggregate the range metric i.e. by hour, day, week, month, or year', 'enum': ['day', 'hour', 'week', 'month', 'year']}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Filter the by this min value. Defaults to 0 i.e. 1970-01-01.  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}}, 'required': ['field', 'name']}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'statisticsMetric': {'type': 'list', 'description': 'The numerical fields for which to generate metrics.', 'items': {'type': 'dict', 'description': "Request for statistics for a set of Asset's numerical fields", 'properties': {'field': {'type': 'str', 'description': 'The Asset field for which we want statistics', 'enum': ['vulnerabilities', 'cvss']}}, 'required': ['field']}, 'uniqueItems': True}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria that are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Assets', 'properties': {'ancestor': {'type': 'list', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups", 'items': {'type': 'str', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups"}, 'uniqueItems': True}, 'asset': {'type': 'list', 'description': 'Search for Assets with these Asset IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Asset IDs or short names'}, 'uniqueItems': True}, 'assetByVulnerability': {'type': 'dict', 'description': 'Search criteria for searching for Assets by Vulnerability properties', 'properties': {'references': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'items': {'type': 'str', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs'}, 'uniqueItems': True}}}, 'assetDefinition': {'type': 'list', 'description': 'Search for Assets with these AssetDefinition IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these AssetDefinition IDs or short names'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Assets by their components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'criticality': {'type': 'list', 'description': 'Search for Assets by Confidentiality, Availability and Integrity.<br />All criteria values must match for an Asset to match. Use sub-criteria for OR query behaviour.', 'items': {'type': 'dict', 'description': 'Search criteria for searching for Assets by their criticality triad properties', 'properties': {'dimension': {'type': 'str', 'description': 'Which criticality dimension to search for', 'enum': ['confidentiality', 'availability', 'integrity']}, 'level': {'type': 'list', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'items': {'type': 'str', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'maxItems': 2147483647, 'minItems': 1, 'uniqueItems': True}}, 'required': ['dimension', 'level']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Search for Assets with these Customer IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Customer IDs or short names'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria will be excluded'}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'hasGroups': {'type': 'bool', 'description': 'Set to filter on/out Assets belonging to a group'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly search for Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly search for Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'includeGroupInfo': {'type': 'bool', 'default': True, 'description': "Whether to include Assets' parent group info in the search response"}, 'keywordFieldStrategy': {'type': 'list', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['all', 'assetAll', 'shortName', 'name', 'description', 'vulnerabilityDefinitionAll', 'references', 'vulnerabilityID', 'vulnerabilityName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match any keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at most this CVSS score', 'minimum': 0}, 'minimumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at least this CVSS score', 'minimum': 0}, 'parent': {'type': 'list', 'description': 'Search for Assets with these parent AssetGroup IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these parent AssetGroup IDs or short names'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria are required (logical AND-ed together with parent criteria)'}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'assetAll', 'firstSeenTimestamp', 'lastSeenTimestamp', 'lastScanTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'expireTimestamp', 'vulnerabilityAll', 'vulnerabilityResolutionTimestamp', 'vulnerabilityResolutionExpiryTimestamp', 'vulnerabilityFirstSeenTimestamp', 'vulnerabilityLastSeenTimestamp', 'vulnerabilityExpireTimestamp', 'vulnerabilityCreatedTimestamp', 'vulnerabilityLastUpdatedTimestamp', 'vulnerabilityDeletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser', 'owner']}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'assetAll', 'firstSeenTimestamp', 'lastSeenTimestamp', 'lastScanTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'expireTimestamp', 'vulnerabilityAll', 'vulnerabilityResolutionTimestamp', 'vulnerabilityResolutionExpiryTimestamp', 'vulnerabilityFirstSeenTimestamp', 'vulnerabilityLastSeenTimestamp', 'vulnerabilityExpireTimestamp', 'vulnerabilityCreatedTimestamp', 'vulnerabilityLastUpdatedTimestamp', 'vulnerabilityDeletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser', 'owner']}, 'uniqueItems': True}}}}}, 'required': True}, 'responses': {'default': {'description': 'Generate the requested statistics for Assets matching the provided search criteria.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'Container for statistics on data stored by the Asset Service', 'properties': {'buckets': {'type': 'array', 'description': 'Metric buckets below this level, contains MetricRecords.', 'items': {'type': 'object', 'description': 'Container for statistics on data stored by the Asset Service', 'properties': {'buckets': {'type': 'array', 'description': 'Metric buckets below this level, contains MetricRecords.', 'items': []}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents contained within this record sharing the same metadata'}, 'metadata': {'type': 'object', 'description': 'Metadata for a stats aggregation container', 'properties': {'identifier': {'type': 'string', 'description': "The aggregation container's identifier e.g. id or enum value"}, 'name': {'type': 'string', 'description': "The aggregation container's name."}, 'shortName': {'type': 'string', 'description': "The aggregation container's shortName. Depending on the type of metadata can also be used to retrieve the full object."}, 'type': {'type': 'string', 'description': "The aggregation container's type", 'enum': ['asset', 'customer', 'assetGroup', 'assetDefinition', 'vulnerabilityDefinition', 'vulnerabilitySeverity', 'resolution']}}}, 'rangeMetrics': {'type': 'array', 'description': 'Range metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Metric representing the number of documents that matched the timestamp filter for a given time field e.g. assets created within the last 2 weeks', 'properties': {'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'dateHistogramBuckets': {'type': 'array', 'description': 'If Resolution is defined in the request, this contains the resulting temporal groupings', 'items': {'type': 'object', 'description': 'A date histogram bucket', 'properties': {'count': {'type': 'integer', 'format': 'int64'}, 'key': {'type': 'string'}}}}, 'endTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The max for the filter used when calculating the range metric. Corresponds to the endTimestamp defined in the RangeMetricRequest'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'resolution': {'type': 'string', 'description': 'The temporal resolution by which to sub aggregate matching documents e.g. hour, day, week, month, year', 'enum': ['day', 'hour', 'week', 'month', 'year']}, 'startTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The min for the filter used when calculating the range metric. Corresponds to the startTimestamp defined in the RangeMetricRequest'}}}}, 'statisticsMetrics': {'type': 'array', 'description': 'Statistics metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Statistics for a given numerical field for a set of matching documents', 'properties': {'avg': {'type': 'number', 'format': 'double', 'description': 'The average field value of all the documents used to calculate these metrics'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'max': {'type': 'number', 'format': 'double', 'description': 'The maximum field value amongst the documents used to calculate these metrics'}, 'min': {'type': 'number', 'format': 'double', 'description': 'The minimum field value amongst the documents used to calculate these metrics'}, 'sum': {'type': 'number', 'format': 'double', 'description': 'The total field value of all the documents used to calculate these metrics'}}}}}}}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents contained within this record sharing the same metadata'}, 'metadata': {'type': 'object', 'description': 'Metadata for a stats aggregation container', 'properties': {'identifier': {'type': 'string', 'description': "The aggregation container's identifier e.g. id or enum value"}, 'name': {'type': 'string', 'description': "The aggregation container's name."}, 'shortName': {'type': 'string', 'description': "The aggregation container's shortName. Depending on the type of metadata can also be used to retrieve the full object."}, 'type': {'type': 'string', 'description': "The aggregation container's type", 'enum': ['asset', 'customer', 'assetGroup', 'assetDefinition', 'vulnerabilityDefinition', 'vulnerabilitySeverity', 'resolution']}}}, 'rangeMetrics': {'type': 'array', 'description': 'Range metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Metric representing the number of documents that matched the timestamp filter for a given time field e.g. assets created within the last 2 weeks', 'properties': {'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'dateHistogramBuckets': {'type': 'array', 'description': 'If Resolution is defined in the request, this contains the resulting temporal groupings', 'items': {'type': 'object', 'description': 'A date histogram bucket', 'properties': {'count': {'type': 'integer', 'format': 'int64'}, 'key': {'type': 'string'}}}}, 'endTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The max for the filter used when calculating the range metric. Corresponds to the endTimestamp defined in the RangeMetricRequest'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'resolution': {'type': 'string', 'description': 'The temporal resolution by which to sub aggregate matching documents e.g. hour, day, week, month, year', 'enum': ['day', 'hour', 'week', 'month', 'year']}, 'startTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The min for the filter used when calculating the range metric. Corresponds to the startTimestamp defined in the RangeMetricRequest'}}}}, 'statisticsMetrics': {'type': 'array', 'description': 'Statistics metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Statistics for a given numerical field for a set of matching documents', 'properties': {'avg': {'type': 'number', 'format': 'double', 'description': 'The average field value of all the documents used to calculate these metrics'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'max': {'type': 'number', 'format': 'double', 'description': 'The maximum field value amongst the documents used to calculate these metrics'}, 'min': {'type': 'number', 'format': 'double', 'description': 'The minimum field value amongst the documents used to calculate these metrics'}, 'sum': {'type': 'number', 'format': 'double', 'description': 'The total field value of all the documents used to calculate these metrics'}}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
delete_asset = {'tags': ['assets/v2/asset'], 'summary': 'Delete asset (PUBLIC)', 'description': '', 'operationId': 'deleteAsset', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when short name was provided", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that updates to the Asset are immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'responses': {'default': {'description': 'Delete asset', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['removeAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
delete_asset_definition = {'tags': ['assets/v2/asset/definition'], 'summary': 'Marks a AssetDefinition as deleted (INTERNAL)', 'description': '', 'operationId': 'deleteAssetDefinition', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'AssetDefinition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'domain', 'in': 'query', 'description': "Domain ID or name (default is user's domain), only used when definition is short name", 'schema': {'type': 'string'}, 'type': 'str'}], 'responses': {'default': {'description': 'Marks a AssetDefinition as deleted', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Definition determines what kind of components may be attached to Assets of this type', 'properties': {'componentDefinitions': {'type': 'array', 'description': 'Component definitions belong to asset definition', 'items': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset definition'}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': 'Flags of asset definition', 'items': {'type': 'string', 'description': 'Flags of asset definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was last updated'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['deleteAssetDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
delete_asset_link = {'tags': ['assets/v2/asset/link'], 'summary': "Delete an existing active Asset Link identified by its id and origin Asset's id or shortname (DEV)", 'description': '', 'operationId': 'deleteAssetLink', 'parameters': [{'name': 'asset', 'in': 'path', 'description': 'Asset link origin Asset id or shortname', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when asset short name was provided", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'link', 'in': 'path', 'description': 'Asset Link id', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}], 'responses': {'default': {'description': "Delete an existing active Asset Link identified by its id and origin Asset's id or shortname", 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A link between two active Assets', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was created'}, 'definition': {'type': 'object', 'description': "Compact representation of an Asset Link Definition's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was deleted'}, 'expiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link is scheduled to expire. 0 means never.'}, 'flags': {'type': 'array', 'items': {'type': 'string', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was lastUpdated'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'How long the link is expected to live. Updates not specifying a new ttl will reuse this value. 0 means forever.'}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAsset', 'updateAsset'], 'x-Argus-ApiStatus': 'DEV'}
delete_asset_link_binding = {'tags': ['assets/v2/asset/link/binding'], 'summary': 'Delete an active AssetLinkBinding from an AssetLinkDefinition (TEST)', 'description': '', 'operationId': 'deleteAssetLinkBinding', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'Asset Link Definition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'binding', 'in': 'path', 'description': 'Asset Link Binding ID', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}], 'responses': {'default': {'description': 'Delete an active AssetLinkBinding from an AssetLinkDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'assetLinkDefinition': {'type': 'object', 'description': "Compact representation of an Asset Link Definition's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Binding was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Binding was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}
delete_asset_link_definition = {'tags': ['assets/v2/asset/link/definition'], 'summary': 'Delete an existing active Asset Link Definition identified by its ID or shortName (TEST)', 'description': '', 'operationId': 'deleteAssetLinkDefinition', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'Asset Link Definition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'responses': {'default': {'description': 'Delete an existing active Asset Link Definition identified by its ID or shortName', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Link Definition describes and restricts relationships between specific types of Assets.', 'properties': {'bindings': {'type': 'array', 'description': 'Asset Link Bindings belonging to this Asset Link Definition.', 'items': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Definition', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was last updated.'}, 'localizedDescription': {'type': 'string', 'description': "Description of the Asset Link Definition based on the current user's language settings. Defaults to English if a localised version is unavailable."}, 'localizedDescriptions': {'type': 'object', 'additionalProperties': {'type': 'string', 'description': 'All available descriptions of the Asset Link Definition'}, 'description': 'All available descriptions of the Asset Link Definition'}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewDeletedAssetLinkDefinition', 'updateAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}
get_asset = {'tags': ['assets/v2/asset'], 'summary': 'Get Asset (PUBLIC)', 'description': '', 'operationId': 'getAsset', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when the asset's short name was provided to retrieve the asset", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'includeDeletedComponents', 'in': 'query', 'description': 'Include deleted components in the response', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'responses': {'default': {'description': 'Get Asset', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
get_asset_ancestors = {'tags': ['assets/v2/asset'], 'summary': 'Get Asset Ancestors (PUBLIC)', 'description': '', 'operationId': 'getAssetAncestors', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when the asset's short name was provided to retrieve the asset", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'offset', 'in': 'query', 'description': 'How many Asset Ancestors to skip, defaults to 0', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'How many Asset Ancestors to return, defaults to all', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}], 'responses': {'default': {'description': 'Get Asset Ancestors', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset Group represents an arbitrary collection of Assets or subgroups', 'properties': {'assetCount': {'type': 'integer', 'format': 'int32', 'description': 'Count of undeleted member assets in the asset group or in descendant groups'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset group'}, 'filter': {'type': 'object', 'description': 'An Info object for dynamic group filter ', 'properties': {'description': {'type': 'string', 'description': 'Description of the filter'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'Filter id'}, 'name': {'type': 'string', 'description': 'Name of the filter'}, 'shortName': {'type': 'string', 'description': 'ShortName of the filter'}, 'usersGrantingAccess': {'type': 'array', 'description': 'Users with permissions to grant access to the filter. Apart from users listed in this field admins can also grant access to the filters.', 'items': {'type': 'object', 'description': 'SPI SubjectInfo', 'discriminator': {'propertyName': 'type'}, 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'uniqueItems': True}}}, 'flags': {'type': 'array', 'description': 'Flags of the asset group', 'items': {'type': 'string', 'description': 'Flags of the asset group', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'canContainAssets', 'dynamicGroup', 'neverDynamic']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastDynamicallyUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was last updated'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was last updated'}, 'memberCount': {'type': 'integer', 'format': 'int32', 'description': 'Count of undeleted member assets or groups in the asset group'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset group', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset group, summed up of attached assets' cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset group, summed up count of attached assets' vulnerabilities"}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
get_asset_component = {'tags': ['assets/v2/asset'], 'summary': 'Get Asset Component (INTERNAL)', 'description': '', 'operationId': 'getAssetComponent', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when the asset's short name was provided to retrieve the asset", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'id', 'in': 'path', 'description': 'Asset Component ID', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}], 'responses': {'default': {'description': 'Get Asset Component', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'INTERNAL'}
get_asset_components = {'tags': ['assets/v2/asset'], 'summary': ' Returns Asset Components in the order in which they were created. Supports pagination i.e. offset and limit and filtering deleted components (INTERNAL)', 'description': '', 'operationId': 'getAssetComponents', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when the asset's short name was provided to retrieve the asset", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Whether to include deleted components', 'schema': {'type': 'boolean'}, 'type': 'bool'}, {'name': 'offset', 'in': 'query', 'description': 'How many components to skip', 'schema': {'type': 'int'}, 'type': 'int'}, {'name': 'limit', 'in': 'query', 'description': 'How many components to return', 'schema': {'type': 'int'}, 'type': 'int'}], 'responses': {'default': {'description': '  Returns Asset Components in the order in which they were created.\n  Supports pagination i.e. offset and limit and filtering deleted components\n', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'INTERNAL'}
get_asset_definition = {'tags': ['assets/v2/asset/definition'], 'summary': 'Get AssetDefinition (INTERNAL)', 'description': '', 'operationId': 'getAssetDefinition', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'AssetDefinition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'domain', 'in': 'query', 'description': "Domain ID or name (default is user's domain), only used when definition is short name", 'schema': {'type': 'string'}, 'type': 'str'}], 'responses': {'default': {'description': 'Get AssetDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Definition determines what kind of components may be attached to Assets of this type', 'properties': {'componentDefinitions': {'type': 'array', 'description': 'Component definitions belong to asset definition', 'items': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset definition'}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': 'Flags of asset definition', 'items': {'type': 'string', 'description': 'Flags of asset definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was last updated'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssetDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
get_asset_link_definition = {'tags': ['assets/v2/asset/link/definition'], 'summary': 'Get an AssetLinkDefinition and its active (non-deleted) bindings (TEST)', 'description': '', 'operationId': 'getAssetLinkDefinition', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'AssetLinkDefinition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'responses': {'default': {'description': 'Get an AssetLinkDefinition and its active (non-deleted) bindings', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Link Definition describes and restricts relationships between specific types of Assets.', 'properties': {'bindings': {'type': 'array', 'description': 'Asset Link Bindings belonging to this Asset Link Definition.', 'items': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Definition', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was last updated.'}, 'localizedDescription': {'type': 'string', 'description': "Description of the Asset Link Definition based on the current user's language settings. Defaults to English if a localised version is unavailable."}, 'localizedDescriptions': {'type': 'object', 'additionalProperties': {'type': 'string', 'description': 'All available descriptions of the Asset Link Definition'}, 'description': 'All available descriptions of the Asset Link Definition'}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}
get_asset_link_definition_bindings = {'tags': ['assets/v2/asset/link/definition'], 'summary': 'Get AssetLinkBindings for a specific AssetLinkDefinition (TEST)', 'description': '', 'operationId': 'getAssetLinkDefinitionBindings', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'AssetLinkDefinition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'limit', 'in': 'query', 'description': 'Limit results', 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'offset', 'in': 'query', 'description': 'Offset results.', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Include deleted bindings in the response', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'Get AssetLinkBindings for a specific AssetLinkDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'assetLinkDefinition': {'type': 'object', 'description': "Compact representation of an Asset Link Definition's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Binding was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Binding was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}
get_asset_parents = {'tags': ['assets/v2/asset'], 'summary': 'Get Asset Parents (PUBLIC)', 'description': '', 'operationId': 'getAssetParents', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when the asset's short name was provided to retrieve the asset", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'offset', 'in': 'query', 'description': 'How many Assets Parents to skip, defaults to 0', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'How many Assets Parents to return, defaults to all', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}], 'responses': {'default': {'description': 'Get Asset Parents', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset Group represents an arbitrary collection of Assets or subgroups', 'properties': {'assetCount': {'type': 'integer', 'format': 'int32', 'description': 'Count of undeleted member assets in the asset group or in descendant groups'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset group'}, 'filter': {'type': 'object', 'description': 'An Info object for dynamic group filter ', 'properties': {'description': {'type': 'string', 'description': 'Description of the filter'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'Filter id'}, 'name': {'type': 'string', 'description': 'Name of the filter'}, 'shortName': {'type': 'string', 'description': 'ShortName of the filter'}, 'usersGrantingAccess': {'type': 'array', 'description': 'Users with permissions to grant access to the filter. Apart from users listed in this field admins can also grant access to the filters.', 'items': {'type': 'object', 'description': 'SPI SubjectInfo', 'discriminator': {'propertyName': 'type'}, 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'uniqueItems': True}}}, 'flags': {'type': 'array', 'description': 'Flags of the asset group', 'items': {'type': 'string', 'description': 'Flags of the asset group', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'canContainAssets', 'dynamicGroup', 'neverDynamic']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastDynamicallyUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was last updated'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset group was last updated'}, 'memberCount': {'type': 'integer', 'format': 'int32', 'description': 'Count of undeleted member assets or groups in the asset group'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset group', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset group, summed up of attached assets' cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset group, summed up count of attached assets' vulnerabilities"}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
get_vulnerability_by_asset = {'tags': ['assets/v2/vulnerability'], 'summary': 'Get Vulnerabilities observed on the requested Asset. (PUBLIC)', 'description': '', 'operationId': 'getVulnerabilityByAsset', 'parameters': [{'name': 'asset', 'in': 'path', 'description': 'Asset ID or short name.', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer) to help resolve asset, only used when asset's short name was provided.", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'includeRawOutput', 'in': 'query', 'description': 'Whether to include raw output, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Whether to include deleted vulnerabilities, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'offset', 'in': 'query', 'description': 'How many vulnerabilities to skip, defaults to 0', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'How many vulnerabilities to return, defaults to all of them', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'skipCount', 'in': 'query', 'description': 'Whether to skip counting vulnerabilities and simply return them', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'Get Vulnerabilities observed on the requested Asset.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'deprecated': True, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
get_vulnerability_by_asset_and_vulnerability_id = {'tags': ['assets/v2/vulnerability'], 'summary': 'Get Vulnerabilities observed on the requested Asset with the same Vulnerability Definition (PUBLIC)', 'description': '', 'operationId': 'getVulnerabilityByAssetAndVulnerabilityID', 'parameters': [{'name': 'asset', 'in': 'path', 'description': 'Asset ID or short name.', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'vulnerabilityID', 'in': 'path', 'description': 'Identifier of vulnerability (e.g. CVE-2015-3429)', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer) to help resolve asset, only used when asset's short name was provided.", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'includeRawOutput', 'in': 'query', 'description': 'Whether to include raw output, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Whether to include deleted vulnerabilities, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'offset', 'in': 'query', 'description': 'How many Vulnerabilities to skip, defaults to 0', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'How many Vulnerabilities to return, defaults to all', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'skipCount', 'in': 'query', 'description': 'Whether to skip counting vulnerabilities and simply return them', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'Get Vulnerabilities observed on the requested Asset with the same Vulnerability Definition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'deprecated': True, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
list_asset_link_definitions = {'tags': ['assets/v2/asset/link/definition'], 'summary': 'List AssetLinkDefinitions (TEST)', 'description': '', 'operationId': 'listAssetLinkDefinitions', 'parameters': [{'name': 'definition', 'in': 'query', 'description': 'AssetLinkDefinition ID or short name', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'limit', 'in': 'query', 'description': 'Limit results', 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'offset', 'in': 'query', 'description': 'Offset results.', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Include deleted definition in the response', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'List AssetLinkDefinitions', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset Link Definition describes and restricts relationships between specific types of Assets.', 'properties': {'bindings': {'type': 'array', 'description': 'Asset Link Bindings belonging to this Asset Link Definition.', 'items': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Definition', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was last updated.'}, 'localizedDescription': {'type': 'string', 'description': "Description of the Asset Link Definition based on the current user's language settings. Defaults to English if a localised version is unavailable."}, 'localizedDescriptions': {'type': 'object', 'additionalProperties': {'type': 'string', 'description': 'All available descriptions of the Asset Link Definition'}, 'description': 'All available descriptions of the Asset Link Definition'}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}
list_asset_links = {'tags': ['assets/v2/asset/link'], 'summary': 'List AssetLinks (DEV)', 'description': '', 'operationId': 'listAssetLinks', 'parameters': [{'name': 'asset', 'in': 'path', 'description': 'Origin Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when the asset's short name was provided to retrieve the asset", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'limit', 'in': 'query', 'description': 'Limit results', 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'offset', 'in': 'query', 'description': 'Offset results.', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Include deleted links in the response', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'List AssetLinks', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A link between two active Assets', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was created'}, 'definition': {'type': 'object', 'description': "Compact representation of an Asset Link Definition's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was deleted'}, 'expiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link is scheduled to expire. 0 means never.'}, 'flags': {'type': 'array', 'items': {'type': 'string', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was lastUpdated'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'How long the link is expected to live. Updates not specifying a new ttl will reuse this value. 0 means forever.'}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'DEV'}
list_asset_vulnerabilities = {'tags': ['assets/v2/vulnerability'], 'summary': 'Get Vulnerabilities observed on the requested Asset. (PUBLIC)', 'description': '', 'operationId': 'getVulnerabilitiesByAsset', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name.', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer) to help resolve asset, only used when asset's short name was provided.", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'includeRawOutput', 'in': 'query', 'description': 'Whether to include raw output, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Whether to include deleted vulnerabilities, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'offset', 'in': 'query', 'description': 'How many vulnerabilities to skip, defaults to 0', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'How many vulnerabilities to return, defaults to all of them', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'skipCount', 'in': 'query', 'description': 'Whether to skip counting vulnerabilities and simply return them', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'Get Vulnerabilities observed on the requested Asset.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
list_asset_vulnerabilities_by_id = {'tags': ['assets/v2/vulnerability'], 'summary': 'Get Vulnerabilities observed on the requested Asset with the same Vulnerability Definition (PUBLIC)', 'description': '', 'operationId': 'getVulnerabilitiesByAssetAndVulnerabilityID', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name.', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'vulnerabilityID', 'in': 'path', 'description': 'Identifier of vulnerability (e.g. CVE-2015-3429)', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer) to help resolve asset, only used when asset's short name was provided.", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'includeRawOutput', 'in': 'query', 'description': 'Whether to include raw output, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Whether to include deleted vulnerabilities, not included by default', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'offset', 'in': 'query', 'description': 'How many Vulnerabilities to skip, defaults to 0', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'How many Vulnerabilities to return, defaults to all', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'skipCount', 'in': 'query', 'description': 'Whether to skip counting vulnerabilities and simply return them', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'Get Vulnerabilities observed on the requested Asset with the same Vulnerability Definition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
list_assets = {'tags': ['assets/v2/asset'], 'summary': "Basic Asset search. If keywords are present, they must all must be present in at least one of the keyword fields. Check the documentation for the KeywordFieldStrategy field for more details. If start/end timestamps are present, the range must be valid for at least one of the requested time stamp fields. Check the documentation for the TimestampFieldStrategy field for more details. Limit and offset added together cannot exceed 10'000 (PUBLIC)", 'description': '', 'operationId': 'listAssets', 'parameters': [{'name': 'customer', 'in': 'query', 'description': 'Customer ids or short names', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'asset', 'in': 'query', 'description': 'Asset ids or short names', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'parent', 'in': 'query', 'description': 'Asset parent group ids or short names', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'dataSource', 'in': 'query', 'description': 'Data source, identified by id or shortname, that detected the Asset', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'keywords', 'in': 'query', 'description': 'Limit search by asset field (short name, name, and description) keywords', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'limit', 'in': 'query', 'description': "Limit results. The sum of limit and offset must be <= 10'000 when not streaming", 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'offset', 'in': 'query', 'description': "Offset results. The sum of limit and offset must be <= 10'000 when not streaming", 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Include deleted', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'startTimestamp', 'in': 'query', 'description': 'Lower bound timestamp filter value', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'endTimestamp', 'in': 'query', 'description': 'Upper bound timestamp filter value', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'timeFieldStrategy', 'in': 'query', 'description': 'Which time field(s) to filter by timestamp start/end', 'schema': {'type': 'list', 'default': 'assetAll', 'items': {'type': 'str', 'enum': ['all', 'assetAll', 'firstSeenTimestamp', 'lastSeenTimestamp', 'lastScanTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'expireTimestamp', 'vulnerabilityAll', 'vulnerabilityResolutionTimestamp', 'vulnerabilityResolutionExpiryTimestamp', 'vulnerabilityFirstSeenTimestamp', 'vulnerabilityLastSeenTimestamp', 'vulnerabilityExpireTimestamp', 'vulnerabilityCreatedTimestamp', 'vulnerabilityLastUpdatedTimestamp', 'vulnerabilityDeletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'type': 'list', 'default': 'assetAll', 'items': {'type': 'str', 'enum': ['all', 'assetAll', 'firstSeenTimestamp', 'lastSeenTimestamp', 'lastScanTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'expireTimestamp', 'vulnerabilityAll', 'vulnerabilityResolutionTimestamp', 'vulnerabilityResolutionExpiryTimestamp', 'vulnerabilityFirstSeenTimestamp', 'vulnerabilityLastSeenTimestamp', 'vulnerabilityExpireTimestamp', 'vulnerabilityCreatedTimestamp', 'vulnerabilityLastUpdatedTimestamp', 'vulnerabilityDeletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, {'name': 'sortBy', 'in': 'query', 'description': 'Field(s) to sort by - refer to the search POST endpoint to see which fields are permitted', 'schema': {'type': 'list', 'items': {'type': 'string'}}, 'type': 'list', 'items': {'type': 'string'}}], 'responses': {'default': {'description': "Basic Asset search.\nIf keywords are present, they must all must be present in at least one of the keyword fields.\nCheck the documentation for the KeywordFieldStrategy field for more details.\nIf start/end timestamps are present, the range must be valid for at least one of the requested time stamp fields.\nCheck the documentation for the TimestampFieldStrategy field for more details.\nLimit and offset added together cannot exceed 10'000\n", 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
recalculate_asset = {'tags': ['assets/v2/asset'], 'summary': "Force a recalculation of an Asset's vulnerabilityCount and totalCVSS fields.NB: any changes are propagated up the Asset Group hierarchy if recalculateGroups is set and the Asset belongs to any group(s). (INTERNAL)", 'description': '', 'operationId': 'recalculateAsset', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when asset short name was provided", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'recalculateGroups', 'in': 'query', 'description': 'Whether to recalculate counts for groups in the hierarchy above the affected asset', 'schema': {'type': 'boolean'}, 'type': 'bool'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh making the recalculated asset immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'responses': {'default': {'description': "Force a recalculation of an Asset's vulnerabilityCount and totalCVSS fields.NB: any changes are propagated up the Asset Group hierarchy if recalculateGroups is set and the Asset belongs to any group(s).", 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['recalculateAsset'], 'x-Argus-ApiStatus': 'INTERNAL'}
refresh_asset_link = {'tags': ['assets/v2/asset/link'], 'summary': "Refresh an existing active Asset Link identified by its id and origin Asset's id or shortname (DEV)", 'description': '', 'operationId': 'refreshAssetLink', 'parameters': [{'name': 'asset', 'in': 'path', 'description': 'Asset link origin Asset id or shortname', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when asset short name was provided", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'link', 'in': 'path', 'description': 'Asset Link id', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}], 'requestBody': {'content': {'*/*': {'schema': {'type': 'object', 'description': 'Request to refresh an existing active Asset Link between two active Assets', 'properties': {'ttl': {'type': 'int', 'description': 'How long the link is considered valid. If set will override the existing TTL on the link.', 'minimum': 0}}}}}, 'required': True}, 'responses': {'default': {'description': "Refresh an existing active Asset Link identified by its id and origin Asset's id or shortname", 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A link between two active Assets', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was created'}, 'definition': {'type': 'object', 'description': "Compact representation of an Asset Link Definition's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was deleted'}, 'expiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link is scheduled to expire. 0 means never.'}, 'flags': {'type': 'array', 'items': {'type': 'string', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the link was lastUpdated'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'How long the link is expected to live. Updates not specifying a new ttl will reuse this value. 0 means forever.'}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAsset', 'updateAsset'], 'x-Argus-ApiStatus': 'DEV'}
reindex_asset = {'tags': ['assets/v2/asset/bulk'], 'summary': 'Re-indexes specified Assets (INTERNAL)', 'description': '', 'operationId': 'reindexAsset', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to reindex a set of Assets', 'properties': {'assetID': {'type': 'list', 'description': 'A set of asset IDs to be re-indexed', 'items': {'type': 'str', 'description': 'A set of asset IDs to be re-indexed'}, 'maxItems': 1000, 'minItems': 0, 'uniqueItems': True}, 'recursive': {'type': 'str', 'default': 'none', 'description': 'Enumerated type of recursion strategy(none - reindex just assets, all - reindex assets and associated vulnerabilities', 'enum': ['none', 'all']}}, 'required': ['recursive']}}}, 'required': True}, 'responses': {'default': {'description': 'Re-indexes specified Assets', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'Response containing information on the results of a reindexing request', 'properties': {'notFoundAssetGroupID': {'type': 'array', 'description': 'A set of asset group IDs that were not found', 'items': {'type': 'string', 'format': 'uuid', 'description': 'A set of asset group IDs that were not found'}, 'uniqueItems': True}, 'notFoundAssetID': {'type': 'array', 'description': 'A set of asset IDs that were not found', 'items': {'type': 'string', 'format': 'uuid', 'description': 'A set of asset IDs that were not found'}, 'uniqueItems': True}, 'reindexedAssetCount': {'type': 'integer', 'format': 'int64', 'description': 'Count of re-indexed assets'}, 'reindexedAssetGroupCount': {'type': 'integer', 'format': 'int64', 'description': 'Count of re-indexed asset groups'}, 'reindexedVulnerabilityCount': {'type': 'integer', 'format': 'int64', 'description': 'Count of re-indexed vulnerabilities'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['reindexAsset'], 'x-Argus-ApiStatus': 'INTERNAL'}
search_asset_definitions = {'tags': ['assets/v2/asset/definition'], 'summary': 'Search AssetDefinitions with specified criteria (INTERNAL)', 'description': '', 'operationId': 'searchAssetDefinitions', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Search criteria for searching for Asset Definitions', 'properties': {'definition': {'type': 'list', 'description': 'Restrict to specified Asset definitions (ID or short name)', 'items': {'type': 'str', 'description': 'Restrict to specified Asset definitions (ID or short name)'}, 'uniqueItems': True}, 'domain': {'type': 'list', 'description': 'Restrict to specified domains (domain ID or name)', 'items': {'type': 'str', 'description': 'Restrict to specified domains (domain ID or name)'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether include deleted results'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['shortName', 'name', 'componentShortName', 'componentName', 'all']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'all', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by keywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by keywordFieldStrategy'}, 'uniqueItems': True}, 'limit': {'type': 'int', 'default': 25, 'description': 'Limit maximum amount of results', 'minimum': 0}, 'offset': {'type': 'int', 'default': 0, 'description': 'Skip specified amount of results', 'minimum': 0}, 'order': {'type': 'list', 'items': {'type': 'string'}}, 'sortBy': {'type': 'list', 'description': 'Field(s) by which to sort the results', 'items': {'type': 'str', 'description': 'Field(s) by which to sort the results', 'enum': ['shortName', 'name', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', '-shortName', '-name', '-createdTimestamp', '-lastUpdatedTimestamp', '-deletedTimestamp']}}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria which are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Asset Definitions', 'properties': {'definition': {'type': 'list', 'description': 'Restrict to specified Asset definitions (ID or short name)', 'items': {'type': 'str', 'description': 'Restrict to specified Asset definitions (ID or short name)'}, 'uniqueItems': True}, 'domain': {'type': 'list', 'description': 'Restrict to specified domains (domain ID or name)', 'items': {'type': 'str', 'description': 'Restrict to specified domains (domain ID or name)'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching this criteria will be excluded'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['shortName', 'name', 'componentShortName', 'componentName', 'all']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'all', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by keywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by keywordFieldStrategy'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching this criteria are required (logical AND-ed together with parent criteria)'}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for Asset Definitions by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for Asset Definitions by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for Asset Definitions by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for Asset Definitions by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}}}}}, 'required': True}, 'responses': {'default': {'description': 'Search AssetDefinitions with specified criteria', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset Definition determines what kind of components may be attached to Assets of this type', 'properties': {'componentDefinitions': {'type': 'array', 'description': 'Component definitions belong to asset definition', 'items': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset definition'}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': 'Flags of asset definition', 'items': {'type': 'string', 'description': 'Flags of asset definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was last updated'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssetDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
search_asset_definitions_simplified = {'tags': ['assets/v2/asset/definition'], 'summary': 'Search AssetDefinitions with specified query parameters (INTERNAL)', 'description': '', 'operationId': 'searchAssetDefinitionsSimplified', 'parameters': [{'name': 'offset', 'in': 'query', 'description': 'Skip a number of results', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'Maximum number of returned results', 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'keywords', 'in': 'query', 'description': 'Search by keywords', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'keywordField', 'in': 'query', 'description': 'Set field strategy for keyword search', 'schema': {'type': 'list', 'default': 'all', 'items': {'type': 'str', 'enum': ['shortName', 'name', 'componentShortName', 'componentName', 'all']}, 'uniqueItems': True}, 'type': 'list', 'default': 'all', 'items': {'type': 'str', 'enum': ['shortName', 'name', 'componentShortName', 'componentName', 'all']}, 'uniqueItems': True}, {'name': 'keywordMatch', 'in': 'query', 'description': 'Set match strategy for keyword search', 'schema': {'type': 'str', 'default': 'all', 'enum': ['any', 'all']}, 'type': 'str', 'default': 'all', 'enum': ['any', 'all']}, {'name': 'startTimestamp', 'in': 'query', 'description': 'Start timestamp for time range search', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'endTimestamp', 'in': 'query', 'description': 'End timestamp for time range search', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'timeField', 'in': 'query', 'description': 'Set field strategy for time range search', 'schema': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'items': {'type': 'str', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, 'type': 'list', 'default': 'lastUpdatedTimestamp', 'items': {'type': 'str', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, {'name': 'timeMatch', 'in': 'query', 'description': 'Set match strategy for time range search', 'schema': {'type': 'str', 'default': 'any', 'enum': ['any', 'all']}, 'type': 'str', 'default': 'any', 'enum': ['any', 'all']}, {'name': 'definition', 'in': 'query', 'description': 'Search by definitions (ID or short name)', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'domain', 'in': 'query', 'description': 'Search by domains (domain ID or name)', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'sortBy', 'in': 'query', 'description': 'Sort search result', 'schema': {'type': 'list', 'items': {'type': 'string'}}, 'type': 'list', 'items': {'type': 'string'}}], 'responses': {'default': {'description': 'Search AssetDefinitions with specified query parameters', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset Definition determines what kind of components may be attached to Assets of this type', 'properties': {'componentDefinitions': {'type': 'array', 'description': 'Component definitions belong to asset definition', 'items': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset definition'}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': 'Flags of asset definition', 'items': {'type': 'string', 'description': 'Flags of asset definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was last updated'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssetDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
search_assets = {'tags': ['assets/v2/asset'], 'summary': 'Search Assets (PUBLIC)', 'description': '', 'operationId': 'searchAssets', 'parameters': [{'name': 'Accept', 'in': 'header', 'schema': {'type': 'string'}}, {'name': 'exportFields', 'in': 'query', 'description': 'Select order of and fields to export.', 'schema': {'type': 'list', 'items': {'type': 'str', 'enum': ['id', 'shortName', 'name', 'description', 'customerName', 'ownerName', 'assetDefinitionName', 'components', 'parentGroupsNames', 'availability', 'confidentiality', 'integrity', 'totalCvss', 'vulnerabilitiesCount', 'firstSeenTimestamp', 'firstSeenByUser', 'lastSeenTimestamp', 'lastSeenByUser', 'lastScanTimestamp', 'ttl', 'createdTimestamp', 'createdByUser', 'lastUpdatedTimestamp', 'lastUpdatedByUser', 'deletedTimestamp', 'deletedByUser', 'flags', 'exploitAvailable', 'vulnerabilityStatus']}}, 'type': 'list', 'items': {'type': 'str', 'enum': ['id', 'shortName', 'name', 'description', 'customerName', 'ownerName', 'assetDefinitionName', 'components', 'parentGroupsNames', 'availability', 'confidentiality', 'integrity', 'totalCvss', 'vulnerabilitiesCount', 'firstSeenTimestamp', 'firstSeenByUser', 'lastSeenTimestamp', 'lastSeenByUser', 'lastScanTimestamp', 'ttl', 'createdTimestamp', 'createdByUser', 'lastUpdatedTimestamp', 'lastUpdatedByUser', 'deletedTimestamp', 'deletedByUser', 'flags', 'exploitAvailable', 'vulnerabilityStatus']}}, {'name': 'exportComponents', 'in': 'query', 'description': 'Select which component values to include in the export.', 'schema': {'type': 'list', 'items': {'type': 'string'}}, 'type': 'list', 'items': {'type': 'string'}}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'ancestor': {'type': 'list', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups", 'items': {'type': 'str', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups"}, 'uniqueItems': True}, 'asset': {'type': 'list', 'description': 'Search for Assets with these Asset IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Asset IDs or short names'}, 'uniqueItems': True}, 'assetByVulnerability': {'type': 'dict', 'description': 'Search criteria for searching for Assets by Vulnerability properties', 'properties': {'references': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'items': {'type': 'str', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs'}, 'uniqueItems': True}}}, 'assetDefinition': {'type': 'list', 'description': 'Search for Assets with these AssetDefinition IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these AssetDefinition IDs or short names'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Assets by their components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'criticality': {'type': 'list', 'description': 'Search for Assets by Confidentiality, Availability and Integrity.<br />All criteria values must match for an Asset to match. Use sub-criteria for OR query behaviour.', 'items': {'type': 'dict', 'description': 'Search criteria for searching for Assets by their criticality triad properties', 'properties': {'dimension': {'type': 'str', 'description': 'Which criticality dimension to search for', 'enum': ['confidentiality', 'availability', 'integrity']}, 'level': {'type': 'list', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'items': {'type': 'str', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'maxItems': 2147483647, 'minItems': 1, 'uniqueItems': True}}, 'required': ['dimension', 'level']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Search for Assets with these Customer IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Customer IDs or short names'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'hasGroups': {'type': 'bool', 'description': 'Set to filter on/out Assets belonging to a group'}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether to include deleted Assets'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly search for Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly search for Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'includeGroupInfo': {'type': 'bool', 'default': True, 'description': "Whether to include Assets' parent group info in the search response"}, 'keywordFieldStrategy': {'type': 'list', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['all', 'assetAll', 'shortName', 'name', 'description', 'vulnerabilityDefinitionAll', 'references', 'vulnerabilityID', 'vulnerabilityName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match any keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'limit': {'type': 'int', 'default': 25, 'description': 'Limit the results to the specified amount', 'minimum': 0}, 'maximumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at most this CVSS score', 'minimum': 0}, 'minimumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at least this CVSS score', 'minimum': 0}, 'offset': {'type': 'int', 'default': 0, 'description': 'Skip specified amount of results', 'minimum': 0}, 'order': {'type': 'list', 'items': {'type': 'string'}}, 'parent': {'type': 'list', 'description': 'Search for Assets with these parent AssetGroup IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these parent AssetGroup IDs or short names'}, 'uniqueItems': True}, 'sortBy': {'type': 'list', 'description': 'Field(s) by which to sort the results', 'items': {'type': 'str', 'description': 'Field(s) by which to sort the results', 'enum': ['id', 'customerID', 'shortName', 'name', 'ownerID', 'firstSeenTimestamp', 'lastSeenTimestamp', 'lastScanTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'totalCVSS', 'vulnerabilitiesCount', '-id', '-customerID', '-shortName', '-name', '-ownerID', '-firstSeenTimestamp', '-lastSeenTimestamp', '-lastScanTimestamp', '-createdTimestamp', '-lastUpdatedTimestamp', '-deletedTimestamp', '-totalCVSS', '-vulnerabilitiesCount']}}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria that are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Assets', 'properties': {'ancestor': {'type': 'list', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups", 'items': {'type': 'str', 'description': "Search for Assets in the hierarchy below this ancestor AssetGroup identified by its ID or short name.<br />This will resolve all descendant Asset Groups and expand the search to return Assets belonging to them in the hierarchy below these 'ancestor' groups"}, 'uniqueItems': True}, 'asset': {'type': 'list', 'description': 'Search for Assets with these Asset IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Asset IDs or short names'}, 'uniqueItems': True}, 'assetByVulnerability': {'type': 'dict', 'description': 'Search criteria for searching for Assets by Vulnerability properties', 'properties': {'references': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these references in their definition'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'items': {'type': 'str', 'default': 'unresolved', 'description': 'Filter by Assets affected by Vulnerabilities with these Resolution statuses', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs', 'items': {'type': 'str', 'description': 'Filter by Assets affected by Vulnerabilities with these vulnerabilityIDs'}, 'uniqueItems': True}}}, 'assetDefinition': {'type': 'list', 'description': 'Search for Assets with these AssetDefinition IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these AssetDefinition IDs or short names'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Assets by their components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'criticality': {'type': 'list', 'description': 'Search for Assets by Confidentiality, Availability and Integrity.<br />All criteria values must match for an Asset to match. Use sub-criteria for OR query behaviour.', 'items': {'type': 'dict', 'description': 'Search criteria for searching for Assets by their criticality triad properties', 'properties': {'dimension': {'type': 'str', 'description': 'Which criticality dimension to search for', 'enum': ['confidentiality', 'availability', 'integrity']}, 'level': {'type': 'list', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'items': {'type': 'str', 'description': 'Which level(s) to filter for specified dimension (joined with OR)', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'maxItems': 2147483647, 'minItems': 1, 'uniqueItems': True}}, 'required': ['dimension', 'level']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Search for Assets with these Customer IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these Customer IDs or short names'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for Assets detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria will be excluded'}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'hasGroups': {'type': 'bool', 'description': 'Set to filter on/out Assets belonging to a group'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly search for Assets that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly search for Assets that contain these flags', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'includeGroupInfo': {'type': 'bool', 'default': True, 'description': "Whether to include Assets' parent group info in the search response"}, 'keywordFieldStrategy': {'type': 'list', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'assetAll', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['all', 'assetAll', 'shortName', 'name', 'description', 'vulnerabilityDefinitionAll', 'references', 'vulnerabilityID', 'vulnerabilityName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match any keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at most this CVSS score', 'minimum': 0}, 'minimumTotalCvss': {'type': 'float', 'description': 'Search for Assets with at least this CVSS score', 'minimum': 0}, 'parent': {'type': 'list', 'description': 'Search for Assets with these parent AssetGroup IDs or short names', 'items': {'type': 'str', 'description': 'Search for Assets with these parent AssetGroup IDs or short names'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria are required (logical AND-ed together with parent criteria)'}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp).  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'assetAll', 'firstSeenTimestamp', 'lastSeenTimestamp', 'lastScanTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'expireTimestamp', 'vulnerabilityAll', 'vulnerabilityResolutionTimestamp', 'vulnerabilityResolutionExpiryTimestamp', 'vulnerabilityFirstSeenTimestamp', 'vulnerabilityLastSeenTimestamp', 'vulnerabilityExpireTimestamp', 'vulnerabilityCreatedTimestamp', 'vulnerabilityLastUpdatedTimestamp', 'vulnerabilityDeletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser', 'owner']}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'assetAll', 'firstSeenTimestamp', 'lastSeenTimestamp', 'lastScanTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'expireTimestamp', 'vulnerabilityAll', 'vulnerabilityResolutionTimestamp', 'vulnerabilityResolutionExpiryTimestamp', 'vulnerabilityFirstSeenTimestamp', 'vulnerabilityLastSeenTimestamp', 'vulnerabilityExpireTimestamp', 'vulnerabilityCreatedTimestamp', 'vulnerabilityLastUpdatedTimestamp', 'vulnerabilityDeletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for Assets by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser', 'owner']}, 'uniqueItems': True}}}}}, 'required': True}, 'responses': {'default': {'description': 'Search Assets', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}, 'text/csv': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
update_asset = {'tags': ['assets/v2/asset'], 'summary': 'Updates an existing Asset (PUBLIC)', 'description': '', 'operationId': 'updateAsset', 'parameters': [{'name': 'idOrShortName', 'in': 'path', 'description': 'Asset ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'customer', 'in': 'query', 'description': "Customer ID or short name (default is user's customer), only used when asset short name was provided", 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that changes to the Asset are immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}, {'name': 'updateLastSeen', 'in': 'query', 'description': 'Temporary field added to make updating lastSeenTimestamp/UserId easier. ', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to update an Asset', 'properties': {'addComponents': {'type': 'list', 'description': 'Specify components to be added with the asset', 'items': {'type': 'dict', 'description': 'Request to add Components to an Asset', 'properties': {'componentDefinition': {'type': 'str', 'description': 'ID or short name of component definition.\nIt should exist within asset definition for the owning asset\n'}, 'ttl': {'type': 'int', 'description': "Time to live duration of this component since last seen, 0 means won't expire.", 'minimum': 0}, 'unique': {'type': 'bool', 'description': 'Whether the Asset can only have a single active component of this type\n<b>NB: Adding unique components to an existing asset will expire existing components with the same component definition</b>\n'}, 'value': {'type': 'str', 'description': "Value for the asset component.\nType of value should match component definition's requirement.\n"}}, 'required': ['componentDefinition', 'value']}, 'uniqueItems': True}, 'criticality': {'type': 'dict', 'description': "Request to update a Group or Asset's criticality triad", 'properties': {'availability': {'type': 'str', 'description': 'Update availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'str', 'description': 'Update confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'str', 'description': 'Update integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'deleteComponents': {'type': 'list', 'description': 'Specify IDs of the components to be deleted from the asset', 'items': {'type': 'str', 'description': 'Specify IDs of the components to be deleted from the asset'}, 'uniqueItems': True}, 'description': {'type': 'str', 'description': 'Update description of asset. Support HTML, which will be sanitized'}, 'name': {'type': 'str', 'description': 'Update name of asset', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'owner': {'type': 'str', 'description': "Update owner of asset, specified by user ID or username. When username is specified, it should exist within current user's domain"}, 'shortName': {'type': 'str', 'description': 'Update short name of asset', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'ttl': {'type': 'int', 'description': "Time to live duration of this asset since last seen, 0 means won't expire.", 'minimum': 0}}}}}, 'required': True}, 'responses': {'default': {'description': 'Updates an existing Asset', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset is a digital or analog resource a customer wishes to model in Argus in the Asset Service framework', 'properties': {'assetDefinition': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'components': {'type': 'array', 'description': 'Components of the asset', 'items': {'type': 'object', 'description': 'An Asset Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of an Asset', 'properties': {'assetId': {'type': 'string', 'format': 'uuid', 'description': 'Asset to which this component is attached'}, 'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created'}, 'customerId': {'type': 'integer', 'format': 'int64', 'description': 'Customer that owns the Asset to which this component is attached'}, 'dataSourceId': {'type': 'string', 'format': 'uuid', 'description': 'Data source which reported component'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of the component', 'items': {'type': 'string', 'description': 'Flags of the component', 'enum': ['deleted', 'unique']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated'}, 'type': {'type': 'string', 'description': 'Type of the component value', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was created'}, 'criticality': {'type': 'object', 'description': 'The criticality triad', 'properties': {'availability': {'type': 'string', 'description': 'The availability criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'confidentiality': {'type': 'string', 'description': 'The confidentiality criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}, 'integrity': {'type': 'string', 'description': 'The integrity criticality level', 'enum': ['unknown', 'low', 'medium', 'high', 'critical']}}}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSource': {'type': 'object', 'description': "Data Source's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}, 'name': {'type': 'string', 'description': 'Name of the data source'}, 'shortName': {'type': 'string', 'description': 'Short name of the data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset'}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the first time'}, 'flags': {'type': 'array', 'description': 'Flags of the asset', 'items': {'type': 'string', 'description': 'Flags of the asset', 'enum': ['deleted', 'hasHighVulnerability', 'hasCriticalVulnerability', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastScanTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was scanned the last time'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset was last updated'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'owner': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'parentGroups': {'type': 'array', 'description': 'Parent groups of the asset', 'items': {'type': 'object', 'description': "Compact representation of an Asset Group's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset group'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset group'}}}, 'uniqueItems': True}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}, 'totalCVSS': {'type': 'number', 'format': 'double', 'description': "Total CVSS of the asset, summed up of assets' vulnerabilities cvss"}, 'vulnerabilitiesCount': {'type': 'integer', 'format': 'int32', 'description': "Amount of vulnerabilities of the asset, summed up count of assets' vulnerabilities"}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
update_asset_definition = {'tags': ['assets/v2/asset/definition'], 'summary': 'Updates an existing AssetDefinition (INTERNAL)', 'description': '', 'operationId': 'updateAssetDefinition', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'AssetDefinition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'domain', 'in': 'query', 'description': "Domain ID or name (default is user's domain), required when definition is identified by short name", 'schema': {'type': 'string'}, 'type': 'str'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to update an Asset Definition', 'properties': {'addComponent': {'type': 'list', 'description': 'Component definitions (ID or short name) to be added to the asset definition, they have to belong to same domain as asset definition.', 'items': {'type': 'str', 'description': 'Component definitions (ID or short name) to be added to the asset definition, they have to belong to same domain as asset definition.'}, 'uniqueItems': True}, 'deleteComponent': {'type': 'list', 'description': 'Component definitions ((ID or short name)) to be deleted from the asset definition', 'items': {'type': 'str', 'description': 'Component definitions ((ID or short name)) to be deleted from the asset definition'}, 'uniqueItems': True}, 'description': {'type': 'str', 'description': 'Optional Asset definition description. Will be HTML sanitized'}, 'name': {'type': 'str', 'description': 'Update asset definition name', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'shortName': {'type': 'str', 'description': 'Update asset definition short name', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}}}}}, 'required': True}, 'responses': {'default': {'description': 'Updates an existing AssetDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Definition determines what kind of components may be attached to Assets of this type', 'properties': {'componentDefinitions': {'type': 'array', 'description': 'Component definitions belong to asset definition', 'items': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was deleted'}, 'description': {'type': 'string', 'description': 'Description of the asset definition'}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': 'Flags of asset definition', 'items': {'type': 'string', 'description': 'Flags of asset definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the asset definition was last updated'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAssetDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
update_asset_link_definition = {'tags': ['assets/v2/asset/link/definition'], 'summary': 'Updates an existing active AssetLinkDefinition (TEST)', 'description': '', 'operationId': 'updateAssetLinkDefinition', 'parameters': [{'name': 'definition', 'in': 'path', 'description': 'Asset Link Definition ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to update an existing active Asset Link Definition', 'properties': {'addBinding': {'type': 'list', 'description': 'Asset Link Bindings to be added to the asset definition.', 'items': {'type': 'dict', 'description': 'Request to create an Asset Link Binding when creating an Asset Link Definition', 'properties': {'origin': {'type': 'str', 'description': 'The Asset Definition id or shortname for origin Assets of the relationship represented by this binding.'}, 'target': {'type': 'str', 'description': 'The Asset Definition id or shortname for target Assets of the relationship represented by this binding.'}}, 'required': ['origin', 'target']}, 'uniqueItems': True}, 'inverse': {'type': 'str', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'localizedDescriptions': {'type': 'dict', 'additionalProperties': {'type': 'str', 'description': 'The localized descriptions of the link definition. There must be always be at English defined.'}, 'description': 'The localized descriptions of the link definition. There must be always be at English defined.'}, 'property': {'type': 'str', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'removeBinding': {'type': 'list', 'description': 'Asset Link Bindings to be deleted and removed from the asset definition.', 'items': {'type': 'str', 'description': 'Asset Link Bindings to be deleted and removed from the asset definition.'}, 'uniqueItems': True}, 'shortName': {'type': 'str', 'description': 'New globally unique shortname for the Asset Link Definition.'}}}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Updates an existing active AssetLinkDefinition', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'An Asset Link Definition describes and restricts relationships between specific types of Assets.', 'properties': {'bindings': {'type': 'array', 'description': 'Asset Link Bindings belonging to this Asset Link Definition.', 'items': {'type': 'object', 'description': 'An Asset Link Binding determines the allowed origin and target of an eventual Asset Link.', 'properties': {'flags': {'type': 'array', 'description': 'Flags of the Asset Link Binding', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Binding', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'origin': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}, 'target': {'type': 'object', 'description': "Compact representation of an Asset Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of asset definition'}, 'name': {'type': 'string', 'description': 'Name of asset definition'}, 'shortName': {'type': 'string', 'description': 'Short name of asset definition'}}}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was created.'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the Asset Link Definition', 'items': {'type': 'string', 'description': 'Flags of the Asset Link Definition', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'inverse': {'type': 'string', 'description': "The optional inverse of the link definition's property e.g. childOf"}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'When the Asset Link Definition was last updated.'}, 'localizedDescription': {'type': 'string', 'description': "Description of the Asset Link Definition based on the current user's language settings. Defaults to English if a localised version is unavailable."}, 'localizedDescriptions': {'type': 'object', 'additionalProperties': {'type': 'string', 'description': 'All available descriptions of the Asset Link Definition'}, 'description': 'All available descriptions of the Asset Link Definition'}, 'property': {'type': 'string', 'description': "The link definition's property describing the relationship e.g. parentOf"}, 'shortName': {'type': 'string', 'description': 'Short name of the Asset Link Definition. This name is globally unique across Asset Link Definitions.'}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAssetLinkDefinition'], 'x-Argus-ApiStatus': 'TEST'}