"""Autogenerated API schema"""
from argus_api import session


add_vulnerability = {'tags': ['assets/v2/vulnerability'], 'summary': 'Creates a new vulnerability instance (PUBLIC)', 'description': '', 'operationId': 'addVulnerability', 'parameters': [{'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh making the new vulnerability immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to create a Vulnerability', 'properties': {'asset': {'type': 'str', 'description': 'ID or short name of asset. If short name is specified, then the asset is looked up from the customer.'}, 'components': {'type': 'list', 'description': 'Specify components to be added with the vulnerability.', 'items': {'type': 'dict', 'description': 'Request to add component(s) to a vulnerability', 'properties': {'componentDefinition': {'type': 'str', 'description': 'ID or short name of component definition, it should exist within asset definition for the owning asset'}, 'ttl': {'type': 'int', 'description': "Time to live duration of this component since last seen, 0 means won't expire.", 'minimum': 0}, 'value': {'type': 'str', 'description': "Value for the vulnerability component, type of value should match component definition's requirement"}}, 'required': ['componentDefinition', 'value']}, 'uniqueItems': True}, 'customer': {'type': 'str', 'description': "ID or short name of customer, if not specified then current user's customer will be assigned."}, 'cvss': {'type': 'float', 'description': 'CVSS score of vulnerability (range from 0 to 10). Setting this field will overwrite\nvulnerability definition settings and the value will not be updated when CVSS or Severity\nin the definition changes.', 'maximum': 10, 'minimum': 0}, 'dataSource': {'type': 'str', 'description': 'ID or short name of the Data Source'}, 'observationDescription': {'type': 'str', 'description': 'Description specific to this vulnerability. May contain customer specific data. Will be sanitized as HTML. '}, 'observationReferences': {'type': 'list', 'description': 'References specific to this vulnerability e.g. internal URLs. May contain customer specific data.', 'items': {'type': 'str', 'description': 'References specific to this vulnerability e.g. internal URLs. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'str', 'description': 'Raw output from vulnerability scan. Will be sanitized as html.'}, 'severity': {'type': 'str', 'description': 'Severity level of vulnerability. If not provided, then severity will be calculated based on CVSS value.\nSetting this field will overwrite vulnerability definition settings and the value will not be updated\nwhen CVSS or Severity in the definition changes.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'int', 'description': "Time to live duration of this vulnerability since last seen, 0 means won't expire.", 'minimum': 0}, 'vulnerability': {'type': 'str', 'description': 'Either vulnerability definition ID (UUID) or vulnerability identifier (e.g. CVE-2015-3429). First the value will be used to lookup vulnerability definition (UUID will only be supported for vulnerability definition lookup). If vulnerability definition exists, then associate this vulnerability instance to it, and require read permission on vulnerability source of the definition. Otherwise create vulnerability instance with vulnerability identifier.', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}}, 'required': ['asset', 'vulnerability']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new vulnerability instance', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['addAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
add_vulnerability_definition = {'tags': ['assets/v2/vulnerability/definition'], 'summary': 'Creates a new vulnerability definition also require to have read & write permission of vulnerability source that the definition is belonging to. (INTERNAL)', 'description': '', 'operationId': 'addVulnerabilityDefinition', 'parameters': [{'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh making the new vulnerability definition immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to create a Vulnerability Definition', 'properties': {'conclusion': {'type': 'str', 'description': 'Short summary of vulnerability. Will be sanitized as html.'}, 'cvss': {'type': 'float', 'default': 0.0, 'description': 'CVSS score of vulnerability definition (range from 0 to 10).', 'maximum': 10, 'minimum': 0}, 'description': {'type': 'str', 'description': 'Description of vulnerability. Will be sanitized as html.'}, 'exploitAvailable': {'type': 'bool', 'description': 'Set if an exploit is available for the vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'int', 'default': 0, 'description': "Set the timestamp when the exploit became available. Setting this value also expects setting exploitAvailable to 'true' with this request. Setting/leaving it to 0 will use current timestamp."}, 'name': {'type': 'str', 'description': 'Name of vulnerability. Will be sanitized as text.'}, 'references': {'type': 'list', 'description': 'References to vulnerability (e.g. CVE number). Will be sanitized as html.', 'items': {'type': 'str', 'description': 'References to vulnerability (e.g. CVE number). Will be sanitized as html.'}, 'uniqueItems': True}, 'severity': {'type': 'str', 'description': 'Severity level of vulnerability definition. If not provided, then severity will be calculated based on CVSS value.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'str', 'description': 'How to fix vulnerability. Will be sanitized as html.'}, 'vulnerabilityID': {'type': 'str', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner). Needs to be unique.', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'vulnerabilitySource': {'type': 'str', 'description': 'Define vulnerability source by source ID or short name.'}}, 'required': ['vulnerabilityID', 'vulnerabilitySource']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new vulnerability definition also require to have read & write permission of vulnerability source that the definition is belonging to.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['addVulnerabilityDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
add_vulnerability_source = {'tags': ['assets/v2/vulnerability/source'], 'summary': 'Creates a new VulnerabilitySource (INTERNAL)', 'description': '', 'operationId': 'addVulnerabilitySource', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to create a Vulnerability Source', 'properties': {'name': {'type': 'str', 'description': 'Name of source', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'readPermission': {'type': 'str', 'description': 'Permission required to allow read access of source & vulnerability definition belong to this source', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'shortName': {'type': 'str', 'description': 'Vulnerability source short name, required and unique', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'writePermission': {'type': 'str', 'description': 'Permission required to allow write access (create/update/delete) source & of vulnerability definition belong to this source', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}}, 'required': ['name', 'readPermission', 'shortName', 'writePermission']}}}, 'required': True}, 'responses': {'default': {'description': 'default response', 'content': {'application/json': {}}}, '201': {'description': 'Creates a new VulnerabilitySource', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'The source of observed Vulnerabilities and their Definitions e.g. a scanner', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of vulnerability source', 'items': {'type': 'string', 'description': 'Flags of vulnerability source', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was last updated'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'readPermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}, 'writePermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['addVulnerabilitySource'], 'x-Argus-ApiStatus': 'INTERNAL'}
bulk_recast_async_observations = {'tags': ['assets/v2/vulnerability'], 'summary': 'Bulk recasts a set of observations. (PUBLIC)', 'description': '', 'operationId': 'bulkRecastAsyncObservations', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': "Request to bulk recast Observations. 'No update' ids will be skipped.", 'properties': {'cvss': {'type': 'float', 'description': 'New CVSS score (range from 0 to 10).', 'maximum': 10, 'minimum': 0}, 'observations': {'type': 'list', 'description': "The observations' unique ids", 'items': {'type': 'str', 'description': "The observations' unique ids"}, 'maxItems': 1000, 'minItems': 1, 'uniqueItems': True}, 'severity': {'type': 'str', 'description': 'New Severity.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}}, 'required': ['severity']}}}, 'required': True}, 'responses': {'default': {'description': 'Bulk recasts a set of observations.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'Bulk recast service task id and metadata.', 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the created ServiceTask.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
bulk_recast_observations = {'tags': ['assets/v2/vulnerability'], 'summary': 'Bulk recasts a set of observations. (PUBLIC)', 'description': '', 'operationId': 'bulkRecastObservations', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': "Request to bulk recast Observations. 'No update' ids will be skipped.", 'properties': {'cvss': {'type': 'float', 'description': 'New CVSS score (range from 0 to 10).', 'maximum': 10, 'minimum': 0}, 'observations': {'type': 'list', 'description': "The observations' unique ids", 'items': {'type': 'str', 'description': "The observations' unique ids"}, 'maxItems': 1000, 'minItems': 1, 'uniqueItems': True}, 'severity': {'type': 'str', 'description': 'New Severity.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}}, 'required': ['severity']}}}, 'required': True}, 'responses': {'default': {'description': 'Bulk recasts a set of observations.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'Bulk recast service task id and metadata.', 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the created ServiceTask.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'deprecated': True, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
bulk_resolve_vulnerabilities = {'tags': ['assets/v2/vulnerability'], 'summary': 'Bulk resolves a set of vulnerabilities. (PUBLIC)', 'description': '', 'operationId': 'bulkResolveVulnerabilities', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'comment': {'type': 'str', 'description': 'Comment on why the vulnerability is resolved. If present will apply to all vulnerabilities in the request.', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'ignoreResolved': {'type': 'bool', 'default': False, 'description': 'Whether to reject the entire request if any of the observations are already resolved.'}, 'observations': {'type': 'list', 'description': "The observations' unique ids", 'items': {'type': 'str', 'description': "The observations' unique ids"}, 'maxItems': 1000, 'minItems': 1, 'uniqueItems': True}, 'resolution': {'type': 'str', 'description': 'Specify resolution reason.', 'enum': ['accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionExpiryTimestamp': {'type': 'str', 'default': '0', 'description': 'When will the current resolution expire (0 means no expiry). Allows unix timestamp (milliseconds), ISO timestamp, or relative time values. See https://docs.mnemonic.no/x/AQDXAQ  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}}, 'required': ['resolution']}}}, 'required': True}, 'responses': {'default': {'description': 'Bulk resolves a set of vulnerabilities.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
bulk_task_resolve_observations = {'tags': ['assets/v2/vulnerability'], 'summary': 'Bulk resolves a set of observations or observations matching the requested criteria. (PUBLIC)', 'description': '', 'operationId': 'bulkResolveAsyncObservations', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'comment': {'type': 'str', 'description': 'Comment on why the vulnerability is resolved. If present will apply to all vulnerabilities in the request.', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'criteria': {'type': 'dict', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether to include deleted Vulnerabilities'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'indexEndTimestamp': {'type': 'str', 'description': 'Limit search to indices created on or before this timestamp. Must be in the range [12 months ago - today].  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'indexStartTimestamp': {'type': 'str', 'description': 'Limit search to indices created on or after this timestamp. Must be in the range [12 months ago - today].  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'limit': {'type': 'int', 'default': 25, 'description': "Limit the results to the specified amount. If the sum of offset and limit is greater than 10'000 the results will likely be incomplete as 10'000 exceeds the search engine's maximum result window.", 'minimum': 0}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'offset': {'type': 'int', 'default': 0, 'description': "Skip specified amount of results. If the sum of offset and limit is greater than 10'000 the results will likely be incomplete as 10'000 exceeds the search engine's maximum result window.", 'minimum': 0}, 'order': {'type': 'list', 'items': {'type': 'string'}}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'sortBy': {'type': 'list', 'description': 'Field(s) by which to sort the results', 'items': {'type': 'str', 'description': 'Field(s) by which to sort the results', 'enum': ['vulnerabilityID', 'cvss', 'severity', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'expireTimestamp', 'socketComponent', '-vulnerabilityID', '-cvss', '-severity', '-createdTimestamp', '-lastUpdatedTimestamp', '-deletedTimestamp', '-firstSeenTimestamp', '-lastSeenTimestamp', '-resolutionTimestamp', '-resolutionExpiryTimestamp', '-expireTimestamp', '-socketComponent']}}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria that are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Vulnerabilities', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria will be excluded'}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria are required (logical AND-ed together with parent criteria)'}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}, 'observations': {'type': 'list', 'description': "The observations' unique ids", 'items': {'type': 'str', 'description': "The observations' unique ids"}, 'maxItems': 1000, 'minItems': 0, 'uniqueItems': True}, 'resolution': {'type': 'str', 'description': 'Specify resolution reason.', 'enum': ['accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionExpiryTimestamp': {'type': 'str', 'default': '0', 'description': 'When will the current resolution expire (0 means no expiry). Allows unix timestamp (milliseconds), ISO timestamp, or relative time values. See https://docs.mnemonic.no/x/AQDXAQ  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}}, 'required': ['resolution']}}}, 'required': True}, 'responses': {'default': {'description': 'Bulk resolves a set of observations or observations matching the requested criteria.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'Bulk resolve service task id and metadata.', 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the created ServiceTask.'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
delete_vulnerability = {'tags': ['assets/v2/vulnerability'], 'summary': 'Delete vulnerability (PUBLIC)', 'description': '', 'operationId': 'deleteVulnerability', 'parameters': [{'name': 'id', 'in': 'path', 'description': 'vulnerability ID', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that updates to the Vulnerability are immediately searchable', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'responses': {'default': {'description': 'Delete vulnerability', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['removeAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
delete_vulnerability_definition = {'tags': ['assets/v2/vulnerability/definition'], 'summary': 'Delete Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to. (INTERNAL)', 'description': '', 'operationId': 'deleteVulnerabilityDefinition', 'parameters': [{'name': 'idOrVulnerabilityID', 'in': 'path', 'description': 'Vulnerability definition ID or identifier of vulnerability', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that updates to the VulnerabilityDefinition are immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'responses': {'default': {'description': 'Delete Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['deleteVulnerabilityDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
delete_vulnerability_source = {'tags': ['assets/v2/vulnerability/source'], 'summary': 'Marks a VulnerabilitySource as deleted (INTERNAL)', 'description': '', 'operationId': 'deleteVulnerabilitySource', 'parameters': [{'name': 'source', 'in': 'path', 'description': 'VulnerabilitySource ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'responses': {'default': {'description': 'Marks a VulnerabilitySource as deleted', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'The source of observed Vulnerabilities and their Definitions e.g. a scanner', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of vulnerability source', 'items': {'type': 'string', 'description': 'Flags of vulnerability source', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was last updated'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'readPermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}, 'writePermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['deleteVulnerabilitySource'], 'x-Argus-ApiStatus': 'INTERNAL'}
get_observations_by_vulnerability_compact_view = {'tags': ['assets/v2/vulnerability'], 'summary': "Search for observations and group them by their vulnerability along with each vulnerability's observation count. This differs from the stats endpoint in that it supports sorting and pagination of the resulting collapsed vulnerability results. (TEST)", 'description': '', 'operationId': 'getObservationsByVulnerabilityCompactView', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether to include deleted Vulnerabilities'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'limit': {'type': 'int', 'default': 25, 'description': 'Limit the results to the specified amount.', 'minimum': 0}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'offset': {'type': 'int', 'default': 0, 'description': 'Skip specified amount of results.', 'minimum': 0}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'sortBy': {'type': 'list', 'default': '-count', 'description': 'Field(s) by which to sort the results', 'items': {'type': 'str', 'default': '-count', 'description': 'Field(s) by which to sort the results', 'enum': ['count', 'severity', '-count', '-severity']}}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria that are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Vulnerabilities', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria will be excluded'}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria are required (logical AND-ed together with parent criteria)'}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}}}, 'required': True}, 'responses': {'default': {'description': "Search for observations and group them by their vulnerability along with each vulnerability's observation count.\nThis differs from the stats endpoint in that it supports sorting and pagination of the resulting collapsed vulnerability results.\n", 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'properties': {'count': {'type': 'integer', 'format': 'int64'}, 'definition': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'TEST'}
get_vulnerability_by_id = {'tags': ['assets/v2/vulnerability'], 'summary': 'Get Vulnerability by ID. (PUBLIC)', 'description': '', 'operationId': 'getVulnerabilityByID', 'parameters': [{'name': 'id', 'in': 'path', 'description': 'Vulnerability ID (UUID)', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}, {'name': 'includeRawOutput', 'in': 'query', 'description': 'Whether include raw output, default not include', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}], 'responses': {'default': {'description': 'Get Vulnerability by ID.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
get_vulnerability_definition = {'tags': ['assets/v2/vulnerability/definition'], 'summary': 'Get Vulnerability definition, also require to have read permission of vulnerability source that the definition is belonging to. (PUBLIC)', 'description': '', 'operationId': 'getVulnerabilityDefinition', 'parameters': [{'name': 'idOrVulnerabilityID', 'in': 'path', 'description': 'Vulnerability definition ID or identifier of vulnerability', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'responses': {'default': {'description': 'Get Vulnerability definition, also require to have read permission of vulnerability source that the definition is belonging to.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewVulnerabilityDefinition'], 'x-Argus-ApiStatus': 'PUBLIC'}
get_vulnerability_source = {'tags': ['assets/v2/vulnerability/source'], 'summary': 'Get VulnerabilitySource (INTERNAL)', 'description': '', 'operationId': 'getVulnerabilitySource', 'parameters': [{'name': 'source', 'in': 'path', 'description': 'VulnerabilitySource ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'responses': {'default': {'description': 'Get VulnerabilitySource', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'The source of observed Vulnerabilities and their Definitions e.g. a scanner', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of vulnerability source', 'items': {'type': 'string', 'description': 'Flags of vulnerability source', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was last updated'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'readPermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}, 'writePermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'INTERNAL'}
get_vulnerability_statistics = {'tags': ['assets/v2/vulnerability'], 'summary': 'Generate the requested statistics for Vulnerabilities matching the provided search criteria. (PUBLIC)', 'description': '', 'operationId': 'getVulnerabilityStatistics', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Parent request for statistics for a set of Vulnerability Observations', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'fieldAggregation': {'type': 'list', 'description': '  The fields by which to group statistics.\n  The order of fieldAggregations in the request will determine the hierarchy of the aggregations and their buckets.\n', 'items': {'type': 'dict', 'description': 'Request to instruct the service by which field(s) to group Vulnerability statistics', 'properties': {'field': {'type': 'str', 'description': 'The field by which to group vulnerabilities when generating statistics.', 'enum': ['customer', 'assetGroup', 'asset', 'vulnerabilityDefinition', 'severity', 'resolution']}, 'limit': {'type': 'int', 'default': 25, 'description': 'The maximum number of distinct categories for the given group field for which to generate statistics', 'minimum': 1}}, 'required': ['field']}}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether to include deleted Vulnerabilities'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'rangeMetric': {'type': 'list', 'description': 'The timestamp fields for which to generate statistics.', 'items': {'type': 'dict', 'description': "Request for statistics for Vulnerability Observations filtered by a time range where 'from' is inclusive and 'to' is exclusive", 'properties': {'endTimestamp': {'type': 'str', 'description': 'Filter by this max value. Defaults to NOW  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'field': {'type': 'str', 'description': 'The timestamp field against which to filter', 'enum': ['created', 'deleted', 'lastSeen', 'lastUpdated', 'resolved', 'resolutionExpiry', 'timeToResolve']}, 'includeEmpty': {'type': 'bool', 'default': False, 'description': 'Whether to return temporal buckets for the entire range regardless of whether they contain valuesMust be used with resolution.'}, 'name': {'type': 'str', 'description': 'The name for this aggregation. Must be unique for the parent statistics request.', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'resolution': {'type': 'str', 'description': 'If set, the temporal resolution by which to sub-aggregate the range metric i.e. by hour, day, week, month, or year', 'enum': ['day', 'hour', 'week', 'month', 'year']}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Filter by this min value. Defaults to 0 i.e. 1970-01-01.  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}}, 'required': ['field', 'name']}, 'uniqueItems': True}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'statisticsMetric': {'type': 'list', 'description': 'The numerical fields for which to generate metrics.', 'items': {'type': 'dict', 'description': "Request for statistics for a set of Vulnerability Observations' numerical fields", 'properties': {'field': {'type': 'str', 'description': 'The Vulnerability field for which we want statistics', 'enum': ['cvss', 'resolutionTime']}}, 'required': ['field']}, 'uniqueItems': True}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria that are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Vulnerabilities', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria will be excluded'}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria are required (logical AND-ed together with parent criteria)'}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}}}, 'required': True}, 'responses': {'default': {'description': 'Generate the requested statistics for Vulnerabilities matching the provided search criteria.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'Container for statistics on data stored by the Asset Service', 'properties': {'buckets': {'type': 'array', 'description': 'Metric buckets below this level, contains MetricRecords.', 'items': {'type': 'object', 'description': 'Container for statistics on data stored by the Asset Service', 'properties': {'buckets': {'type': 'array', 'description': 'Metric buckets below this level, contains MetricRecords.', 'items': []}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents contained within this record sharing the same metadata'}, 'metadata': {'type': 'object', 'description': 'Metadata for a stats aggregation container', 'properties': {'identifier': {'type': 'string', 'description': "The aggregation container's identifier e.g. id or enum value"}, 'name': {'type': 'string', 'description': "The aggregation container's name."}, 'shortName': {'type': 'string', 'description': "The aggregation container's shortName. Depending on the type of metadata can also be used to retrieve the full object."}, 'type': {'type': 'string', 'description': "The aggregation container's type", 'enum': ['asset', 'customer', 'assetGroup', 'assetDefinition', 'vulnerabilityDefinition', 'vulnerabilitySeverity', 'resolution']}}}, 'rangeMetrics': {'type': 'array', 'description': 'Range metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Metric representing the number of documents that matched the timestamp filter for a given time field e.g. assets created within the last 2 weeks', 'properties': {'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'dateHistogramBuckets': {'type': 'array', 'description': 'If Resolution is defined in the request, this contains the resulting temporal groupings', 'items': {'type': 'object', 'description': 'A date histogram bucket', 'properties': {'count': {'type': 'integer', 'format': 'int64'}, 'key': {'type': 'string'}}}}, 'endTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The max for the filter used when calculating the range metric. Corresponds to the endTimestamp defined in the RangeMetricRequest'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'resolution': {'type': 'string', 'description': 'The temporal resolution by which to sub aggregate matching documents e.g. hour, day, week, month, year', 'enum': ['day', 'hour', 'week', 'month', 'year']}, 'startTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The min for the filter used when calculating the range metric. Corresponds to the startTimestamp defined in the RangeMetricRequest'}}}}, 'statisticsMetrics': {'type': 'array', 'description': 'Statistics metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Statistics for a given numerical field for a set of matching documents', 'properties': {'avg': {'type': 'number', 'format': 'double', 'description': 'The average field value of all the documents used to calculate these metrics'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'max': {'type': 'number', 'format': 'double', 'description': 'The maximum field value amongst the documents used to calculate these metrics'}, 'min': {'type': 'number', 'format': 'double', 'description': 'The minimum field value amongst the documents used to calculate these metrics'}, 'sum': {'type': 'number', 'format': 'double', 'description': 'The total field value of all the documents used to calculate these metrics'}}}}}}}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents contained within this record sharing the same metadata'}, 'metadata': {'type': 'object', 'description': 'Metadata for a stats aggregation container', 'properties': {'identifier': {'type': 'string', 'description': "The aggregation container's identifier e.g. id or enum value"}, 'name': {'type': 'string', 'description': "The aggregation container's name."}, 'shortName': {'type': 'string', 'description': "The aggregation container's shortName. Depending on the type of metadata can also be used to retrieve the full object."}, 'type': {'type': 'string', 'description': "The aggregation container's type", 'enum': ['asset', 'customer', 'assetGroup', 'assetDefinition', 'vulnerabilityDefinition', 'vulnerabilitySeverity', 'resolution']}}}, 'rangeMetrics': {'type': 'array', 'description': 'Range metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Metric representing the number of documents that matched the timestamp filter for a given time field e.g. assets created within the last 2 weeks', 'properties': {'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'dateHistogramBuckets': {'type': 'array', 'description': 'If Resolution is defined in the request, this contains the resulting temporal groupings', 'items': {'type': 'object', 'description': 'A date histogram bucket', 'properties': {'count': {'type': 'integer', 'format': 'int64'}, 'key': {'type': 'string'}}}}, 'endTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The max for the filter used when calculating the range metric. Corresponds to the endTimestamp defined in the RangeMetricRequest'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'resolution': {'type': 'string', 'description': 'The temporal resolution by which to sub aggregate matching documents e.g. hour, day, week, month, year', 'enum': ['day', 'hour', 'week', 'month', 'year']}, 'startTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'The min for the filter used when calculating the range metric. Corresponds to the startTimestamp defined in the RangeMetricRequest'}}}}, 'statisticsMetrics': {'type': 'array', 'description': 'Statistics metrics for the requested fields. Only ever found at the bottom of the bucket hierarchy.', 'items': {'type': 'object', 'description': 'Statistics for a given numerical field for a set of matching documents', 'properties': {'avg': {'type': 'number', 'format': 'double', 'description': 'The average field value of all the documents used to calculate these metrics'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'The number of documents whose field was used to calculate these metrics'}, 'key': {'type': 'string', 'description': 'The unique identifying name for this metric. Corresponds to the key defined in the StatisticsMetricRequest'}, 'max': {'type': 'number', 'format': 'double', 'description': 'The maximum field value amongst the documents used to calculate these metrics'}, 'min': {'type': 'number', 'format': 'double', 'description': 'The minimum field value amongst the documents used to calculate these metrics'}, 'sum': {'type': 'number', 'format': 'double', 'description': 'The total field value of all the documents used to calculate these metrics'}}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
list_vulnerabilities = {'tags': ['assets/v2/vulnerability'], 'summary': 'Basic Vulnerability search. Will not return vulnerabilities seen more than 31 days ago. Please use advanced search to return older vulnerabilities. If start/end timestamps are present, the range must be valid for at least one of the time stamp fields. Check the documentation for the TimestampFieldStrategy field for more details. (PUBLIC)', 'description': '', 'operationId': 'listVulnerabilities', 'parameters': [{'name': 'customer', 'in': 'query', 'description': 'Customer ids or short names', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'asset', 'in': 'query', 'description': 'Asset ids or short names. Will resolve with the provided customers, or those the current user has access to if none are provided', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'dataSource', 'in': 'query', 'description': 'Data source, identified by id or shortname, that detected the Vulnerability', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'vulnerabilityID', 'in': 'query', 'description': 'Vulnerability external ID', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'severity', 'in': 'query', 'description': 'Vulnerability Severity', 'schema': {'type': 'list', 'items': {'type': 'str', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'str', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, {'name': 'resolution', 'in': 'query', 'description': 'Vulnerability Resolution', 'schema': {'type': 'list', 'items': {'type': 'str', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'str', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, {'name': 'limit', 'in': 'query', 'description': "Limit results. The sum of limit and offset must be <= 10'000 when not streaming", 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'offset', 'in': 'query', 'description': "Offset results. The sum of limit and offset must be <= 10'000 when not streaming", 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Include deleted', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'includeRawOutput', 'in': 'query', 'description': 'Include raw output from scanner', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'startTimestamp', 'in': 'query', 'description': 'Lower bound timestamp filter value', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'endTimestamp', 'in': 'query', 'description': 'Upper bound timestamp filter value', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'timeFieldStrategy', 'in': 'query', 'description': 'Which time field(s) to filter by timestamp start/end', 'schema': {'type': 'list', 'default': 'all', 'items': {'type': 'str', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'type': 'list', 'default': 'all', 'items': {'type': 'str', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, {'name': 'sortBy', 'in': 'query', 'description': 'Field(s) to sort by - refer to the search POST endpoint to see which fields are permitted', 'schema': {'type': 'list', 'items': {'type': 'string'}}, 'type': 'list', 'items': {'type': 'string'}}], 'responses': {'default': {'description': 'Basic Vulnerability search. Will not return vulnerabilities seen more than 31 days ago.\nPlease use advanced search to return older vulnerabilities.\nIf start/end timestamps are present, the range must be valid\nfor at least one of the time stamp fields.\nCheck the documentation for the TimestampFieldStrategy field for more details.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
list_vulnerability_definitions = {'tags': ['assets/v2/vulnerability/definition'], 'summary': 'Basic Vulnerability Definition search. (PUBLIC)', 'description': '', 'operationId': 'listVulnerabilityDefinitions', 'parameters': [{'name': 'vulnerabilityID', 'in': 'query', 'description': 'External Vulnerability ID', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'limit', 'in': 'query', 'description': "Limit results. The sum of limit and offset must be <= 10'000 when not streaming", 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'offset', 'in': 'query', 'description': "Offset results. The sum of limit and offset must be <= 10'000 when not streaming", 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'includeDeleted', 'in': 'query', 'description': 'Include deleted', 'schema': {'type': 'bool', 'default': 'false'}, 'type': 'bool', 'default': 'false'}, {'name': 'startTimestamp', 'in': 'query', 'description': 'Lower bound timestamp filter value', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'endTimestamp', 'in': 'query', 'description': 'Upper bound timestamp filter value', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'keywords', 'in': 'query', 'description': "Keywords to search by. Will return matches in any field defined by the 'all' KeywordFieldStrategy", 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'sortBy', 'in': 'query', 'description': 'Field(s) to sort by - refer to the search POST endpoint to see which fields are permitted', 'schema': {'type': 'list', 'items': {'type': 'string'}}, 'type': 'list', 'items': {'type': 'string'}}], 'responses': {'default': {'description': 'Basic Vulnerability Definition search.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewVulnerabilityDefinition'], 'x-Argus-ApiStatus': 'PUBLIC'}
reopen_vulnerability = {'tags': ['assets/v2/vulnerability'], 'summary': 'Reopens a vulnerability, marking it as unresolved. (PUBLIC)', 'description': '', 'operationId': 'reopenVulnerability', 'parameters': [{'name': 'id', 'in': 'path', 'description': 'Vulnerability ID', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that the reopened vulnerability is immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'comment': {'type': 'str', 'description': 'Comment describing why the vulnerability should be reopened'}}, 'required': ['comment']}}}, 'required': True}, 'responses': {'default': {'description': 'Reopens a vulnerability, marking it as unresolved.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
resolve_vulnerability = {'tags': ['assets/v2/vulnerability'], 'summary': 'Resolves a vulnerability. (PUBLIC)', 'description': '', 'operationId': 'resolveVulnerability', 'parameters': [{'name': 'id', 'in': 'path', 'description': 'Vulnerability ID', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that the resolved vulnerability is immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'comment': {'type': 'str', 'description': 'Comment on why the vulnerability is resolved.', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'resolution': {'type': 'str', 'description': 'Specify resolution reason.', 'enum': ['accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionExpiryTimestamp': {'type': 'str', 'default': '0', 'description': 'When will the current resolution expire (0 means no expiry). Allows unix timestamp (milliseconds), ISO timestamp, or relative time values. See https://docs.mnemonic.no/x/AQDXAQ  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}}, 'required': ['comment', 'resolution']}}}, 'required': True}, 'responses': {'default': {'description': 'Resolves a vulnerability.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
search_vulnerabilities = {'tags': ['assets/v2/vulnerability'], 'summary': 'Search Vulnerabilities (PUBLIC)', 'description': '', 'operationId': 'searchVulnerabilities', 'parameters': [{'name': 'Accept', 'in': 'header', 'schema': {'type': 'string'}}, {'name': 'exportFields', 'in': 'query', 'description': 'Select order of and fields to export.', 'schema': {'type': 'list', 'items': {'type': 'str', 'enum': ['vulnerabilityId', 'severity', 'cvss', 'definitionName', 'definitionFlags', 'assetName', 'customerName', 'resolution', 'resolutionExpiryTimestamp', 'createdTimestamp', 'createdByUserName', 'lastUpdatedTimestamp', 'lastUpdatedByUserName', 'resolutionTimestamp', 'resolvedByUserName', 'firstSeenTimestamp', 'firstSeenByUserName', 'lastSeenTimestamp', 'lastSeenByUserName', 'deletedTimestamp', 'deletedByUserName', 'resolutionComment', 'components', 'definitionExploitAvailable', 'references']}}, 'type': 'list', 'items': {'type': 'str', 'enum': ['vulnerabilityId', 'severity', 'cvss', 'definitionName', 'definitionFlags', 'assetName', 'customerName', 'resolution', 'resolutionExpiryTimestamp', 'createdTimestamp', 'createdByUserName', 'lastUpdatedTimestamp', 'lastUpdatedByUserName', 'resolutionTimestamp', 'resolvedByUserName', 'firstSeenTimestamp', 'firstSeenByUserName', 'lastSeenTimestamp', 'lastSeenByUserName', 'deletedTimestamp', 'deletedByUserName', 'resolutionComment', 'components', 'definitionExploitAvailable', 'references']}}, {'name': 'exportComponents', 'in': 'query', 'description': 'Select which component values to include in the export.', 'schema': {'type': 'list', 'items': {'type': 'string'}}, 'type': 'list', 'items': {'type': 'string'}}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether to include deleted Vulnerabilities'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'indexEndTimestamp': {'type': 'str', 'description': 'Limit search to indices created on or before this timestamp. Must be in the range [12 months ago - today].  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'indexStartTimestamp': {'type': 'str', 'description': 'Limit search to indices created on or after this timestamp. Must be in the range [12 months ago - today].  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'limit': {'type': 'int', 'default': 25, 'description': "Limit the results to the specified amount. If the sum of offset and limit is greater than 10'000 the results will likely be incomplete as 10'000 exceeds the search engine's maximum result window.", 'minimum': 0}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'offset': {'type': 'int', 'default': 0, 'description': "Skip specified amount of results. If the sum of offset and limit is greater than 10'000 the results will likely be incomplete as 10'000 exceeds the search engine's maximum result window.", 'minimum': 0}, 'order': {'type': 'list', 'items': {'type': 'string'}}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'sortBy': {'type': 'list', 'description': 'Field(s) by which to sort the results', 'items': {'type': 'str', 'description': 'Field(s) by which to sort the results', 'enum': ['vulnerabilityID', 'cvss', 'severity', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'expireTimestamp', 'socketComponent', '-vulnerabilityID', '-cvss', '-severity', '-createdTimestamp', '-lastUpdatedTimestamp', '-deletedTimestamp', '-firstSeenTimestamp', '-lastSeenTimestamp', '-resolutionTimestamp', '-resolutionExpiryTimestamp', '-expireTimestamp', '-socketComponent']}}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria that are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Vulnerabilities', 'properties': {'asset': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on these assets by asset id or shortNames. As assets are customer specific we will resolve the assets for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'assetComponent': {'type': 'list', 'description': 'Search for Vulnerabilities observed on Assets containing these components using AND search mode i.e. all criteria must match.<br />If you want different behaviour try using subcriteria', 'items': {'type': 'dict', 'description': "Search criteria for searching for Assets by their Components' properties", 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'assetGroup': {'type': 'list', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities detected on assets who are members or descendants of these groups identified by id or shortNames. As asset groups are customer specific we will resolve the groups for the customers provided in the criteria, if any. If none are provided, we attempt to use the customers the current user has access to.'}, 'uniqueItems': True}, 'component': {'type': 'list', 'description': 'Search for Vulnerabilities with these components', 'items': {'type': 'dict', 'description': 'Search criteria for searching for vulnerabilities by their components', 'properties': {'componentDefinition': {'type': 'str', 'description': 'The ComponentDefinition ID or shortName'}, 'value': {'type': 'list', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail.", 'items': {'type': 'str', 'description': "Restrict to components with field value matching any of the specified values. If the value is of the wrong type for the component's definition the search will fail."}, 'uniqueItems': True}}, 'required': ['componentDefinition']}, 'uniqueItems': True}, 'customer': {'type': 'list', 'description': 'Retrieve vulnerabilities by Customer id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by Customer id or name'}, 'uniqueItems': True}, 'dataSource': {'type': 'list', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName', 'items': {'type': 'str', 'description': 'Search for vulnerabilities detected and reported by these data sources, identified by id or shortName'}, 'uniqueItems': True}, 'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria will be excluded'}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeFlags': {'type': 'list', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly retrieve Vulnerabilities that contain these flags', 'enum': ['deleted', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'exploitAvailable']}, 'uniqueItems': True}, 'includeRawOutput': {'type': 'bool', 'default': False, 'description': 'Whether to include raw output in the result objects'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be queried for keywords (defaults to all observation fields)', 'enum': ['all', 'observationAll', 'definitionAll', 'name', 'description', 'observationDescription', 'solution', 'conclusion', 'references', 'observationReferences', 'vulnerabilityID', 'assetName']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by their definition keywords against definition text fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at most this CVSS score', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Retrieve Vulnerabilities with at least this CVSS score', 'maximum': 10, 'minimum': 0}, 'observationReferences': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their references'}, 'uniqueItems': True}, 'references': {'type': 'list', 'description': 'Search for vulnerabilities by exact matches in their definition references', 'items': {'type': 'str', 'description': 'Search for vulnerabilities by exact matches in their definition references'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria are required (logical AND-ed together with parent criteria)'}, 'resolution': {'type': 'list', 'description': 'Retrieve vulnerabilities with these resolutions', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these resolutions', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'uniqueItems': True}, 'severity': {'type': 'list', 'description': 'Retrieve vulnerabilities with these severities', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities with these severities', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastSeenTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'resolutionTimestamp', 'resolutionExpiryTimestamp', 'firstSeenTimestamp', 'lastSeenTimestamp', 'expireTimestamp', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailable']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by these users, identified by their id or username'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields will be filtered with user ids or shortnames', 'enum': ['all', 'resolvedByUser', 'firstSeenByUser', 'lastSeenByUser', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their vulnerability definition id or name'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve vulnerabilities by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}}}}}, 'required': True}, 'responses': {'default': {'description': 'Search Vulnerabilities', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}, 'text/csv': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'PUBLIC'}
search_vulnerability_definitions = {'tags': ['assets/v2/vulnerability/definition'], 'summary': 'Search Vulnerability Definitions (PUBLIC)', 'description': '', 'operationId': 'searchVulnerabilityDefinitions', 'parameters': [{'name': 'Accept', 'in': 'header', 'schema': {'type': 'string'}}, {'name': 'exportFields', 'in': 'query', 'description': 'Select order of and fields to export.', 'schema': {'type': 'list', 'items': {'type': 'str', 'enum': ['references', 'definitionName', 'definitionExploitAvailable', 'definitionExploitPublished', 'sourceName', 'createdTimestamp', 'lastUpdatedTimestamp']}}, 'type': 'list', 'items': {'type': 'str', 'enum': ['references', 'definitionName', 'definitionExploitAvailable', 'definitionExploitPublished', 'sourceName', 'createdTimestamp', 'lastUpdatedTimestamp']}}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Search criteria for searching for Vulnerability Definitions', 'properties': {'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out VulnerabilityDefinitions that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out VulnerabilityDefinitions that contain these flags', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether to include deleted Vulnerability Definitions'}, 'includeFlags': {'type': 'list', 'description': 'Explicitly search for VulnerabilityDefinitions that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly search for VulnerabilityDefinitions that contain these flags', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['all', 'name', 'description', 'solution', 'conclusion', 'references', 'vulnerabilityID']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'limit': {'type': 'int', 'default': 25, 'description': 'Limit the results to the specified amount', 'minimum': 0}, 'maximumCvss': {'type': 'float', 'description': 'Filter VulnerabilityDefinitions by CVSS lower or equal than.', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Filter VulnerabilityDefinitions by CVSS higher or equal than.', 'maximum': 10, 'minimum': 0}, 'offset': {'type': 'int', 'default': 0, 'description': 'Skip specified amount of results', 'minimum': 0}, 'order': {'type': 'list', 'items': {'type': 'string'}}, 'severity': {'type': 'list', 'description': 'Filter VulnerabilityDefinitions by Severity.', 'items': {'type': 'str', 'description': 'Filter VulnerabilityDefinitions by Severity.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'sortBy': {'type': 'list', 'description': 'Field(s) by which to sort the results', 'items': {'type': 'str', 'description': 'Field(s) by which to sort the results', 'enum': ['name', 'vulnerabilityID', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailableTimestamp', '-name', '-vulnerabilityID', '-createdTimestamp', '-lastUpdatedTimestamp', '-deletedTimestamp', '-exploitAvailableTimestamp']}}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria that are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Vulnerability Definitions', 'properties': {'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria will be excluded'}, 'excludeFlags': {'type': 'list', 'description': 'Explicitly filter out VulnerabilityDefinitions that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly filter out VulnerabilityDefinitions that contain these flags', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'includeFlags': {'type': 'list', 'description': 'Explicitly search for VulnerabilityDefinitions that contain these flags', 'items': {'type': 'str', 'description': 'Explicitly search for VulnerabilityDefinitions that contain these flags', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['all', 'name', 'description', 'solution', 'conclusion', 'references', 'vulnerabilityID']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by a KeywordFieldStrategy'}, 'uniqueItems': True}, 'maximumCvss': {'type': 'float', 'description': 'Filter VulnerabilityDefinitions by CVSS lower or equal than.', 'maximum': 10, 'minimum': 0}, 'minimumCvss': {'type': 'float', 'description': 'Filter VulnerabilityDefinitions by CVSS higher or equal than.', 'maximum': 10, 'minimum': 0}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching these criteria are required (logical AND-ed together with parent criteria)'}, 'severity': {'type': 'list', 'description': 'Filter VulnerabilityDefinitions by Severity.', 'items': {'type': 'str', 'description': 'Filter VulnerabilityDefinitions by Severity.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailableTimestamp']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for VulnerabilityDefinitions by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for VulnerabilityDefinitions by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve definitions by their internal db ID or vulnerabilityID', 'items': {'type': 'str', 'description': 'Retrieve definitions by their internal db ID or vulnerabilityID'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve definitions by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve definitions by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}, 'vulnerabilitySource': {'type': 'list', 'description': 'Retrieve definitions by their source ID or shortname', 'items': {'type': 'str', 'description': 'Retrieve definitions by their source ID or shortname'}, 'uniqueItems': True}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps fields will be filtered with start/endTimestamp', 'enum': ['all', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'exploitAvailableTimestamp']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}, 'user': {'type': 'list', 'description': 'Search for VulnerabilityDefinitions by associated user IDs or short names defined by the UserFieldStrategy', 'items': {'type': 'str', 'description': 'Search for VulnerabilityDefinitions by associated user IDs or short names defined by the UserFieldStrategy'}, 'uniqueItems': True}, 'userFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which user fields to filter with the submitted user value(s)', 'enum': ['all', 'createdByUser', 'lastUpdatedByUser', 'deletedByUser']}, 'uniqueItems': True}, 'vulnerabilityDefinition': {'type': 'list', 'description': 'Retrieve definitions by their internal db ID or vulnerabilityID', 'items': {'type': 'str', 'description': 'Retrieve definitions by their internal db ID or vulnerabilityID'}, 'uniqueItems': True}, 'vulnerabilityID': {'type': 'list', 'description': 'Retrieve definitions by their external ID e.g. CVE-XXXX', 'items': {'type': 'str', 'description': 'Retrieve definitions by their external ID e.g. CVE-XXXX'}, 'uniqueItems': True}, 'vulnerabilitySource': {'type': 'list', 'description': 'Retrieve definitions by their source ID or shortname', 'items': {'type': 'str', 'description': 'Retrieve definitions by their source ID or shortname'}, 'uniqueItems': True}}}}}, 'required': True}, 'responses': {'default': {'description': 'Search Vulnerability Definitions', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}, 'text/csv': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}, 'text/csv': {}, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': {}}}}, 'x-Argus-Required-Permissions': ['viewVulnerabilityDefinition'], 'x-Argus-ApiStatus': 'PUBLIC'}
search_vulnerability_sources = {'tags': ['assets/v2/vulnerability/source'], 'summary': 'Search VulnerabilitySource with specified criteria (INTERNAL)', 'description': '', 'operationId': 'searchVulnerabilitySources', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'includeDeleted': {'type': 'bool', 'default': False, 'description': 'Whether include deleted results'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['shortName', 'name', 'all']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'all', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by keywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by keywordFieldStrategy'}, 'uniqueItems': True}, 'limit': {'type': 'int', 'default': 25, 'description': 'Limit maximum amount of results', 'minimum': 0}, 'offset': {'type': 'int', 'default': 0, 'description': 'Skip specified amount of results', 'minimum': 0}, 'sortBy': {'type': 'list', 'description': 'Field(s) by which to sort the results', 'items': {'type': 'str', 'description': 'Field(s) by which to sort the results', 'enum': ['shortName', 'name', 'createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', '-shortName', '-name', '-createdTimestamp', '-lastUpdatedTimestamp', '-deletedTimestamp']}}, 'source': {'type': 'list', 'description': 'Restrict to specified Vulnerability source (ID or short name)', 'items': {'type': 'str', 'description': 'Restrict to specified Vulnerability source (ID or short name)'}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'subCriteria': {'type': 'list', 'description': 'Set additional criteria which are applied with logical OR by default', 'items': {'type': 'dict', 'description': 'Search sub-criteria for searching for Vulnerability Sources', 'properties': {'endTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (end timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'exclude': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching this criteria will be excluded'}, 'keywordFieldStrategy': {'type': 'list', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'items': {'type': 'str', 'default': 'all', 'description': 'Defines which fields will be searched by keywords (default all supported fields)', 'enum': ['shortName', 'name', 'all']}, 'uniqueItems': True}, 'keywordMatchStrategy': {'type': 'str', 'default': 'all', 'description': 'Defines how strict different keywords should be matched (default match all keywords)', 'enum': ['any', 'all']}, 'keywords': {'type': 'list', 'description': 'Search for keywords against fields defined by keywordFieldStrategy', 'items': {'type': 'str', 'description': 'Search for keywords against fields defined by keywordFieldStrategy'}, 'uniqueItems': True}, 'required': {'type': 'bool', 'default': False, 'description': 'If set to true, objects matching this criteria are required (logical AND-ed together with parent criteria)'}, 'source': {'type': 'list', 'description': 'Restrict to specified Vulnerability source (ID or short name)', 'items': {'type': 'str', 'description': 'Restrict to specified Vulnerability source (ID or short name)'}, 'uniqueItems': True}, 'startTimestamp': {'type': 'str', 'default': '0', 'description': 'Restrict to a time frame based on the set timeFieldStrategy (start timestamp)  Allows to specify a unix timestamp (milliseconds), ISO timestamp, or relative time.', 'externalDocs': {'description': 'See documentation', 'url': 'https://docs.mnemonic.no/api/general_integration_guide/08-time_fields.html#api-iguide-general-time-search'}}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}}}, 'uniqueItems': True}, 'timeFieldStrategy': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'items': {'type': 'str', 'default': 'lastUpdatedTimestamp', 'description': 'Defines which timestamps will be included in the search (default lastUpdatedTimestamp)', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, 'timeMatchStrategy': {'type': 'str', 'default': 'any', 'description': 'Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any)', 'enum': ['any', 'all']}}}}}, 'required': True}, 'responses': {'default': {'description': 'Search VulnerabilitySource with specified criteria', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'The source of observed Vulnerabilities and their Definitions e.g. a scanner', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of vulnerability source', 'items': {'type': 'string', 'description': 'Flags of vulnerability source', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was last updated'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'readPermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}, 'writePermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'INTERNAL'}
search_vulnerability_sources_simplified = {'tags': ['assets/v2/vulnerability/source'], 'summary': 'Search VulnerabilitySources with specified query parameters (INTERNAL)', 'description': '', 'operationId': 'searchVulnerabilitySourcesSimplified', 'parameters': [{'name': 'offset', 'in': 'query', 'description': 'Skip a number of results', 'schema': {'type': 'int', 'default': '0', 'minimum': 0}, 'type': 'int', 'default': '0', 'minimum': 0}, {'name': 'limit', 'in': 'query', 'description': 'Maximum number of returned results', 'schema': {'type': 'int', 'default': '25', 'minimum': 0}, 'type': 'int', 'default': '25', 'minimum': 0}, {'name': 'keywords', 'in': 'query', 'description': 'Search by keywords', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'keywordField', 'in': 'query', 'description': 'Set field strategy for keyword search', 'schema': {'type': 'list', 'default': 'all', 'items': {'type': 'str', 'enum': ['shortName', 'name', 'all']}, 'uniqueItems': True}, 'type': 'list', 'default': 'all', 'items': {'type': 'str', 'enum': ['shortName', 'name', 'all']}, 'uniqueItems': True}, {'name': 'keywordMatch', 'in': 'query', 'description': 'Set match strategy for keyword search', 'schema': {'type': 'str', 'default': 'all', 'enum': ['any', 'all']}, 'type': 'str', 'default': 'all', 'enum': ['any', 'all']}, {'name': 'startTimestamp', 'in': 'query', 'description': 'Start timestamp for time range search', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'endTimestamp', 'in': 'query', 'description': 'End timestamp for time range search', 'schema': {'type': 'str', 'default': '0'}, 'type': 'str', 'default': '0'}, {'name': 'timeField', 'in': 'query', 'description': 'Set field strategy for time range search', 'schema': {'type': 'list', 'default': 'lastUpdatedTimestamp', 'items': {'type': 'str', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, 'type': 'list', 'default': 'lastUpdatedTimestamp', 'items': {'type': 'str', 'enum': ['createdTimestamp', 'lastUpdatedTimestamp', 'deletedTimestamp', 'all']}, 'uniqueItems': True}, {'name': 'timeMatch', 'in': 'query', 'description': 'Set match strategy for time range search', 'schema': {'type': 'str', 'default': 'any', 'enum': ['any', 'all']}, 'type': 'str', 'default': 'any', 'enum': ['any', 'all']}, {'name': 'source', 'in': 'query', 'description': 'Search by sources (ID or short name)', 'schema': {'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, 'type': 'list', 'items': {'type': 'string'}, 'uniqueItems': True}, {'name': 'sortBy', 'in': 'query', 'description': 'Sort search result', 'schema': {'type': 'list', 'items': {'type': 'string'}}, 'type': 'list', 'items': {'type': 'string'}}], 'responses': {'default': {'description': 'Search VulnerabilitySources with specified query parameters', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'array', 'items': {'type': 'object', 'description': 'The source of observed Vulnerabilities and their Definitions e.g. a scanner', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of vulnerability source', 'items': {'type': 'string', 'description': 'Flags of vulnerability source', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was last updated'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'readPermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}, 'writePermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['viewAssets'], 'x-Argus-ApiStatus': 'INTERNAL'}
update_vulnerability = {'tags': ['assets/v2/vulnerability'], 'summary': 'Updates an existing Vulnerability (PUBLIC)', 'description': '', 'operationId': 'updateVulnerability', 'parameters': [{'name': 'id', 'in': 'path', 'description': 'vulnerability ID', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that changes to the vulnerability are immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}, {'name': 'updateLastSeen', 'in': 'query', 'description': 'Temporary field added to make updating lastSeenTimestamp/UserId easier. ', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'properties': {'addComponents': {'type': 'list', 'description': 'Specify components to be added with the vulnerability.', 'items': {'type': 'dict', 'description': 'Request to add component(s) to a vulnerability', 'properties': {'componentDefinition': {'type': 'str', 'description': 'ID or short name of component definition, it should exist within asset definition for the owning asset'}, 'ttl': {'type': 'int', 'description': "Time to live duration of this component since last seen, 0 means won't expire.", 'minimum': 0}, 'value': {'type': 'str', 'description': "Value for the vulnerability component, type of value should match component definition's requirement"}}, 'required': ['componentDefinition', 'value']}, 'uniqueItems': True}, 'addObservationReferences': {'type': 'list', 'description': 'Add references specific to this vulnerability e.g. internal URLs. May contain customer specific data. Will be sanitized as HTML.', 'items': {'type': 'str', 'description': 'Add references specific to this vulnerability e.g. internal URLs. May contain customer specific data. Will be sanitized as HTML.'}, 'uniqueItems': True}, 'cvss': {'type': 'float', 'description': 'Update CVSS score of vulnerability (range from 0 to 10).', 'maximum': 10, 'minimum': 0}, 'deleteComponents': {'type': 'list', 'description': 'Specify IDs of the components to be deleted from the vulnerability.', 'items': {'type': 'str', 'description': 'Specify IDs of the components to be deleted from the vulnerability.'}, 'uniqueItems': True}, 'deleteObservationReferences': {'type': 'list', 'description': 'Delete references specific to this vulnerability. Will be sanitized as HTML first before checking against existing references.', 'items': {'type': 'str', 'description': 'Delete references specific to this vulnerability. Will be sanitized as HTML first before checking against existing references.'}, 'uniqueItems': True}, 'observationDescription': {'type': 'str', 'description': 'Description specific to this vulnerability. May contain customer specific data. Will be sanitized as HTML.'}, 'rawOutput': {'type': 'str', 'description': 'Update raw output from vulnerability scan. Will be sanitized as html.'}, 'severity': {'type': 'str', 'description': 'Update severity level of vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'int', 'description': "Time to live duration of this vulnerability since last seen, 0 means won't expire.", 'minimum': 0}, 'vulnerabilityDefinition': {'type': 'str', 'description': 'Assign vulnerability definition. Only allowed when there was no vulnerability definition assigned.'}}}}}, 'required': True}, 'responses': {'default': {'description': 'Updates an existing Vulnerability', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability observed on an Asset', 'properties': {'asset': {'type': 'object', 'description': "Compact representation of an Asset's metadata", 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'dataSourceID': {'type': 'string', 'format': 'uuid', 'description': 'ID of the Data Source that reported this asset'}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the asset'}, 'shortName': {'type': 'string', 'description': 'Short name of the asset'}}}, 'components': {'type': 'array', 'description': 'Components of the vulnerability.', 'items': {'type': 'object', 'description': 'An Vulnerability Component is a piece of information, whose validity can potentially be quantified and expired, describing an aspect of a Vulnerability', 'properties': {'componentDefinition': {'type': 'object', 'description': "Compact representation of a Component Definition's metadata", 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of component definition'}, 'shortName': {'type': 'string', 'description': 'Short name of component definition'}, 'type': {'type': 'string', 'description': "Expected data type of the component's value. The value of any component created with this definition will be validated against the type and rejected, if invalid.", 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'visibility': {'type': 'string', 'description': 'Whether this Component Definition is appropriate for Assets, Vulnerabilities, or both', 'enum': ['any', 'asset', 'vulnerability']}}}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was created.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was deleted.'}, 'flags': {'type': 'array', 'description': 'Flags of the component.', 'items': {'type': 'string', 'description': 'Flags of the component.', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was seen the last time'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the component was last updated.'}, 'type': {'type': 'string', 'description': 'Type of the component value.', 'enum': ['booleanType', 'intType', 'longType', 'doubleType', 'stringType', 'fullTextType', 'ipType', 'socketType', 'domainType', 'cpeType', 'userType', 'assetType', 'dateType', 'enumType']}, 'value': {'description': 'Value of the component, depends on type the presentation could be different.'}}}, 'uniqueItems': True}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was created.'}, 'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability.'}, 'dataSource': {'type': 'object', 'description': "Compact representation of a Data Source's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of data source'}}}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was deleted. If 0 it is undeleted.'}, 'expireTimestamp': {'type': 'integer', 'format': 'int64', 'description': "When would the vulnerability be expired, determined by 'lastSeenTimestamp + ttl'"}, 'firstSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'firstSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the first time.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability.', 'enum': ['deleted', 'controlledVersion', 'createdByCVM', 'updatedByCVM', 'detectedByCVM', 'missingFromCVM', 'deletedFromCVM', 'definitionMissing', 'overwrittenDefinitionCVSS', 'overwrittenDefinitionSeverity']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastSeenByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastSeenTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was seen the last time.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was last updated.'}, 'observationDescription': {'type': 'string', 'description': 'Description specific to this observed vulnerability. May contain customer specific data.'}, 'observationReferences': {'type': 'array', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.', 'items': {'type': 'string', 'description': 'References specific to this observed vulnerability e.g. internal URLS. May contain customer specific data.'}, 'uniqueItems': True}, 'rawOutput': {'type': 'string', 'description': 'Raw output from vulnerability scan.'}, 'resolution': {'type': 'string', 'description': 'Resolution of the vulnerability.', 'enum': ['unresolved', 'accepted', 'temporarilyAccepted', 'falsePositive', 'serviceNotAvailable', 'noLongerVulnerable']}, 'resolutionComment': {'type': 'string', 'description': 'Comment for the vulnerability resolution.'}, 'resolutionExpiryTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the resolution will expire (0 means no expiry).'}, 'resolutionTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability was resolved.'}, 'resolvedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'ttl': {'type': 'integer', 'format': 'int64', 'description': 'Time to live duration of this vulnerability'}, 'vulnerabilityDefinition': {'type': 'object', 'description': "Compact representation of a Vulnerability Definition's metadata", 'properties': {'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateAsset'], 'x-Argus-ApiStatus': 'PUBLIC'}
update_vulnerability_definition = {'tags': ['assets/v2/vulnerability/definition'], 'summary': 'Updates an existing Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to. (INTERNAL)', 'description': '', 'operationId': 'updateVulnerabilityDefinition', 'parameters': [{'name': 'idOrVulnerabilityID', 'in': 'path', 'description': 'Vulnerability definition ID or identifier of vulnerability', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}, {'name': 'forceIndex', 'in': 'query', 'description': 'Whether to force an Elasticsearch index refresh such that changes to the VulnerabilityDefinition are immediately searchable.', 'schema': {'type': 'boolean'}, 'type': 'bool'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to update a Vulnerability Definition', 'properties': {'addReferences': {'type': 'list', 'description': 'Add references to the vulnerability definition (e.g. CVE number). Will be sanitized as html.', 'items': {'type': 'str', 'description': 'Add references to the vulnerability definition (e.g. CVE number). Will be sanitized as html.'}, 'uniqueItems': True}, 'conclusion': {'type': 'str', 'description': 'Update short summary of the vulnerability. Will be sanitized as html.'}, 'cvss': {'type': 'float', 'description': 'CVSS score of vulnerability (range from 0 to 10).', 'maximum': 10, 'minimum': 0}, 'deleteReferences': {'type': 'list', 'description': 'Delete references from the vulnerability definition. Will be sanitized as html first before check against existing references.', 'items': {'type': 'str', 'description': 'Delete references from the vulnerability definition. Will be sanitized as html first before check against existing references.'}, 'uniqueItems': True}, 'description': {'type': 'str', 'description': 'Update description of the vulnerability definition. Will be sanitized as html.'}, 'exploitAvailable': {'type': 'bool', 'description': 'Update whether an exploit is available for the vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'int', 'default': 0, 'description': "Set or update the timestamp when the exploit became available. Setting this value also expects having exploitAvailable as 'true' or setting it with this request. If it hasn't been set before, then setting/leaving it to 0 will use current timestamp."}, 'name': {'type': 'str', 'description': 'Update name of the vulnerability definition. Will be sanitized as text.'}, 'overwriteReferences': {'type': 'list', 'description': 'Overwrite any existing references with these new values. Requests with this and add/deleteReferences populated are considered invalid and will be rejected. Contents will be sanitized as HTML before saving.', 'items': {'type': 'str', 'description': 'Overwrite any existing references with these new values. Requests with this and add/deleteReferences populated are considered invalid and will be rejected. Contents will be sanitized as HTML before saving.'}, 'uniqueItems': True}, 'severity': {'type': 'str', 'description': 'Severity level of vulnerability. If not provided, then severity will be calculated based on CVSS value.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'str', 'description': 'Update how to fix the vulnerability. Will be sanitized as html.'}, 'updateObservations': {'type': 'bool', 'default': False, 'description': 'If true this will trigger updating all active (non-deleted with resolution status: unresolved,\naccepted or temp_accepted) Observations thus overwriting their current Severity/CVSS values. Unless\nthe values were previously overwritten manually.'}}}}}, 'required': True}, 'responses': {'default': {'description': 'Updates an existing Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to.', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'A Vulnerability Definition determines what kind of components may be attached to Vulnerabilities of this type and contains qualitative informationdescribing the Vulnerability and eventual mitigation/resolution procedures', 'properties': {'conclusion': {'type': 'string', 'description': 'Short summary of vulnerability.'}, 'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was created.'}, 'cvss': {'type': 'number', 'format': 'double', 'description': 'CVSS of the vulnerability definition (scale 0-10).'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was deleted.'}, 'description': {'type': 'string', 'description': 'Description of vulnerability.'}, 'exploitAvailableTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the exploit became available.'}, 'flags': {'type': 'array', 'description': 'Flags of the vulnerability definition.', 'items': {'type': 'string', 'description': 'Flags of the vulnerability definition.', 'enum': ['deleted', 'exploitAvailable']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability definition was last updated.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability.'}, 'references': {'type': 'array', 'description': 'References to vulnerability (e.g. CVE number).', 'items': {'type': 'string', 'description': 'References to vulnerability (e.g. CVE number).'}, 'uniqueItems': True}, 'severity': {'type': 'string', 'description': 'Severity of the vulnerability definition.', 'enum': ['info', 'low', 'medium', 'high', 'critical']}, 'solution': {'type': 'string', 'description': 'How to fix vulnerability.'}, 'vulnerabilityID': {'type': 'string', 'description': 'Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner).'}, 'vulnerabilitySource': {'type': 'object', 'description': "Compact representation of a Vulnerability Sources's metadata", 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}}}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateVulnerabilityDefinition'], 'x-Argus-ApiStatus': 'INTERNAL'}
update_vulnerability_source = {'tags': ['assets/v2/vulnerability/source'], 'summary': 'Updates an existing VulnerabilitySource (INTERNAL)', 'description': '', 'operationId': 'updateVulnerabilitySource', 'parameters': [{'name': 'source', 'in': 'path', 'description': 'VulnerabilitySource ID or short name', 'required': True, 'schema': {'type': 'string'}, 'type': 'str'}], 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to update a Vulnerability Source', 'properties': {'name': {'type': 'str', 'description': 'Update name of vulnerability source', 'pattern': '[\\s\\w\\{\\}\\$\\-\\(\\)\\.\\[\\]"\\\'_/\\\\,\\*\\+\\#:@!?;=]*'}, 'readPermission': {'type': 'str', 'description': 'Update the permission required to allow read access of source & vulnerability definition belong to the source', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'shortName': {'type': 'str', 'description': 'Update short name of vulnerability source', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}, 'writePermission': {'type': 'str', 'description': 'Update the permission required to allow write access (create/update/delete) of source & vulnerability definition belong to the source', 'pattern': '[a-zA-Z0-9_:\\-\\.]*'}}}}}, 'required': True}, 'responses': {'default': {'description': 'Updates an existing VulnerabilitySource', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'description': 'The source of observed Vulnerabilities and their Definitions e.g. a scanner', 'properties': {'createdByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'createdTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was created'}, 'deletedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'deletedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was deleted'}, 'flags': {'type': 'array', 'description': 'Flags of vulnerability source', 'items': {'type': 'string', 'description': 'Flags of vulnerability source', 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'lastUpdatedByUser': {'type': 'object', 'description': 'SPI UserInfo', 'properties': {'customer': {'type': 'object', 'description': 'SPI CustomerInfo', 'properties': {'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the customer.'}, 'shortName': {'type': 'string', 'description': 'Shortname of the customer.'}}}, 'domain': {'type': 'object', 'description': 'SPI DomainInfo', 'properties': {'id': {'type': 'integer', 'format': 'int32', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the domain.'}}}, 'flags': {'type': 'array', 'description': "The subject's flags", 'items': {'type': 'string', 'description': "The subject's flags", 'enum': ['deleted']}, 'uniqueItems': True}, 'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': "The subject's name"}, 'shortName': {'type': 'string', 'description': "The subject's short name"}, 'type': {'type': 'string', 'description': 'Type of the subject', 'enum': ['user', 'group'], 'readOnly': True}}}, 'lastUpdatedTimestamp': {'type': 'integer', 'format': 'int64', 'description': 'Timestamp when the vulnerability source was last updated'}, 'name': {'type': 'string', 'description': 'Name of vulnerability source'}, 'readPermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}, 'shortName': {'type': 'string', 'description': 'Short name of vulnerability source'}, 'writePermission': {'type': 'object', 'description': 'SPI FunctionInfo', 'properties': {'id': {'type': 'integer', 'format': 'int64', 'description': 'ID of the response object.'}, 'name': {'type': 'string', 'description': 'Name of the function.'}}}}}}}}}}, '220': {'description': 'No update performed', 'content': {'application/json': {}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['updateVulnerabilitySource'], 'x-Argus-ApiStatus': 'INTERNAL'}