"""Autogenerated API schema"""
from argus_api import session


prepare_notification = {'tags': ['notifications/v1/notification'], 'summary': 'Prepare a new notification (INTERNAL)', 'description': '', 'operationId': 'prepareNotification', 'requestBody': {'content': {'application/json': {'schema': {'type': 'object', 'description': 'Request to prepare sending notifications. This is only used for API testing, and is not permitted for ordinary users.', 'properties': {'attachments': {'type': 'list', 'description': 'Attachments to process in this notification. The notification rules and destinations will determine how these are used.', 'items': {'type': 'dict', 'description': 'Attachment object which is part of a NotificationRequest', 'properties': {'contentType': {'type': 'str', 'description': 'MIME Content type of this attachment.'}, 'data': {'type': 'string'}, 'fileName': {'type': 'str', 'description': 'Filename of this attachment.'}}, 'required': ['contentType', 'data', 'fileName']}}, 'context': {'type': 'dict', 'additionalProperties': {'type': 'object'}, 'description': 'The notification context objects. The notification event determines which context objects are expected. Each object should be JSON serializable.'}, 'customerID': {'type': 'int', 'description': 'Set the customer context of this notification request. If not set, it should default to the current users customer.', 'minimum': 0}, 'event': {'type': 'str', 'description': 'The name of the notification event to trigger. This will determine notification behaviour and rules.'}, 'recipients': {'type': 'list', 'description': 'Recipients to notify, either user or non-user.', 'items': {'type': 'dict', 'description': 'Recipient object which is part of a NotificationRequest', 'properties': {'email': {'type': 'str', 'description': 'This specifies the notification email of the recipient. Only applicable for recipients of type email, which is not connected to a user.'}, 'language': {'type': 'str', 'description': 'Type specifies the preferred notification language for this recipient. Defaults to english.', 'enum': ['english', 'norwegian']}, 'name': {'type': 'str', 'description': 'This specifies the name of the recipient. Only applicable if the recipient is not referring to a user.'}, 'phone': {'type': 'str', 'description': 'This specifies the notification phone number of the recipient. Only applicable for recipients of type sms, which is not connected to a user.'}, 'role': {'type': 'str', 'description': 'This specifies the role this recipient has (why is the recipient being notified). Default is undefined.', 'enum': ['serviceContact', 'watcher', 'copy', 'currentUser']}, 'type': 'str', 'userID': {'type': 'int', 'description': 'If set, this ID referse to the user to notify', 'minimum': 0}, 'options': ['sms', 'email']}, 'required': ['format', 'language', 'type']}, 'maxItems': 2147483647, 'minItems': 1}}, 'required': ['event', 'recipients']}}}, 'required': True}, 'responses': {'default': {'description': 'Prepare a new notification', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'recipients': {'type': 'array', 'description': 'The recipients that this notification sends to', 'items': {'type': 'object', 'description': 'The recipients that this notification sends to', 'properties': {'destination': {'type': 'string', 'description': 'The destination of this recipient (email, sms or other)'}, 'flags': {'type': 'array', 'description': 'The flags of this recipient', 'items': {'type': 'string', 'description': 'The flags of this recipient', 'enum': ['verbose']}, 'uniqueItems': True}, 'name': {'type': 'string', 'description': 'Name of this recipient'}, 'type': {'type': 'string', 'description': 'The type of this recipient', 'enum': ['email', 'sms', 'app']}, 'userID': {'type': 'integer', 'format': 'int64', 'description': 'The ID of the user this notification is bound to, or 0 if not a user'}}}}, 'state': {'type': 'string', 'description': 'The state of this notification.', 'enum': ['prepared', 'sent']}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['prepareNotification'], 'x-Argus-ApiStatus': 'INTERNAL'}
send_notification = {'tags': ['notifications/v1/notification'], 'summary': 'Send a pending notification (INTERNAL)', 'description': '', 'operationId': 'sendNotification', 'parameters': [{'name': 'notificationID', 'in': 'path', 'description': 'Notification ID', 'required': True, 'schema': {'type': 'str'}, 'type': 'str'}], 'responses': {'default': {'description': 'Send a pending notification', 'content': {'application/json': {'schema': {'type': 'object', 'properties': {'responseCode': {'type': 'integer', 'format': 'int32', 'description': 'Status code returned from API'}, 'limit': {'type': 'integer', 'format': 'int64', 'description': 'Maximum number of returned results'}, 'offset': {'type': 'integer', 'format': 'int64', 'description': 'Number of skipped results'}, 'count': {'type': 'integer', 'format': 'int64', 'description': 'Number of available results on server'}, 'size': {'type': 'integer', 'format': 'int64', 'description': 'Actual number of returned results'}, 'metaData': {'type': 'object', 'description': 'Additional unstructured meta data associated with response'}, 'messages': {'type': 'array', 'description': 'Contains messages returned from the API, usually error messages', 'items': {'type': 'object', 'properties': {'message': {'type': 'string', 'description': 'Message text'}, 'messageTemplate': {'type': 'string', 'description': 'Message template'}, 'type': {'type': 'string', 'description': 'Message type', 'enum': ['FIELD_ERROR', 'ACTION_ERROR', 'WARNING', 'NOTIFICATION', 'INFO']}, 'field': {'type': 'string', 'description': 'Field which this message applies to'}, 'parameter': {'type': 'string', 'description': 'The affected parameter'}}}}, 'data': {'type': 'object', 'properties': {'id': {'type': 'string', 'format': 'uuid', 'description': 'ID of the response object.'}, 'recipients': {'type': 'array', 'description': 'The recipients that this notification sends to', 'items': {'type': 'object', 'description': 'The recipients that this notification sends to', 'properties': {'destination': {'type': 'string', 'description': 'The destination of this recipient (email, sms or other)'}, 'flags': {'type': 'array', 'description': 'The flags of this recipient', 'items': {'type': 'string', 'description': 'The flags of this recipient', 'enum': ['verbose']}, 'uniqueItems': True}, 'name': {'type': 'string', 'description': 'Name of this recipient'}, 'type': {'type': 'string', 'description': 'The type of this recipient', 'enum': ['email', 'sms', 'app']}, 'userID': {'type': 'integer', 'format': 'int64', 'description': 'The ID of the user this notification is bound to, or 0 if not a user'}}}}, 'state': {'type': 'string', 'description': 'The state of this notification.', 'enum': ['prepared', 'sent']}}}}}}}}, '401': {'description': 'Authentication failed', 'content': {'application/json': {}}}, '403': {'description': 'Access denied', 'content': {'application/json': {}}}, '404': {'description': 'Object not found', 'content': {'application/json': {}}}, '412': {'description': 'Validation error', 'content': {'application/json': {}}}}, 'x-Argus-Required-Permissions': ['prepareNotification'], 'x-Argus-ApiStatus': 'INTERNAL'}